@@ -8,10 +8,12 @@ import android.view.accessibility.AccessibilityNodeInfo;
 
 class QtNativeAccessibility
 {
+    static native boolean accessibilitySupported();
     static native void setActive(boolean enable);
     static native int[] childIdListForAccessibleObject(int objectId);
     static native int parentId(int objectId);
     static native String descriptionForAccessibleObject(int objectId);
+    static native String languageTag(int objectId);
     static native Rect screenRect(int objectId);
     static native int hitTest(float x, float y);
     static native boolean clickAction(int objectId);
