@@ -133,11 +133,6 @@ QLocale::Language QLocale::codeToLanguage(QStringView languageCode) noexcept
 
 #include "qoperatingsystemversion.h"
 
-QOperatingSystemVersion QOperatingSystemVersion::current()
-{
-    return QOperatingSystemVersionBase::current();
-}
-
 QString QOperatingSystemVersion::name() const
 {
     return QOperatingSystemVersionBase::name();
@@ -759,6 +754,7 @@ void QJniObject::callVoidMethodV(JNIEnv *env, jmethodID id, va_list args) const
 #endif // Q_OS_ANDROID
 
 #include "qlocale.h"
+#include "qlist.h"
 
 QStringList QLocale::uiLanguages() const
 {
@@ -1299,6 +1295,7 @@ void QBasicMutex::destroyInternal(QMutexPrivate *d)
 
 
 #include "qobject.h"
+#include "private/qobject_p.h"
 
 #ifdef Q_COMPILER_MANGLES_RETURN_TYPE
 QMetaObject *QObjectData::dynamicMetaObject() const
@@ -1483,3 +1480,63 @@ bool QObject::doSetProperty(const char *name, const QVariant *lvalue, QVariant *
 // order sections alphabetically to reduce chances of merge conflicts
 
 #endif // QT_CORE_REMOVED_SINCE(6, 10)
+
+#if QT_CORE_REMOVED_SINCE(6, 11)
+
+#include "qbytearray.h"
+
+QByteArray QByteArray::percentDecoded(char percent) const
+{
+    return fromPercentEncoding(*this, percent);
+}
+
+#if QT_CONFIG(thread)
+// some of the previously inlined API became removed
+#include "qreadwritelock.h"
+
+void QReadWriteLock::lockForRead()
+{
+    tryLockForRead(QDeadlineTimer(QDeadlineTimer::Forever));
+}
+
+bool QReadWriteLock::tryLockForRead(QDeadlineTimer timeout)
+{
+    return QBasicReadWriteLock::tryLockForRead(timeout);
+}
+
+void QReadWriteLock::lockForWrite()
+{
+    tryLockForWrite(QDeadlineTimer(QDeadlineTimer::Forever));
+}
+
+bool QReadWriteLock::tryLockForWrite(QDeadlineTimer timeout)
+{
+    return QBasicReadWriteLock::tryLockForWrite(timeout);
+}
+
+void QReadWriteLock::unlock()
+{
+    QBasicReadWriteLock::unlock();
+}
+#endif // QT_CONFIG(thread)
+
+#include "qsavefile.h"
+
+QSaveFile::QSaveFile(const QString &name)
+    : QSaveFile(name, nullptr)
+{
+}
+
+#include "qtimer.h"
+
+void QTimer::singleShotImpl(int msec, Qt::TimerType timerType,
+                            const QObject *receiver, QtPrivate::QSlotObjectBase *slotObj)
+{
+    singleShotImpl(std::chrono::milliseconds{msec}, timerType, receiver, slotObj);
+}
+
+// #include "qotherheader.h"
+// // implement removed functions from qotherheader.h
+// order sections alphabetically to reduce chances of merge conflicts
+
+#endif // QT_CORE_REMOVED_SINCE(6, 11)
