@@ -112,7 +112,8 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \fn QOperatingSystemVersion::current()
+    \fn QOperatingSystemVersion QOperatingSystemVersion::current()
+
     Returns a QOperatingSystemVersion indicating the current OS and its version number.
 
     \sa currentType()
@@ -485,6 +486,12 @@ const QOperatingSystemVersionBase QOperatingSystemVersion::Windows11_22H2;
     \since 6.8.1
  */
 
+/*!
+    \variable QOperatingSystemVersion::Windows11_25H2
+    \brief a version corresponding to Windows 11 Version 25H2 (version 10.0.26200).
+    \since 6.11
+ */
+
 /*!
     \variable QOperatingSystemVersion::OSXMavericks
     \brief a version corresponding to OS X Mavericks (version 10.9).
@@ -547,7 +554,7 @@ const QOperatingSystemVersion QOperatingSystemVersion::MacOSCatalina =
     \since 6.0
  */
 const QOperatingSystemVersion QOperatingSystemVersion::MacOSBigSur =
-    QOperatingSystemVersion(QOperatingSystemVersion::MacOS, 11, 0);
+    QOperatingSystemVersion(QOperatingSystemVersion::MacOS, 11);
 
 /*!
     \variable QOperatingSystemVersion::MacOSMonterey
@@ -555,7 +562,7 @@ const QOperatingSystemVersion QOperatingSystemVersion::MacOSBigSur =
     \since 6.3
  */
 const QOperatingSystemVersion QOperatingSystemVersion::MacOSMonterey =
-    QOperatingSystemVersion(QOperatingSystemVersion::MacOS, 12, 0);
+    QOperatingSystemVersion(QOperatingSystemVersion::MacOS, 12);
 
 /*!
     \variable QOperatingSystemVersion::MacOSVentura
@@ -576,6 +583,12 @@ const QOperatingSystemVersionBase QOperatingSystemVersion::MacOSVentura;
     \since 6.8
 */
 
+/*!
+    \variable QOperatingSystemVersion::MacOSTahoe
+    \brief a version corresponding to macOS Tahoe (version 26).
+    \since 6.10
+*/
+
 /*!
     \variable QOperatingSystemVersion::AndroidJellyBean
     \brief a version corresponding to Android Jelly Bean (version 4.1, API level 16).
