@@ -105,7 +105,6 @@ void QProcessEnvironmentPrivate::insert(const QProcessEnvironmentPrivate &other)
         vars.insert(it.key(), it.value());
 
 #ifdef Q_OS_UNIX
-    const OrderedNameMapMutexLocker locker(this, &other);
     auto nit = other.nameMap.constBegin();
     const auto nend = other.nameMap.constEnd();
     for ( ; nit != nend; ++nit)
@@ -206,8 +205,10 @@ bool comparesEqual(const QProcessEnvironment &lhs, const QProcessEnvironment &rh
 {
     if (lhs.d == rhs.d)
         return true;
-
-    return lhs.d && rhs.d && lhs.d->vars == rhs.d->vars;
+    if (!(lhs.d && rhs.d))
+        return false;
+    QProcessEnvironmentPrivate::OrderedMutexLocker locker(lhs.d, rhs.d);
+    return lhs.d->vars == rhs.d->vars;
 }
 
 /*!
@@ -265,6 +266,7 @@ bool QProcessEnvironment::contains(const QString &name) const
 {
     if (!d)
         return false;
+    QProcessEnvironmentPrivate::MutexLocker locker(d);
     return d->vars.contains(d->prepareName(name));
 }
 
@@ -315,6 +317,7 @@ QString QProcessEnvironment::value(const QString &name, const QString &defaultVa
     if (!d)
         return defaultValue;
 
+    QProcessEnvironmentPrivate::MutexLocker locker(d);
     const auto it = d->vars.constFind(d->prepareName(name));
     if (it == d->vars.constEnd())
         return defaultValue;
@@ -339,6 +342,7 @@ QStringList QProcessEnvironment::toStringList() const
 {
     if (!d)
         return QStringList();
+    QProcessEnvironmentPrivate::MutexLocker locker(d);
     return d->toList();
 }
 
@@ -355,6 +359,7 @@ QStringList QProcessEnvironment::keys() const
 {
     if (!d)
         return QStringList();
+    QProcessEnvironmentPrivate::MutexLocker locker(d);
     return d->keys();
 }
 
@@ -371,6 +376,7 @@ void QProcessEnvironment::insert(const QProcessEnvironment &e)
         return;
 
     // our re-impl of detach() detaches from null
+    QProcessEnvironmentPrivate::MutexLocker locker(e.d);
     d->insert(*e.d);
 }
 
@@ -2423,6 +2429,7 @@ void QProcessPrivate::start(QIODevice::OpenMode mode)
     errorString.clear();
     startProcess();
 }
+#endif // QT_CONFIG(process)
 
 /*!
     \since 5.15
@@ -2473,6 +2480,7 @@ QStringList QProcess::splitCommand(QStringView command)
     return args;
 }
 
+#if QT_CONFIG(process)
 /*!
     \since 5.0
 
@@ -2728,4 +2736,6 @@ QString QProcess::nullDevice()
 
 QT_END_NAMESPACE
 
+#if QT_CONFIG(process)
 #include "moc_qprocess.cpp"
+#endif
