@@ -12,12 +12,12 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-static inline QString textUriListLiteral() { return QStringLiteral("text/uri-list"); }
-static inline QString textHtmlLiteral() { return QStringLiteral("text/html"); }
-static inline QString textPlainLiteral() { return QStringLiteral("text/plain"); }
-static inline QString textPlainUtf8Literal() { return QStringLiteral("text/plain;charset=utf-8"); }
-static inline QString applicationXColorLiteral() { return QStringLiteral("application/x-color"); }
-static inline QString applicationXQtImageLiteral() { return QStringLiteral("application/x-qt-image"); }
+static inline QString textUriListLiteral() { return u"text/uri-list"_s; }
+static inline QString textHtmlLiteral() { return u"text/html"_s; }
+static inline QString textPlainLiteral() { return u"text/plain"_s; }
+static inline QString textPlainUtf8Literal() { return u"text/plain;charset=utf-8"_s; }
+static inline QString applicationXColorLiteral() { return u"application/x-color"_s; }
+static inline QString applicationXQtImageLiteral() { return u"application/x-qt-image"_s; }
 
 struct QMimeDataStruct
 {
@@ -76,6 +76,28 @@ QVariant QMimeDataPrivate::getData(const QString &format) const
         return it->data;
 }
 
+static QList<QVariant> dataToUrls(QByteArrayView text)
+{
+    QList<QVariant> list;
+    qsizetype newLineIndex = -1;
+    qsizetype from = 0;
+    const char *begin = text.data();
+    while ((newLineIndex = text.indexOf('\n', from)) != -1) {
+        const auto bav = QByteArrayView(begin + from, begin + newLineIndex).trimmed();
+        if (!bav.isEmpty())
+            list.push_back(QUrl::fromEncoded(bav));
+        from = newLineIndex + 1;
+        if (from >= text.size())
+            break;
+    }
+    if (from != text.size()) {
+        const auto bav = QByteArrayView(begin + from, text.end()).trimmed();
+        if (!bav.isEmpty())
+            list.push_back(QUrl::fromEncoded(bav));
+    }
+    return list;
+}
+
 QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType type) const
 {
     Q_Q(const QMimeData);
@@ -86,15 +108,15 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType ty
     // Text data requested: fallback to URL data if available
     if (format == "text/plain"_L1 && !data.isValid()) {
         data = retrieveTypedData(textUriListLiteral(), QMetaType(QMetaType::QVariantList));
-        if (data.metaType().id() == QMetaType::QUrl) {
-            data = QVariant(data.toUrl().toDisplayString());
-        } else if (data.metaType().id() == QMetaType::QVariantList) {
+        if (const QUrl *url = get_if<QUrl>(&std::as_const(data))) {
+            data = QVariant(url->toDisplayString());
+        } else if (const QVariantList *list = get_if<QVariantList>(&std::as_const(data))) {
             QString text;
             int numUrls = 0;
-            const QList<QVariant> list = data.toList();
-            for (int i = 0; i < list.size(); ++i) {
-                if (list.at(i).metaType().id() == QMetaType::QUrl) {
-                    text += list.at(i).toUrl().toDisplayString() + u'\n';
+            for (const auto &element : *list) {
+                if (const QUrl *url = get_if<QUrl>(&element)) {
+                    text += url->toDisplayString();
+                    text += u'\n';
                     ++numUrls;
                 }
             }
@@ -119,21 +141,20 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType ty
         || (typeId == QMetaType::QImage && data.metaType().id() == QMetaType::QPixmap))
         return data;
 
-    if (data.metaType().id() == QMetaType::QByteArray) {
+    if (const QByteArray *ba = get_if<QByteArray>(&std::as_const(data))) {
         // see if we can convert to the requested type
         switch (typeId) {
         case QMetaType::QString: {
-            const QByteArray ba = data.toByteArray();
-            if (ba.isNull())
+            if (ba->isNull())
                 return QVariant();
             if (format == "text/html"_L1) {
-                QStringDecoder decoder = QStringDecoder::decoderForHtml(ba);
+                QStringDecoder decoder = QStringDecoder::decoderForHtml(*ba);
                 if (decoder.isValid()) {
-                    return QString(decoder(ba));
+                    return QString(decoder(*ba));
                 }
                 // fall back to utf8
             }
-            return QString::fromUtf8(ba);
+            return QString::fromUtf8(*ba);
         }
         case QMetaType::QColor: {
             QVariant newData = data;
@@ -146,21 +167,13 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType ty
             Q_FALLTHROUGH();
         }
         case QMetaType::QUrl: {
-            QByteArray ba = data.toByteArray();
+            auto bav = data.view<QByteArrayView>();
             // Qt 3.x will send text/uri-list with a trailing
             // null-terminator (that is *not* sent for any other
             // text/* mime-type), so chop it off
-            if (ba.endsWith('\0'))
-                ba.chop(1);
-
-            QList<QByteArray> urls = ba.split('\n');
-            QList<QVariant> list;
-            for (int i = 0; i < urls.size(); ++i) {
-                QByteArray ba = urls.at(i).trimmed();
-                if (!ba.isEmpty())
-                    list.append(QUrl::fromEncoded(ba));
-            }
-            return list;
+            if (bav.endsWith('\0'))
+                bav.chop(1);
+            return dataToUrls(bav);
         }
         default:
             break;
@@ -180,10 +193,10 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType ty
         case QMetaType::QVariantList: {
             // has to be list of URLs
             QByteArray result;
-            QList<QVariant> list = data.toList();
-            for (int i = 0; i < list.size(); ++i) {
-                if (list.at(i).metaType().id() == QMetaType::QUrl) {
-                    result += list.at(i).toUrl().toEncoded();
+            const QList<QVariant> list = data.toList();
+            for (const auto &element : list) {
+                if (const QUrl *url = get_if<QUrl>(&element)) {
+                    result += url->toEncoded();
                     result += "\r\n";
                 }
             }
@@ -313,15 +326,14 @@ QMimeData::~QMimeData()
 QList<QUrl> QMimeData::urls() const
 {
     Q_D(const QMimeData);
-    QVariant data = d->retrieveTypedData(textUriListLiteral(), QMetaType(QMetaType::QVariantList));
+    const QVariant data = d->retrieveTypedData(textUriListLiteral(), QMetaType(QMetaType::QVariantList));
     QList<QUrl> urls;
-    if (data.metaType().id() == QMetaType::QUrl)
-        urls.append(data.toUrl());
-    else if (data.metaType().id() == QMetaType::QVariantList) {
-        QList<QVariant> list = data.toList();
-        for (int i = 0; i < list.size(); ++i) {
-            if (list.at(i).metaType().id() == QMetaType::QUrl)
-                urls.append(list.at(i).toUrl());
+    if (const QUrl *url = get_if<QUrl>(&data)) {
+        urls.append(*url);
+    } else if (const QVariantList *list = get_if<QVariantList>(&data)) {
+        for (const auto &element : *list) {
+            if (const QUrl *url = get_if<QUrl>(&element))
+                urls.append(*url);
         }
     }
     return urls;
@@ -341,13 +353,7 @@ QList<QUrl> QMimeData::urls() const
 void QMimeData::setUrls(const QList<QUrl> &urls)
 {
     Q_D(QMimeData);
-    QList<QVariant> list;
-    const int numUrls = urls.size();
-    list.reserve(numUrls);
-    for (int i = 0; i < numUrls; ++i)
-        list.append(urls.at(i));
-
-    d->setData(textUriListLiteral(), list);
+    d->setData(textUriListLiteral(), QList<QVariant>(urls.cbegin(), urls.cend()));
 }
 
 /*!
@@ -365,8 +371,9 @@ bool QMimeData::hasUrls() const
 
 
 /*!
-    Returns a plain text (MIME type \c text/plain) representation of
-    the data.
+    Returns the plain text (MIME type \c text/plain) representation of
+    the data if this object contains plain text. If it contains some other
+    content, this function makes a best effort to convert it to plain text.
 
     \sa hasText(), html(), data()
 */
@@ -401,7 +408,7 @@ void QMimeData::setText(const QString &text)
 */
 bool QMimeData::hasText() const
 {
-    return hasFormat(textPlainLiteral()) || hasUrls();
+    return hasFormat(textPlainLiteral()) || hasFormat(textPlainUtf8Literal()) || hasUrls();
 }
 
 /*!
@@ -532,7 +539,11 @@ bool QMimeData::hasColor() const
 
 /*!
     Returns the data stored in the object in the format described by
-    the MIME type specified by \a mimeType.
+    the MIME type specified by \a mimeType. If this object does not contain
+    data for the \a mimeType MIME type (see hasFormat()), this function may
+    perform a best effort conversion to it.
+
+    \sa hasFormat(), setData()
 */
 QByteArray QMimeData::data(const QString &mimeType) const
 {
@@ -560,17 +571,10 @@ void QMimeData::setData(const QString &mimeType, const QByteArray &data)
     Q_D(QMimeData);
 
     if (mimeType == "text/uri-list"_L1) {
-        QByteArray ba = data;
+        auto ba = QByteArrayView(data);
         if (ba.endsWith('\0'))
             ba.chop(1);
-        QList<QByteArray> urls = ba.split('\n');
-        QList<QVariant> list;
-        for (int i = 0; i < urls.size(); ++i) {
-            QByteArray ba = urls.at(i).trimmed();
-            if (!ba.isEmpty())
-                list.append(QUrl::fromEncoded(ba));
-        }
-        d->setData(mimeType, list);
+        d->setData(mimeType, dataToUrls(ba));
     } else {
         d->setData(mimeType, QVariant(data));
     }
@@ -588,6 +592,8 @@ void QMimeData::setData(const QString &mimeType, const QByteArray &data)
 */
 bool QMimeData::hasFormat(const QString &mimeType) const
 {
+    // formats() is virtual and could be reimplemented in sub-classes,
+    // so we have to use it here.
     return formats().contains(mimeType);
 }
 
