@@ -39,11 +39,6 @@ inline bool isUncontendedLocked(const QReadWriteLockPrivate *d)
 { return quintptr(d) & StateMask; }
 }
 
-static bool contendedTryLockForRead(QAtomicPointer<QReadWriteLockPrivate> &d_ptr,
-                                    QDeadlineTimer timeout, QReadWriteLockPrivate *d);
-static bool contendedTryLockForWrite(QAtomicPointer<QReadWriteLockPrivate> &d_ptr,
-                                     QDeadlineTimer timeout, QReadWriteLockPrivate *d);
-
 /*! \class QReadWriteLock
     \inmodule QtCore
     \brief The QReadWriteLock class provides read-write locking.
@@ -65,6 +60,7 @@ static bool contendedTryLockForWrite(QAtomicPointer<QReadWriteLockPrivate> &d_pt
 
     Example:
 
+    \snippet code/src_corelib_thread_qreadwritelock.cpp lock
     \snippet code/src_corelib_thread_qreadwritelock.cpp 0
 
     To ensure that writers aren't blocked forever by readers, readers
@@ -174,6 +170,7 @@ static Q_ALWAYS_INLINE bool fastTryLock(QAtomicPointer<QReadWriteLockPrivate> &d
 }
 
 /*!
+    \fn bool QReadWriteLock::tryLockForRead(QDeadlineTimer timeout)
     \overload
     \since 6.6
 
@@ -190,17 +187,11 @@ static Q_ALWAYS_INLINE bool fastTryLock(QAtomicPointer<QReadWriteLockPrivate> &d
 
     \sa unlock(), lockForRead()
 */
-bool QReadWriteLock::tryLockForRead(QDeadlineTimer timeout)
-{
-    QReadWriteLockPrivate *d = d_ptr.loadRelaxed();
-    if (fastTryLock(d_ptr, dummyLockedForRead, d))
-        return true;
-    return contendedTryLockForRead(d_ptr, timeout, d);
-}
 
-Q_NEVER_INLINE static bool contendedTryLockForRead(QAtomicPointer<QReadWriteLockPrivate> &d_ptr,
-                                                   QDeadlineTimer timeout, QReadWriteLockPrivate *d)
+Q_NEVER_INLINE bool
+QBasicReadWriteLock::contendedTryLockForRead(QDeadlineTimer timeout, void *dd)
 {
+    auto d = static_cast<QReadWriteLockPrivate *>(dd);
     while (true) {
         qYieldCpu();
         if (d == nullptr) {
@@ -211,8 +202,8 @@ Q_NEVER_INLINE static bool contendedTryLockForRead(QAtomicPointer<QReadWriteLock
 
         if ((quintptr(d) & StateMask) == StateLockedForRead) {
             // locked for read, increase the counter
-            const auto val = reinterpret_cast<QReadWriteLockPrivate *>(quintptr(d) + (1U<<4));
-            Q_ASSERT_X(quintptr(val) > (1U<<4), "QReadWriteLock::tryLockForRead()",
+            const auto val = reinterpret_cast<QReadWriteLockPrivate *>(quintptr(d) + Counter);
+            Q_ASSERT_X(quintptr(val) > Counter, "QReadWriteLock::tryLockForRead()",
                        "Overflow in lock counter");
             if (!d_ptr.testAndSetAcquire(d, val, d))
                 continue;
@@ -292,6 +283,7 @@ Q_NEVER_INLINE static bool contendedTryLockForRead(QAtomicPointer<QReadWriteLock
 */
 
 /*!
+    \fn bool QReadWriteLock::tryLockForWrite(QDeadlineTimer timeout)
     \overload
     \since 6.6
 
@@ -308,17 +300,11 @@ Q_NEVER_INLINE static bool contendedTryLockForRead(QAtomicPointer<QReadWriteLock
 
     \sa unlock(), lockForWrite()
 */
-bool QReadWriteLock::tryLockForWrite(QDeadlineTimer timeout)
-{
-    QReadWriteLockPrivate *d = d_ptr.loadRelaxed();
-    if (fastTryLock(d_ptr, dummyLockedForWrite, d))
-        return true;
-    return contendedTryLockForWrite(d_ptr, timeout, d);
-}
 
-Q_NEVER_INLINE static bool contendedTryLockForWrite(QAtomicPointer<QReadWriteLockPrivate> &d_ptr,
-                                                    QDeadlineTimer timeout, QReadWriteLockPrivate *d)
+Q_NEVER_INLINE bool
+QBasicReadWriteLock::contendedTryLockForWrite(QDeadlineTimer timeout, void *dd)
 {
+    auto d = static_cast<QReadWriteLockPrivate *>(dd);
     while (true) {
         qYieldCpu();
         if (d == nullptr) {
@@ -366,6 +352,32 @@ Q_NEVER_INLINE static bool contendedTryLockForWrite(QAtomicPointer<QReadWriteLoc
 }
 
 /*!
+    \internal
+    \since 6.11
+
+    Returns true if this QReadWriteLock is locked for reading. Can only be
+    called from unlock(), meaning the current thread must have it locked. This
+    is necessary for user code built with Thread Sanitizer, because it can't
+    tell from inline what type of lock a contended QReadWriteLock is locked
+    for.
+
+    This function is always present in QtCore, whether it was compiled for TSan
+    or not.
+
+    \sa unlock()
+*/
+bool QBasicReadWriteLock::isContendedLockForRead(const void *dd)
+{
+    auto d = static_cast<const QReadWriteLockPrivate *>(dd);
+    Q_ASSERT(d);
+    Q_ASSERT_X(!isUncontendedLocked(d), "QReadWriteLock::unlock", "Not in a contended or recursive lock");
+
+    // if there's a writer, then we're locked for writing
+    return d->writerCount == 0;
+}
+
+/*!
+    \fn void QReadWriteLock::unlock()
     Unlocks the lock.
 
     Attempting to unlock a lock that is not locked is an error, and will result
@@ -373,9 +385,10 @@ Q_NEVER_INLINE static bool contendedTryLockForWrite(QAtomicPointer<QReadWriteLoc
 
     \sa lockForRead(), lockForWrite(), tryLockForRead(), tryLockForWrite()
 */
-void QReadWriteLock::unlock()
+
+void QBasicReadWriteLock::contendedUnlock(void *dd)
 {
-    QReadWriteLockPrivate *d = d_ptr.loadAcquire();
+    auto d = static_cast<QReadWriteLockPrivate *>(dd);
     while (true) {
         Q_ASSERT_X(d, "QReadWriteLock::unlock()", "Cannot unlock an unlocked lock");
 
@@ -387,7 +400,7 @@ void QReadWriteLock::unlock()
         }
 
         if ((quintptr(d) & StateMask) == StateLockedForRead) {
-            Q_ASSERT(quintptr(d) > (1U<<4)); //otherwise that would be the fast case
+            Q_ASSERT(quintptr(d) > Counter); //otherwise that would be the fast case
             // Just decrease the reader's count.
             auto val = reinterpret_cast<QReadWriteLockPrivate *>(quintptr(d) - (1U<<4));
             if (!d_ptr.testAndSetOrdered(d, val, d))
@@ -610,10 +623,12 @@ void QReadWriteLockPrivate::release()
     Here's an example that uses QReadLocker to lock and unlock a
     read-write lock for reading:
 
+    \snippet code/src_corelib_thread_qreadwritelock.cpp lock
     \snippet code/src_corelib_thread_qreadwritelock.cpp 1
 
     It is equivalent to the following code:
 
+    \snippet code/src_corelib_thread_qreadwritelock.cpp lock
     \snippet code/src_corelib_thread_qreadwritelock.cpp 2
 
     The QMutexLocker documentation shows examples where the use of a
@@ -683,10 +698,12 @@ void QReadWriteLockPrivate::release()
     Here's an example that uses QWriteLocker to lock and unlock a
     read-write lock for writing:
 
+    \snippet code/src_corelib_thread_qreadwritelock.cpp lock
     \snippet code/src_corelib_thread_qreadwritelock.cpp 3
 
     It is equivalent to the following code:
 
+    \snippet code/src_corelib_thread_qreadwritelock.cpp lock
     \snippet code/src_corelib_thread_qreadwritelock.cpp 4
 
     The QMutexLocker documentation shows examples where the use of a
