@@ -17,6 +17,8 @@
 #include "qtexttable.h"
 #include "qtextengine_p.h"
 
+#include <QtCore/q20utility.h>
+
 #include <stdlib.h>
 
 QT_BEGIN_NAMESPACE
@@ -202,7 +204,7 @@ void QTextDocumentPrivate::clear()
 {
     Q_Q(QTextDocument);
 
-    for (QTextCursorPrivate *curs : qAsConst(cursors)) {
+    for (QTextCursorPrivate *curs : std::as_const(cursors)) {
         curs->setPosition(0);
         curs->currentCharFormat = -1;
         curs->anchor = 0;
@@ -255,7 +257,7 @@ void QTextDocumentPrivate::clear()
 
 QTextDocumentPrivate::~QTextDocumentPrivate()
 {
-    for (QTextCursorPrivate *curs : qAsConst(cursors))
+    for (QTextCursorPrivate *curs : std::as_const(cursors))
         curs->priv = nullptr;
     cursors.clear();
     undoState = 0;
@@ -333,7 +335,7 @@ int QTextDocumentPrivate::insert_block(int pos, uint strPos, int format, int blo
     int n = blocks.findNode(block_pos);
     int key = n ? blocks.position(n) : blocks.length();
 
-    Q_ASSERT(n || (!n && block_pos == blocks.length()));
+    Q_ASSERT(n || block_pos == blocks.length());
     if (key != block_pos) {
         Q_ASSERT(key < block_pos);
         int oldSize = blocks.size(n);
@@ -349,8 +351,10 @@ int QTextDocumentPrivate::insert_block(int pos, uint strPos, int format, int blo
     QTextBlockGroup *group = qobject_cast<QTextBlockGroup *>(objectForFormat(blockFormat));
     if (group) {
         group->blockInserted(QTextBlock(this, b));
-        docChangeOldLength--;
-        docChangeLength--;
+        if (command != QTextUndoCommand::BlockDeleted) {
+            docChangeOldLength--;
+            docChangeLength--;
+        }
     }
 
     QTextFrame *frame = qobject_cast<QTextFrame *>(objectForFormat(formats.format(format)));
@@ -373,7 +377,7 @@ int QTextDocumentPrivate::insertBlock(QChar blockSeparator,
 
     beginEditBlock();
 
-    int strPos = text.length();
+    int strPos = text.size();
     text.append(blockSeparator);
 
     int ob = blocks.findNode(pos);
@@ -443,29 +447,29 @@ void QTextDocumentPrivate::insert(int pos, int strPos, int strLength, int format
     finishEdit();
 }
 
-void QTextDocumentPrivate::insert(int pos, const QString &str, int format)
+void QTextDocumentPrivate::insert(int pos, QStringView str, int format)
 {
     if (str.size() == 0)
         return;
 
     Q_ASSERT(noBlockInString(str));
 
-    int strPos = text.length();
+    int strPos = text.size();
     text.append(str);
-    insert(pos, strPos, str.length(), format);
+    insert(pos, strPos, str.size(), format);
 }
 
 int QTextDocumentPrivate::remove_string(int pos, uint length, QTextUndoCommand::Operation op)
 {
     Q_ASSERT(pos >= 0);
     Q_ASSERT(blocks.length() == fragments.length());
-    Q_ASSERT(blocks.length() >= pos+(int)length);
+    Q_ASSERT(q20::cmp_greater_equal(blocks.length(), pos+length));
 
     int b = blocks.findNode(pos);
     uint x = fragments.findNode(pos);
 
     Q_ASSERT(blocks.size(b) > length);
-    Q_ASSERT(x && fragments.position(x) == (uint)pos && fragments.size(x) == length);
+    Q_ASSERT(x && q20::cmp_equal(fragments.position(x), pos) && fragments.size(x) == length);
     Q_ASSERT(noBlockInString(QStringView{text}.mid(fragments.fragment(x)->stringPosition, length)));
 
     blocks.setSize(b, blocks.size(b)-length);
@@ -646,7 +650,7 @@ void QTextDocumentPrivate::remove(int pos, int length, QTextUndoCommand::Operati
     blockCursorAdjustment = true;
     move(pos, -1, length, op);
     blockCursorAdjustment = false;
-    for (QTextCursorPrivate *curs : qAsConst(cursors)) {
+    for (QTextCursorPrivate *curs : std::as_const(cursors)) {
         if (curs->adjustPosition(pos, -length, op) == QTextCursorPrivate::CursorMoved) {
             curs->changed = true;
         }
@@ -723,7 +727,7 @@ void QTextDocumentPrivate::setCharFormat(int pos, int length, const QTextCharFor
         appendUndoItem(c);
 
         pos += length;
-        Q_ASSERT(pos == (int)(it.position() + fragment->size_array[0]) || pos >= endPos);
+        Q_ASSERT(q20::cmp_equal(pos, (it.position() + fragment->size_array[0])) || pos >= endPos);
     }
 
     int n = fragments.findNode(startPos - 1);
@@ -1002,6 +1006,7 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 }
 
 /*!
+    \internal
     Appends a custom undo \a item to the undo stack.
 */
 void QTextDocumentPrivate::appendUndoItem(QAbstractUndoItem *item)
@@ -1034,7 +1039,7 @@ void QTextDocumentPrivate::appendUndoItem(const QTextUndoCommand &c)
         clearUndoRedoStacks(QTextDocument::RedoStack);
 
     if (editBlock != 0 && editBlockCursorPosition >= 0) { // we had a beginEditBlock() with a cursor position
-        if (c.pos != (quint32) editBlockCursorPosition) { // and that cursor position is different from the command
+        if (q20::cmp_not_equal(c.pos, editBlockCursorPosition)) { // and that cursor position is different from the command
             // generate a CursorMoved undo item
             QT_INIT_TEXTUNDOCOMMAND(cc, QTextUndoCommand::CursorMoved, true, QTextUndoCommand::MoveCursor,
                                     0, 0, editBlockCursorPosition, 0, 0);
@@ -1204,13 +1209,13 @@ void QTextDocumentPrivate::finishEdit()
     }
 
     QList<QTextCursor> changedCursors;
-    for (QTextCursorPrivate *curs : qAsConst(cursors)) {
+    for (QTextCursorPrivate *curs : std::as_const(cursors)) {
         if (curs->changed) {
             curs->changed = false;
             changedCursors.append(QTextCursor(curs));
         }
     }
-    for (const QTextCursor &cursor : qAsConst(changedCursors))
+    for (const QTextCursor &cursor : std::as_const(changedCursors))
         emit q->cursorPositionChanged(cursor);
 
     contentsChanged();
@@ -1256,7 +1261,7 @@ void QTextDocumentPrivate::adjustDocumentChangesAndCursors(int from, int addedOr
     if (blockCursorAdjustment)  {
         ; // postpone, will be called again from QTextDocumentPrivate::remove()
     } else {
-        for (QTextCursorPrivate *curs : qAsConst(cursors)) {
+        for (QTextCursorPrivate *curs : std::as_const(cursors)) {
             if (curs->adjustPosition(from, addedOrRemoved, op) == QTextCursorPrivate::CursorMoved) {
                 curs->changed = true;
             }
@@ -1419,6 +1424,16 @@ QTextFrame *QTextDocumentPrivate::rootFrame() const
     return rtFrame;
 }
 
+void QTextDocumentPrivate::addCursor(QTextCursorPrivate *c)
+{
+    cursors.insert(c);
+}
+
+void QTextDocumentPrivate::removeCursor(QTextCursorPrivate *c)
+{
+    cursors.remove(c);
+}
+
 QTextFrame *QTextDocumentPrivate::frameAt(int pos) const
 {
     QTextFrame *f = rootFrame();
@@ -1433,7 +1448,7 @@ QTextFrame *QTextDocumentPrivate::frameAt(int pos) const
 
 void QTextDocumentPrivate::clearFrame(QTextFrame *f)
 {
-    for (int i = 0; i < f->d_func()->childFrames.count(); ++i)
+    for (int i = 0; i < f->d_func()->childFrames.size(); ++i)
         clearFrame(f->d_func()->childFrames.at(i));
     f->d_func()->childFrames.clear();
     f->d_func()->parentFrame = nullptr;
@@ -1703,7 +1718,7 @@ bool QTextDocumentPrivate::ensureMaximumBlockCount()
 void QTextDocumentPrivate::aboutToRemoveCell(int from, int to)
 {
     Q_ASSERT(from <= to);
-    for (QTextCursorPrivate *curs : qAsConst(cursors))
+    for (QTextCursorPrivate *curs : std::as_const(cursors))
         curs->aboutToRemoveCell(from, to);
 }
 
