@@ -2,6 +2,7 @@
 // Copyright (C) 2012 BogDan Vatra <bogdan@kde.org>
 // Copyright (C) 2016 Olivier Goffart <ogoffart@woboq.com>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <QtGui/qtguiglobal.h>
 
@@ -58,14 +59,14 @@ namespace QtAndroidInput
     void resetSoftwareKeyboard()
     {
         AndroidBackendRegister *reg = QtAndroid::backendRegister();
-        reg->callInterface<QtJniTypes::QtInputInterface, void>("resetSoftwareKeyboard");
+        reg->callInterface<QtJniTypes::QtInputInterface>("resetSoftwareKeyboard");
         qCDebug(lcQpaInputMethods) << "@@@ RESETSOFTWAREKEYBOARD";
     }
 
     void hideSoftwareKeyboard()
     {
         AndroidBackendRegister *reg = QtAndroid::backendRegister();
-        reg->callInterface<QtJniTypes::QtInputInterface, void>("hideSoftwareKeyboard");
+        reg->callInterface<QtJniTypes::QtInputInterface>("hideSoftwareKeyboard");
         qCDebug(lcQpaInputMethods) << "@@@ HIDESOFTWAREKEYBOARD";
     }
 
@@ -150,10 +151,9 @@ namespace QtAndroidInput
 
         static_assert (sizeof(eventButtons) <= sizeof(uint), "Qt::MouseButtons size changed. Adapt code.");
 
-        if (eventButtons == Qt::NoButton) {
-            QWindowSystemInterface::handleMouseEvent(topLevel, localPos, globalPos, qtButtons, Qt::NoButton, type);
+        if (eventButtons == Qt::NoButton)
             return;
-        }
+
         for (uint buttonInt = 0x1; static_cast<uint>(eventButtons) >= buttonInt; buttonInt <<= 1) {
             const auto button = static_cast<Qt::MouseButton>(buttonInt);
             if (eventButtons.testFlag(button)) {
@@ -202,7 +202,10 @@ namespace QtAndroidInput
             window = windowFromId(winId);
         const QPoint globalPos = window && window->handle() ?
                                     window->handle()->mapToGlobal(localPos) : localPos;
-        sendMouseButtonEvents(window, localPos, globalPos, mouseButtonState, QEvent::MouseMove);
+        const Qt::MouseButtons qtButtons = toMouseButtons(mouseButtonState);
+        m_lastSeenButtons = qtButtons;
+        QWindowSystemInterface::handleMouseEvent(window, localPos, globalPos,
+                                                 qtButtons, Qt::NoButton, QEvent::MouseMove);
     }
 
     static void mouseWheel(JNIEnv */*env*/, jobject /*thiz*/, jint winId, jint x, jint y, jfloat hdelta, jfloat vdelta)
