@@ -1,6 +1,7 @@
 // Copyright (C) 2014 BogDan Vatra <bogdan@kde.org>
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <dlfcn.h>
 #include <pthread.h>
@@ -9,7 +10,6 @@
 
 #include "androidcontentfileengine.h"
 #include "qandroidapkfileengine.h"
-#include "androiddeadlockprotector.h"
 #include "androidjniinput.h"
 #include "androidjnimain.h"
 #include "androidjnimenu.h"
@@ -24,6 +24,7 @@
 #if QT_CONFIG(accessibility)
 #include "androidjniaccessibility.h"
 #endif
+#include "qandroidplatformscreen.h"
 #include "qandroidplatformwindow.h"
 
 #include <android/api-level.h>
@@ -71,16 +72,10 @@ static jmethodID m_bitmapDrawableConstructorMethodID = nullptr;
 
 extern "C" typedef int (*Main)(int, char **); //use the standard main method to start the application
 static Main m_main = nullptr;
-static void *m_mainLibraryHnd = nullptr;
-static QList<QByteArray> m_applicationParams;
-static sem_t m_exitSemaphore, m_terminateSemaphore;
-
+static sem_t m_exitSemaphore, m_stopQtSemaphore;
 
 static QAndroidPlatformIntegration *m_androidPlatformIntegration = nullptr;
 
-static int m_availableWidthPixels  = 0;
-static int m_availableHeightPixels = 0;
-static double m_scaledDensity = 0;
 static double m_density = 1.0;
 
 static AndroidAssetsFileEngineHandler *m_androidAssetsFileEngineHandler = nullptr;
@@ -92,6 +87,7 @@ static AndroidBackendRegister *m_backendRegister = nullptr;
 static const char m_qtTag[] = "Qt";
 static const char m_classErrorMsg[] = "Can't find class \"%s\"";
 static const char m_methodErrorMsg[] = "Can't find method \"%s%s\"";
+static const char m_staticFieldErrorMsg[] = "Can't find static field \"%s\"";
 
 Q_CONSTINIT static QBasicAtomicInt startQtAndroidPluginCalled = Q_BASIC_ATOMIC_INITIALIZER(0);
 
@@ -99,6 +95,8 @@ Q_CONSTINIT static QBasicAtomicInt startQtAndroidPluginCalled = Q_BASIC_ATOMIC_I
 Q_DECLARE_JNI_CLASS(QtAccessibilityInterface, "org/qtproject/qt/android/QtAccessibilityInterface");
 #endif
 
+Q_DECLARE_JNI_CLASS(QtThread, "org/qtproject/qt/android/QtThread");
+
 namespace QtAndroid
 {
     QBasicMutex *platformInterfaceMutex()
@@ -150,21 +148,6 @@ namespace QtAndroid
         return nullptr;
     }
 
-    int availableWidthPixels()
-    {
-        return m_availableWidthPixels;
-    }
-
-    int availableHeightPixels()
-    {
-        return m_availableHeightPixels;
-    }
-
-    double scaledDensity()
-    {
-        return m_scaledDensity;
-    }
-
     double pixelDensity()
     {
         return m_density;
@@ -199,12 +182,6 @@ namespace QtAndroid
     }
 
 #if QT_CONFIG(accessibility)
-    void initializeAccessibility()
-    {
-        m_backendRegister->callInterface<QtJniTypes::QtAccessibilityInterface, void>(
-                "initializeAccessibility");
-    }
-
     void notifyAccessibilityLocationChange(uint accessibilityObjectId)
     {
         m_backendRegister->callInterface<QtJniTypes::QtAccessibilityInterface, void>(
@@ -342,6 +319,11 @@ namespace QtAndroid
         return m_methodErrorMsg;
     }
 
+    const char *staticFieldErrorMsgFmt()
+    {
+        return m_staticFieldErrorMsg;
+    }
+
     const char *qtTagText()
     {
         return m_qtTag;
@@ -384,9 +366,11 @@ namespace QtAndroid
 
 static bool initJavaReferences(QJniEnvironment &env);
 
-static jboolean startQtAndroidPlugin(JNIEnv *env, jobject /*object*/, jstring paramsString)
+static bool initAndroidQpaPlugin(JNIEnv *jenv, jobject object)
 {
-    Q_UNUSED(env)
+    Q_UNUSED(jenv)
+    Q_UNUSED(object)
+
     // Init all the Java refs, if they haven't already been initialized. They get initialized
     // when the library is loaded, but in case Qt is terminated, they are cleared, and in case
     // Qt is then started again JNI_OnLoad will not be called again, since the library is already
@@ -395,68 +379,35 @@ static jboolean startQtAndroidPlugin(JNIEnv *env, jobject /*object*/, jstring pa
     // want to reset those, too.
     QJniEnvironment qEnv;
     if (!qEnv.isValid()) {
-        __android_log_print(ANDROID_LOG_FATAL, "Qt", "Failed to initialize the JNI Environment");
-        return JNI_ERR;
+        qCritical() << "Failed to initialize the JNI Environment";
+        return false;
     }
+
     if (!initJavaReferences(qEnv))
         return false;
 
     m_androidPlatformIntegration = nullptr;
+
+    // File engine handler instantiation registers the handler
     m_androidAssetsFileEngineHandler = new AndroidAssetsFileEngineHandler();
     m_androidContentFileEngineHandler = new AndroidContentFileEngineHandler();
     m_androidApkFileEngineHandler = new QAndroidApkFileEngineHandler();
-    m_mainLibraryHnd = nullptr;
+
     m_backendRegister = new AndroidBackendRegister();
 
-    const QStringList argsList = QProcess::splitCommand(QJniObject(paramsString).toString());
-
-    for (const QString &arg : argsList)
-        m_applicationParams.append(arg.toUtf8());
-
-    // Go home
-    QDir::setCurrent(QDir::homePath());
-
-    //look for main()
-    if (m_applicationParams.length()) {
-        // Obtain a handle to the main library (the library that contains the main() function).
-        // This library should already be loaded, and calling dlopen() will just return a reference to it.
-        m_mainLibraryHnd = dlopen(m_applicationParams.constFirst().data(), 0);
-        if (Q_UNLIKELY(!m_mainLibraryHnd)) {
-            qCritical() << "dlopen failed:" << dlerror();
-            return false;
-        }
-        m_main = (Main)dlsym(m_mainLibraryHnd, "main");
-    } else {
-        qWarning("No main library was specified; searching entire process (this is slow!)");
-        m_main = (Main)dlsym(RTLD_DEFAULT, "main");
-    }
-
-    if (Q_UNLIKELY(!m_main)) {
-        qCritical() << "dlsym failed:" << dlerror() << Qt::endl
-                    << "Could not find main method";
+    if (sem_init(&m_exitSemaphore, 0, 0) == -1 && sem_init(&m_stopQtSemaphore, 0, 0) == -1) {
+        qCritical() << "Failed to init Qt application cleanup semaphores";
         return false;
     }
 
-    if (sem_init(&m_exitSemaphore, 0, 0) == -1)
-        return false;
-
-    if (sem_init(&m_terminateSemaphore, 0, 0) == -1)
-        return false;
-
     return true;
 }
 
-static void waitForServiceSetup(JNIEnv *env, jclass /*clazz*/)
+static void startQtNativeApplication(JNIEnv *jenv, jobject object, jstring paramsString)
 {
-    Q_UNUSED(env);
-    // The service must wait until the QCoreApplication starts otherwise onBind will be
-    // called too early
-    if (QtAndroidPrivate::service().isValid() && QtAndroid::isQtApplication())
-        QtAndroidPrivate::waitForServiceSetup();
-}
+    Q_UNUSED(jenv)
+    Q_UNUSED(object)
 
-static void startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
-{
     {
         JNIEnv* env = nullptr;
         JavaVMAttachArgs args;
@@ -468,6 +419,42 @@ static void startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
             vm->AttachCurrentThread(&env, &args);
     }
 
+    const QStringList argsList = QProcess::splitCommand(QJniObject(paramsString).toString());
+    const int argc = argsList.size();
+    QVarLengthArray<char *> argv(argc + 1);
+    QList<QByteArray> argvData;
+    argvData.reserve(argc);
+    for (int i = 0; i < argc; ++i) {
+        argvData.append(argsList.at(i).toUtf8());
+        argv[i] = argvData.back().data();
+    }
+    argv[argc] = nullptr;
+
+    // Go home
+    QDir::setCurrent(QDir::homePath());
+
+    // look for main()
+    void *mainLibraryHnd = nullptr;
+    if (argc) {
+        // Obtain a handle to the main library (the library that contains the main() function).
+        // This library should already be loaded, and calling dlopen() will just return a reference to it.
+        mainLibraryHnd = dlopen(argv.first(), 0);
+        if (Q_UNLIKELY(!mainLibraryHnd)) {
+            qCritical() << "dlopen failed:" << dlerror();
+            return;
+        }
+        m_main = (Main)dlsym(mainLibraryHnd, "main");
+    } else {
+        qWarning("No main library was specified; searching entire process (this is slow!)");
+        m_main = (Main)dlsym(RTLD_DEFAULT, "main");
+    }
+
+    if (Q_UNLIKELY(!m_main)) {
+        qCritical() << "dlsym failed:" << dlerror() << Qt::endl
+                    << "Could not find main method";
+        return;
+    }
+
     // Register type for invokeMethod() calls.
     qRegisterMetaType<Qt::ScreenOrientation>("Qt::ScreenOrientation");
 
@@ -475,28 +462,35 @@ static void startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
     if (QFile{QStringLiteral("assets:/android_rcc_bundle.rcc")}.exists())
         QResource::registerResource(QStringLiteral("assets:/android_rcc_bundle.rcc"));
 
-    const int argc = m_applicationParams.size();
-    QVarLengthArray<char *> argv(argc + 1);
-    for (int i = 0; i < argc; i++)
-        argv[i] = m_applicationParams[i].data();
-    argv[argc] = nullptr;
-
     startQtAndroidPluginCalled.fetchAndAddRelease(1);
+
     const int ret = m_main(argc, argv.data());
     qInfo() << "main() returned" << ret;
 
-    if (m_mainLibraryHnd) {
-        int res = dlclose(m_mainLibraryHnd);
+    if (mainLibraryHnd) {
+        int res = dlclose(mainLibraryHnd);
         if (res < 0)
             qWarning() << "dlclose failed:" << dlerror();
     }
 
-    if (m_applicationClass) {
-        const auto quitMethodName = QtAndroid::isQtApplication() ? "quitApp" : "quitQt";
-        QJniObject::callStaticMethod<void>(m_applicationClass, quitMethodName);
-    }
+    QNativeInterface::QAndroidApplication::runOnAndroidMainThread([]() {
+        QtNative::callStaticMethod("setStarted", false);
 
-    sem_post(&m_terminateSemaphore);
+        if (QtAndroid::isQtApplication()) {
+            // Now, that the Qt application has exited, tear down the Activity and Service
+            auto activity = QtAndroidPrivate::activity();
+            if (activity.isValid())
+                activity.callMethod("finish");
+            auto service = QtAndroidPrivate::service();
+            if (service.isValid())
+                service.callMethod("stopSelf");
+        } else {
+            // For the embedded case, we only need to terminate Qt
+            QtNative::callStaticMethod("terminateQtNativeApplication");
+        }
+    });
+
+    sem_post(&m_stopQtSemaphore);
     sem_wait(&m_exitSemaphore);
     sem_destroy(&m_exitSemaphore);
 
@@ -505,24 +499,6 @@ static void startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
         exit(ret);
 }
 
-static void quitQtCoreApplication(JNIEnv *env, jclass /*clazz*/)
-{
-    Q_UNUSED(env);
-    QCoreApplication::quit();
-}
-
-static void quitQtAndroidPlugin(JNIEnv *env, jclass /*clazz*/)
-{
-    Q_UNUSED(env);
-    m_androidPlatformIntegration = nullptr;
-    delete m_androidAssetsFileEngineHandler;
-    m_androidAssetsFileEngineHandler = nullptr;
-    delete m_androidContentFileEngineHandler;
-    m_androidContentFileEngineHandler = nullptr;
-    delete m_androidApkFileEngineHandler;
-    m_androidApkFileEngineHandler = nullptr;
-}
-
 static void clearJavaReferences(JNIEnv *env)
 {
     if (m_applicationClass) {
@@ -563,19 +539,30 @@ static void clearJavaReferences(JNIEnv *env)
     }
 }
 
-static void terminateQt(JNIEnv *env, jclass /*clazz*/)
+static void waitForServiceSetup(JNIEnv *env, jclass /*clazz*/)
 {
-    // QAndroidEventDispatcherStopper is stopped when the user uses the task manager to kill the application
-    if (QAndroidEventDispatcherStopper::instance()->stopped()) {
+    Q_UNUSED(env);
+    // The service must wait until the QCoreApplication starts otherwise onBind will be
+    // called too early
+    if (QtAndroidPrivate::service().isValid() && QtAndroid::isQtApplication())
+        QtAndroidPrivate::waitForServiceSetup();
+}
+
+static void terminateQtNativeApplication(JNIEnv *env, jclass /*clazz*/)
+{
+    // QAndroidEventDispatcherStopper is stopped when the user uses the task manager
+    // to kill the application. Also, in case of a service ensure to call quit().
+    if (QAndroidEventDispatcherStopper::instance()->stopped()
+        || QtAndroidPrivate::service().isValid()) {
         QAndroidEventDispatcherStopper::instance()->startAll();
         QCoreApplication::quit();
         QAndroidEventDispatcherStopper::instance()->goingToStop(false);
     }
 
     if (startQtAndroidPluginCalled.loadAcquire())
-        sem_wait(&m_terminateSemaphore);
+        sem_wait(&m_stopQtSemaphore);
 
-    sem_destroy(&m_terminateSemaphore);
+    sem_destroy(&m_stopQtSemaphore);
 
     clearJavaReferences(env);
 
@@ -589,41 +576,23 @@ static void terminateQt(JNIEnv *env, jclass /*clazz*/)
     delete m_backendRegister;
     m_backendRegister = nullptr;
     sem_post(&m_exitSemaphore);
+
+    // Terminate the QtThread
+    QtNative::callStaticMethod<QtThread>("getQtThread").callMethod("exit");
 }
 
-static void setDisplayMetrics(JNIEnv * /*env*/, jclass /*clazz*/, jint screenWidthPixels,
-                              jint screenHeightPixels, jint availableLeftPixels,
-                              jint availableTopPixels, jint availableWidthPixels,
-                              jint availableHeightPixels, jdouble xdpi, jdouble ydpi,
-                              jdouble scaledDensity, jdouble density, jfloat refreshRate)
+static void handleLayoutSizeChanged(JNIEnv * /*env*/, jclass /*clazz*/,
+                                    jint availableWidth, jint availableHeight)
 {
-    Q_UNUSED(availableLeftPixels)
-    Q_UNUSED(availableTopPixels)
-
-    m_availableWidthPixels = availableWidthPixels;
-    m_availableHeightPixels = availableHeightPixels;
-    m_scaledDensity = scaledDensity;
-    m_density = density;
-
-    const QSize screenSize(screenWidthPixels, screenHeightPixels);
-    // available geometry always starts from top left
-    const QRect availableGeometry(0, 0, availableWidthPixels, availableHeightPixels);
-    const QSize physicalSize(qRound(double(screenWidthPixels) / xdpi * 25.4),
-                             qRound(double(screenHeightPixels) / ydpi * 25.4));
-
     QMutexLocker lock(&m_platformMutex);
-    if (!m_androidPlatformIntegration) {
-        QAndroidPlatformIntegration::setDefaultDisplayMetrics(
-                availableGeometry.left(), availableGeometry.top(), availableGeometry.width(),
-                availableGeometry.height(), physicalSize.width(), physicalSize.height(),
-                screenSize.width(), screenSize.height());
-    } else {
-        m_androidPlatformIntegration->setScreenSizeParameters(physicalSize, screenSize,
-                                                              availableGeometry);
-        m_androidPlatformIntegration->setRefreshRate(refreshRate);
-    }
+    // available geometry always starts from top left
+    const QRect availableGeometry(0, 0, availableWidth, availableHeight);
+    if (m_androidPlatformIntegration)
+        m_androidPlatformIntegration->setAvailableGeometry(availableGeometry);
+    else if (QAndroidPlatformScreen::defaultAvailableGeometry().isNull())
+        QAndroidPlatformScreen::defaultAvailableGeometry() = availableGeometry;
 }
-Q_DECLARE_JNI_NATIVE_METHOD(setDisplayMetrics)
+Q_DECLARE_JNI_NATIVE_METHOD(handleLayoutSizeChanged)
 
 static void updateApplicationState(JNIEnv */*env*/, jobject /*thiz*/, jint state)
 {
@@ -744,6 +713,12 @@ static void handleUiDarkModeChanged(JNIEnv */*env*/, jobject /*thiz*/, jint newU
 }
 Q_DECLARE_JNI_NATIVE_METHOD(handleUiDarkModeChanged)
 
+static void handleScreenDensityChanged(JNIEnv */*env*/, jclass /*cls*/, jdouble density)
+{
+    m_density = density;
+}
+Q_DECLARE_JNI_NATIVE_METHOD(handleScreenDensityChanged)
+
 static void onActivityResult(JNIEnv */*env*/, jclass /*cls*/,
                              jint requestCode,
                              jint resultCode,
@@ -763,11 +738,9 @@ static jobject onBind(JNIEnv */*env*/, jclass /*cls*/, jobject intent)
 }
 
 static JNINativeMethod methods[] = {
-    { "startQtAndroidPlugin", "(Ljava/lang/String;)Z", (void *)startQtAndroidPlugin },
-    { "startQtApplication", "()V", (void *)startQtApplication },
-    { "quitQtAndroidPlugin", "()V", (void *)quitQtAndroidPlugin },
-    { "quitQtCoreApplication", "()V", (void *)quitQtCoreApplication },
-    { "terminateQt", "()V", (void *)terminateQt },
+    { "initAndroidQpaPlugin", "()Z", (void *)initAndroidQpaPlugin },
+    { "startQtNativeApplication", "(Ljava/lang/String;)V", (void *)startQtNativeApplication },
+    { "terminateQtNativeApplication", "()V", (void *)terminateQtNativeApplication },
     { "waitForServiceSetup", "()V", (void *)waitForServiceSetup },
     { "updateApplicationState", "(I)V", (void *)updateApplicationState },
     { "onActivityResult", "(IILandroid/content/Intent;)V", (void *)onActivityResult },
@@ -820,13 +793,14 @@ static bool registerNatives(QJniEnvironment &env)
     success &= env.registerNativeMethods(
             QtJniTypes::Traits<QtJniTypes::QtDisplayManager>::className(),
             {
-                    Q_JNI_NATIVE_METHOD(setDisplayMetrics),
+                    Q_JNI_NATIVE_METHOD(handleLayoutSizeChanged),
                     Q_JNI_NATIVE_METHOD(handleOrientationChanged),
                     Q_JNI_NATIVE_METHOD(handleRefreshRateChanged),
                     Q_JNI_NATIVE_METHOD(handleScreenAdded),
                     Q_JNI_NATIVE_METHOD(handleScreenChanged),
                     Q_JNI_NATIVE_METHOD(handleScreenRemoved),
-                    Q_JNI_NATIVE_METHOD(handleUiDarkModeChanged)
+                    Q_JNI_NATIVE_METHOD(handleUiDarkModeChanged),
+                    Q_JNI_NATIVE_METHOD(handleScreenDensityChanged)
             });
 
     success = success
@@ -915,7 +889,7 @@ static bool initJavaReferences(QJniEnvironment &env)
 
 QT_END_NAMESPACE
 
-Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM */*vm*/, void */*reserved*/)
+extern "C" Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM */*vm*/, void */*reserved*/)
 {
     static bool initialized = false;
     if (initialized)
@@ -938,6 +912,6 @@ Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM */*vm*/, void */*reserved*/)
         return JNI_ERR;
     }
 
-    __android_log_print(ANDROID_LOG_INFO, "Qt", "qt started");
+    __android_log_print(ANDROID_LOG_INFO, "Qt", "Qt platform plugin started");
     return JNI_VERSION_1_6;
 }
