@@ -2,10 +2,10 @@
 // Copyright (C) 2012 BogDan Vatra <bogdan@kde.org>
 // Copyright (C) 2016 Olivier Goffart <ogoffart@woboq.com>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <android/log.h>
 
-#include "androiddeadlockprotector.h"
 #include "androidjniinput.h"
 #include "androidjnimain.h"
 #include "qandroideventdispatcher.h"
@@ -27,6 +27,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 namespace {
 
 class BatchEditLock
@@ -69,15 +71,31 @@ static jfieldID m_textFieldID = 0;
 
 static void runOnQtThread(const std::function<void()> &func)
 {
-    AndroidDeadlockProtector protector;
+    QtAndroidPrivate::AndroidDeadlockProtector protector(
+        u"QAndroidInputContext::runOnQtThread()"_s);
     if (!protector.acquire())
         return;
     QMetaObject::invokeMethod(m_androidInputContext, "safeCall", Qt::BlockingQueuedConnection, Q_ARG(std::function<void()>, func));
 }
 
-static jboolean beginBatchEdit(JNIEnv */*env*/, jobject /*thiz*/)
+static bool hasValidFocusObject()
 {
     if (!m_androidInputContext)
+        return false;
+
+    const auto focusObject = m_androidInputContext->focusObject();
+    if (!focusObject)
+        return false;
+
+    if (!focusObject->property("inputMethodHints").isValid())
+        return false;
+
+    return true;
+}
+
+static jboolean beginBatchEdit(JNIEnv */*env*/, jobject /*thiz*/)
+{
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ BEGINBATCH";
@@ -88,7 +106,7 @@ static jboolean beginBatchEdit(JNIEnv */*env*/, jobject /*thiz*/)
 
 static jboolean endBatchEdit(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ ENDBATCH";
@@ -101,7 +119,7 @@ static jboolean endBatchEdit(JNIEnv */*env*/, jobject /*thiz*/)
 
 static jboolean commitText(JNIEnv *env, jobject /*thiz*/, jstring text, jint newCursorPosition)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     jboolean isCopy;
@@ -117,7 +135,7 @@ static jboolean commitText(JNIEnv *env, jobject /*thiz*/, jstring text, jint new
 
 static jboolean deleteSurroundingText(JNIEnv */*env*/, jobject /*thiz*/, jint leftLength, jint rightLength)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ DELETE" << leftLength << rightLength;
@@ -128,7 +146,7 @@ static jboolean deleteSurroundingText(JNIEnv */*env*/, jobject /*thiz*/, jint le
 
 static jboolean finishComposingText(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ FINISH";
@@ -139,7 +157,7 @@ static jboolean finishComposingText(JNIEnv */*env*/, jobject /*thiz*/)
 
 static jboolean replaceText(JNIEnv *env, jobject /*thiz*/, jint start, jint end, jstring text, jint newCursorPosition)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     jboolean isCopy;
@@ -156,7 +174,7 @@ static jboolean replaceText(JNIEnv *env, jobject /*thiz*/, jint start, jint end,
 
 static jint getCursorCapsMode(JNIEnv */*env*/, jobject /*thiz*/, jint reqModes)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return 0;
 
     jint res = 0;
@@ -166,7 +184,7 @@ static jint getCursorCapsMode(JNIEnv */*env*/, jobject /*thiz*/, jint reqModes)
 
 static jobject getExtractedText(JNIEnv *env, jobject /*thiz*/, int hintMaxChars, int hintMaxLines, jint flags)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return 0;
 
     QAndroidInputContext::ExtractedText extractedText;
@@ -190,7 +208,7 @@ static jobject getExtractedText(JNIEnv *env, jobject /*thiz*/, int hintMaxChars,
 
 static jstring getSelectedText(JNIEnv *env, jobject /*thiz*/, jint flags)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return 0;
 
     QString text;
@@ -203,7 +221,7 @@ static jstring getSelectedText(JNIEnv *env, jobject /*thiz*/, jint flags)
 
 static jstring getTextAfterCursor(JNIEnv *env, jobject /*thiz*/, jint length, jint flags)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return 0;
 
     QString text;
@@ -214,7 +232,7 @@ static jstring getTextAfterCursor(JNIEnv *env, jobject /*thiz*/, jint length, ji
 
 static jstring getTextBeforeCursor(JNIEnv *env, jobject /*thiz*/, jint length, jint flags)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return 0;
 
     QString text;
@@ -225,7 +243,7 @@ static jstring getTextBeforeCursor(JNIEnv *env, jobject /*thiz*/, jint length, j
 
 static jboolean setComposingText(JNIEnv *env, jobject /*thiz*/, jstring text, jint newCursorPosition)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     jboolean isCopy;
@@ -241,7 +259,7 @@ static jboolean setComposingText(JNIEnv *env, jobject /*thiz*/, jstring text, ji
 
 static jboolean setComposingRegion(JNIEnv */*env*/, jobject /*thiz*/, jint start, jint end)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ SETR" << start << end;
@@ -253,7 +271,7 @@ static jboolean setComposingRegion(JNIEnv */*env*/, jobject /*thiz*/, jint start
 
 static jboolean setSelection(JNIEnv */*env*/, jobject /*thiz*/, jint start, jint end)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ SETSEL" << start << end;
@@ -265,7 +283,7 @@ static jboolean setSelection(JNIEnv */*env*/, jobject /*thiz*/, jint start, jint
 
 static jboolean selectAll(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ SELALL";
@@ -276,7 +294,7 @@ static jboolean selectAll(JNIEnv */*env*/, jobject /*thiz*/)
 
 static jboolean cut(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@";
@@ -287,7 +305,7 @@ static jboolean cut(JNIEnv */*env*/, jobject /*thiz*/)
 
 static jboolean copy(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@";
@@ -298,7 +316,7 @@ static jboolean copy(JNIEnv */*env*/, jobject /*thiz*/)
 
 static jboolean copyURL(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@";
@@ -309,7 +327,7 @@ static jboolean copyURL(JNIEnv */*env*/, jobject /*thiz*/)
 
 static jboolean paste(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ PASTE";
@@ -320,7 +338,7 @@ static jboolean paste(JNIEnv */*env*/, jobject /*thiz*/)
 
 static jboolean updateCursorPosition(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return JNI_FALSE;
 
     qCDebug(lcQpaInputMethods) << "@@@ UPDATECURSORPOS";
@@ -331,7 +349,7 @@ static jboolean updateCursorPosition(JNIEnv */*env*/, jobject /*thiz*/)
 
 static void reportFullscreenMode(JNIEnv */*env*/, jobject /*thiz*/, jboolean enabled)
 {
-    if (!m_androidInputContext)
+    if (!hasValidFocusObject())
         return;
 
     runOnQtThread([&]{m_androidInputContext->reportFullscreenMode(enabled);});
@@ -339,7 +357,10 @@ static void reportFullscreenMode(JNIEnv */*env*/, jobject /*thiz*/, jboolean ena
 
 static jboolean fullscreenMode(JNIEnv */*env*/, jobject /*thiz*/)
 {
-    return m_androidInputContext ? m_androidInputContext->fullscreenMode() : false;
+    if (!hasValidFocusObject())
+        return false;
+
+    return m_androidInputContext->fullscreenMode();
 }
 
 static JNINativeMethod methods[] = {
@@ -978,6 +999,11 @@ void QAndroidInputContext::clear()
 }
 
 
+QObject *QAndroidInputContext::focusObject()
+{
+    return m_focusObject;
+}
+
 void QAndroidInputContext::setFocusObject(QObject *object)
 {
     if (object != m_focusObject) {
