@@ -1,5 +1,6 @@
 // Copyright (C) 2018 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QCOCOANSMENU_H
 #define QCOCOANSMENU_H
@@ -17,6 +18,8 @@
 
 #include <QtCore/private/qcore_mac_p.h>
 
+#import <AppKit/NSMenu.h>
+
 QT_FORWARD_DECLARE_CLASS(QCocoaMenu);
 QT_FORWARD_DECLARE_CLASS(QCocoaMenuItem);
 
