@@ -2,6 +2,7 @@
 // Copyright (C) 2017 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
 // Copyright (C) 2016 Pelagicore AG
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qeglfskmsscreen_p.h"
 #include "qeglfskmsdevice_p.h"
@@ -16,6 +17,8 @@
 
 QT_BEGIN_NAMESPACE
 
+QSet<QEglFSKmsScreen *> QEglFSKmsScreen::s_screens;
+
 class QEglFSKmsInterruptHandler : public QObject
 {
 public:
@@ -59,10 +62,14 @@ QEglFSKmsScreen::QEglFSKmsScreen(QEglFSKmsDevice *device, const QKmsOutput &outp
     } else {
         qCDebug(qLcEglfsKmsDebug) << "No EDID data for output" << name();
     }
+
+    s_screens.insert(this);
 }
 
 QEglFSKmsScreen::~QEglFSKmsScreen()
 {
+    s_screens.remove(this);
+
     m_output.cleanup(m_device);
     delete m_interruptHandler;
 }
@@ -166,6 +173,11 @@ void QEglFSKmsScreen::waitForFlip()
 {
 }
 
+void QEglFSKmsScreen::updateOutput(QKmsOutput output)
+{
+    m_output = output;
+}
+
 void QEglFSKmsScreen::restoreMode()
 {
     m_output.restoreMode(m_device);
@@ -180,6 +192,11 @@ qreal QEglFSKmsScreen::refreshRate() const
     return refresh > 0 ? refresh : 60;
 }
 
+void QEglFSKmsScreen::removeSibling(QPlatformScreen *screen)
+{
+    m_siblings.removeAll(screen);
+}
+
 QList<QPlatformScreen::Mode> QEglFSKmsScreen::modes() const
 {
     QList<QPlatformScreen::Mode> list;
@@ -227,4 +244,9 @@ void QEglFSKmsScreen::pageFlipped(unsigned int sequence, unsigned int tv_sec, un
     Q_UNUSED(tv_usec);
 }
 
+bool QEglFSKmsScreen::isScreenKnown(QEglFSKmsScreen *s)
+{
+    return s_screens.contains(s);
+}
+
 QT_END_NAMESPACE
