@@ -1,12 +1,13 @@
 // Copyright (C) 2018 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <qpa/qplatformintegrationplugin.h>
 #include "qwasmintegration.h"
 
 QT_BEGIN_NAMESPACE
 
-using namespace Qt::Literals::StringLiterals;
+using namespace Qt::StringLiterals;
 
 class QWasmIntegrationPlugin : public QPlatformIntegrationPlugin
 {
