@@ -1,13 +1,14 @@
 // Copyright (C) 2018 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qwasmscreen.h"
-#include "qwasmwindow.h"
-#include "qwasmeventtranslator.h"
+
 #include "qwasmcompositor.h"
-#include "qwasmintegration.h"
-#include "qwasmstring.h"
 #include "qwasmcssstyle.h"
+#include "qwasmintegration.h"
+#include "qwasmkeytranslator.h"
+#include "qwasmwindow.h"
 
 #include <emscripten/bind.h>
 #include <emscripten/val.h>
@@ -28,24 +29,30 @@ const char *QWasmScreen::m_canvasResizeObserverCallbackContextPropertyName =
 
 QWasmScreen::QWasmScreen(const emscripten::val &containerOrCanvas)
     : m_container(containerOrCanvas),
+      m_intermediateContainer(emscripten::val::undefined()),
       m_shadowContainer(emscripten::val::undefined()),
-      m_compositor(new QWasmCompositor(this)),
-      m_eventTranslator(new QWasmEventTranslator())
+      m_compositor(new QWasmCompositor(this))
 {
     auto document = m_container["ownerDocument"];
+
+    // Create an intermediate container which we can remove during cleanup in ~QWasmScreen().
+    // This is required due to the attachShadow() call below; there is no corresponding
+    // "detachShadow()" API to return the container to its previous state.
+    m_intermediateContainer = document.call<emscripten::val>("createElement", emscripten::val("div"));
+    m_intermediateContainer.set("id", std::string("qt-shadow-container"));
+    emscripten::val intermediateContainerStyle = m_intermediateContainer["style"];
+    intermediateContainerStyle.set("width", std::string("100%"));
+    intermediateContainerStyle.set("height", std::string("100%"));
+    m_container.call<void>("appendChild", m_intermediateContainer);
     // Each screen is represented by a div container. All of the windows exist therein as
-    // its children. Qt versions < 6.5 used to represent screens as canvas. Support that by
-    // transforming the canvas into a div.
-    if (m_container["tagName"].call<std::string>("toLowerCase") == "canvas") {
-        qWarning() << "Support for canvas elements as an element backing screen is deprecated. The "
-                      "canvas provided for the screen will be transformed into a div.";
-        auto container = document.call<emscripten::val>("createElement", emscripten::val("div"));
-        m_container["parentNode"].call<void>("replaceChild", container, m_container);
-        m_container = container;
-    }
+    // its children. Qt versions < 6.5 used to represent screens as canvas elements; this
+    // is no longer supported.
+    if (m_container["tagName"].call<std::string>("toLowerCase") == "canvas")
+        qFatal() << "Qt does not support using a canvas element as the container element. Use a div element instead";
+
     auto shadowOptions = emscripten::val::object();
     shadowOptions.set("mode", "open");
-    auto shadow = m_container.call<emscripten::val>("attachShadow", shadowOptions);
+    auto shadow = m_intermediateContainer.call<emscripten::val>("attachShadow", shadowOptions);
 
     m_shadowContainer = document.call<emscripten::val>("createElement", emscripten::val("div"));
 
@@ -65,24 +72,35 @@ QWasmScreen::QWasmScreen(const emscripten::val &containerOrCanvas)
     // Create "specialHTMLTargets" mapping for the canvas - the element  might be unreachable based
     // on its id only under some conditions, like the target being embedded in a shadow DOM or a
     // subframe.
-    emscripten::val::module_property("specialHTMLTargets")
-            .set(eventTargetId().toStdString(), m_shadowContainer);
-
     emscripten::val::module_property("specialHTMLTargets")
             .set(outerScreenId().toStdString(), m_container);
 
-    // Install event handlers on the container/canvas. This must be
-    // done after the canvas has been created above.
-    m_compositor->initEventHandlers();
-
-    updateQScreenAndCanvasRenderSize();
+    updateQScreenSize();
     m_shadowContainer.call<void>("focus");
+
+    m_touchDevice = std::make_unique<QPointingDevice>(
+            "touchscreen", 1, QInputDevice::DeviceType::TouchScreen,
+            QPointingDevice::PointerType::Finger,
+            QPointingDevice::Capability::Position | QPointingDevice::Capability::Area
+                    | QPointingDevice::Capability::NormalizedPosition,
+            10, 0);
+    m_tabletDevice = std::make_unique<QPointingDevice>(
+            "stylus", 2, QInputDevice::DeviceType::Stylus,
+            QPointingDevice::PointerType::Pen,
+            QPointingDevice::Capability::Position | QPointingDevice::Capability::Pressure
+                | QPointingDevice::Capability::NormalizedPosition
+                | QInputDevice::Capability::MouseEmulation
+                | QInputDevice::Capability::Hover | QInputDevice::Capability::Rotation
+                | QInputDevice::Capability::XTilt | QInputDevice::Capability::YTilt
+                | QInputDevice::Capability::TangentialPressure,
+            0, 0);
+
+    QWindowSystemInterface::registerInputDevice(m_touchDevice.get());
 }
 
 QWasmScreen::~QWasmScreen()
 {
-    emscripten::val::module_property("specialHTMLTargets")
-            .set(eventTargetId().toStdString(), emscripten::val::undefined());
+    m_intermediateContainer.call<void>("remove");
 
     m_shadowContainer.set(m_canvasResizeObserverCallbackContextPropertyName,
                           emscripten::val(intptr_t(0)));
@@ -90,7 +108,6 @@ QWasmScreen::~QWasmScreen()
 
 void QWasmScreen::deleteScreen()
 {
-    m_compositor->onScreenDeleting();
     // Deletes |this|!
     QWindowSystemInterface::handleScreenRemoved(this);
 }
@@ -112,22 +129,11 @@ QWasmCompositor *QWasmScreen::compositor()
     return m_compositor.get();
 }
 
-QWasmEventTranslator *QWasmScreen::eventTranslator()
-{
-    return m_eventTranslator.get();
-}
-
 emscripten::val QWasmScreen::element() const
 {
     return m_shadowContainer;
 }
 
-QString QWasmScreen::eventTargetId() const
-{
-    // Return a globally unique id for the canvas. We can choose any string,
-    // as long as it starts with a "!".
-    return QString("!qtcanvas_%1").arg(uintptr_t(this));
-}
 
 QString QWasmScreen::outerScreenId() const
 {
@@ -183,7 +189,7 @@ qreal QWasmScreen::devicePixelRatio() const
 
 QString QWasmScreen::name() const
 {
-    return QWasmString::toQString(m_shadowContainer["id"]);
+    return QString::fromEcmaString(m_shadowContainer["id"]);
 }
 
 QPlatformCursor *QWasmScreen::cursor() const
@@ -200,23 +206,30 @@ void QWasmScreen::resizeMaximizedWindows()
 
 QWindow *QWasmScreen::topWindow() const
 {
-    return m_compositor->keyWindow();
+    return activeChild() ? activeChild()->window() : nullptr;
 }
 
 QWindow *QWasmScreen::topLevelAt(const QPoint &p) const
 {
-    return m_compositor->windowAt(p);
+    const auto found =
+            std::find_if(childStack().begin(), childStack().end(), [&p](const QWasmWindow *window) {
+                const QRect geometry = window->windowFrameGeometry();
+
+                return window->isVisible() && geometry.contains(p);
+            });
+    return found != childStack().end() ? (*found)->window() : nullptr;
 }
 
-QPoint QWasmScreen::mapFromLocal(const QPoint &p) const
+QPointF QWasmScreen::mapFromLocal(const QPointF &p) const
 {
     return geometry().topLeft() + p;
 }
 
-QPoint QWasmScreen::clipPoint(const QPoint &p) const
+QPointF QWasmScreen::clipPoint(const QPointF &p) const
 {
-    return QPoint(qBound(screen()->geometry().left(), p.x(), screen()->geometry().right()),
-                  qBound(screen()->geometry().top(), p.y(), screen()->geometry().bottom()));
+    const auto geometryF = screen()->geometry().toRectF();
+    return QPointF(qBound(geometryF.left(), p.x(), geometryF.right()),
+                   qBound(geometryF.top(), p.y(), geometryF.bottom()));
 }
 
 void QWasmScreen::invalidateSize()
@@ -232,24 +245,29 @@ void QWasmScreen::setGeometry(const QRect &rect)
     resizeMaximizedWindows();
 }
 
-void QWasmScreen::updateQScreenAndCanvasRenderSize()
+void QWasmScreen::onSubtreeChanged(QWasmWindowTreeNodeChangeType changeType,
+                                   QWasmWindowTreeNode *parent, QWasmWindow *child)
+{
+    Q_UNUSED(parent);
+
+    QWindow *window = child->window();
+    const bool isMaxFull = (window->windowState() & Qt::WindowMaximized) ||
+                           (window->windowState() & Qt::WindowFullScreen);
+    if (changeType == QWasmWindowTreeNodeChangeType::NodeInsertion && parent == this
+        && childStack().size() == 1 && isMaxFull) {
+        window->setFlag(Qt::WindowStaysOnBottomHint);
+    }
+    QWasmWindowTreeNode::onSubtreeChanged(changeType, parent, child);
+    m_compositor->onWindowTreeChanged(changeType, child);
+}
+
+void QWasmScreen::updateQScreenSize()
 {
-    // The HTML canvas has two sizes: the CSS size and the canvas render size.
-    // The CSS size is determined according to standard CSS rules, while the
-    // render size is set using the "width" and "height" attributes. The render
-    // size must be set manually and is not auto-updated on CSS size change.
-    // Setting the render size to a value larger than the CSS size enables high-dpi
-    // rendering.
     double css_width;
     double css_height;
     emscripten_get_element_css_size(outerScreenId().toUtf8().constData(), &css_width, &css_height);
     QSizeF cssSize(css_width, css_height);
 
-    QSizeF canvasSize = cssSize * devicePixelRatio();
-
-    m_shadowContainer.set("width", canvasSize.width());
-    m_shadowContainer.set("height", canvasSize.height());
-
     // Returns the html elements document/body position
     auto getElementBodyPosition = [](const emscripten::val &element) -> QPoint {
         emscripten::val bodyRect =
@@ -260,7 +278,6 @@ void QWasmScreen::updateQScreenAndCanvasRenderSize()
     };
 
     setGeometry(QRect(getElementBodyPosition(m_shadowContainer), cssSize.toSize()));
-    m_compositor->requestUpdateAllWindows();
 }
 
 void QWasmScreen::canvasResizeObserverCallback(emscripten::val entries, emscripten::val)
@@ -276,10 +293,7 @@ void QWasmScreen::canvasResizeObserverCallback(emscripten::val entries, emscript
         return;
     }
 
-    // We could access contentBoxSize|contentRect|devicePixelContentBoxSize on the entry here, but
-    // these are not universally supported across all browsers. Get the sizes from the canvas
-    // instead.
-    screen->updateQScreenAndCanvasRenderSize();
+    screen->updateQScreenSize();
 }
 
 EMSCRIPTEN_BINDINGS(qtCanvasResizeObserverCallback)
@@ -306,4 +320,36 @@ void QWasmScreen::installCanvasResizeObserver()
     resizeObserver.call<void>("observe", m_shadowContainer);
 }
 
+emscripten::val QWasmScreen::containerElement()
+{
+    return m_shadowContainer;
+}
+
+QWasmWindowTreeNode<> *QWasmScreen::parentNode()
+{
+    return nullptr;
+}
+
+QList<QWasmWindow *> QWasmScreen::allWindows() const
+{
+    QList<QWasmWindow *> currentChildren;
+    QList<QWasmWindow *> result;
+
+    for (auto *w : childStack())
+        currentChildren << w;
+
+    while (!currentChildren.empty()) {
+        result << currentChildren;
+
+        QList<QWasmWindow *> toIterate;
+        currentChildren.swap(toIterate);
+
+        for (auto child : toIterate) {
+            for (auto *w : child->childStack())
+                currentChildren << w;
+        }
+    }
+    return result;
+}
+
 QT_END_NAMESPACE
