@@ -1,25 +1,26 @@
 // Copyright (C) 2018 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QWASMWINDOW_H
 #define QWASMWINDOW_H
 
 #include "qwasmintegration.h"
-#include <qpa/qplatformwindow.h>
-#include <qpa/qplatformwindow_p.h>
-#include <emscripten/html5.h>
 #include "qwasmbackingstore.h"
 #include "qwasmscreen.h"
 #include "qwasmcompositor.h"
 #include "qwasmwindownonclientarea.h"
 #include "qwasmwindowstack.h"
 #include "qwasmwindowtreenode.h"
+#include "qwasmevent.h"
 
 #include <QtCore/private/qstdweb_p.h>
-#include "QtGui/qopenglcontext.h"
-#include <QtOpenGL/qopengltextureblitter.h>
+#include <qpa/qwindowsysteminterface.h>
+#include <qpa/qplatformwindow.h>
+#include <qpa/qplatformwindow_p.h>
 
 #include <emscripten/val.h>
+#include <emscripten/html5.h>
 
 #include <memory>
 
@@ -29,26 +30,29 @@ namespace qstdweb {
 class EventCallback;
 }
 
-class ClientArea;
 struct KeyEvent;
 struct PointerEvent;
-class QWasmDeadKeySupport;
 struct WheelEvent;
 
 Q_DECLARE_LOGGING_CATEGORY(qLcQpaWasmInputContext)
 
 class QWasmWindow final : public QPlatformWindow,
-                          public QWasmWindowTreeNode,
+                          public QWasmWindowTreeNode<>,
                           public QNativeInterface::Private::QWasmWindow
 {
 public:
-    QWasmWindow(QWindow *w, QWasmDeadKeySupport *deadKeySupport, QWasmCompositor *compositor,
-                QWasmBackingStore *backingStore);
+    QWasmWindow(QWindow *w, QWasmCompositor *compositor,
+                QWasmBackingStore *backingStore, WId nativeHandle);
     ~QWasmWindow() final;
 
-    static QWasmWindow *fromWindow(QWindow *window);
+    static QWasmWindow *fromWindow(const QWindow *window);
+    QWasmWindow *transientParent() const;
+    Qt::WindowFlags windowFlags() const;
+    bool isModal() const;
     QSurfaceFormat format() const override;
 
+    void registerEventHandlers();
+
     void paint();
     void setZOrder(int order);
     void setWindowCursor(QByteArray cssCursorName);
@@ -85,35 +89,45 @@ public:
     bool windowEvent(QEvent *event) final;
     void setMask(const QRegion &region) final;
     void setParent(const QPlatformWindow *window) final;
+    void focus();
+    void onAccessibilityEnable();
 
     QWasmScreen *platformScreen() const;
     void setBackingStore(QWasmBackingStore *store) { m_backingStore = store; }
     QWasmBackingStore *backingStore() const { return m_backingStore; }
-    QWindow *window() const { return m_window; }
 
     std::string canvasSelector() const;
 
     emscripten::val context2d() const { return m_context2d; }
     emscripten::val a11yContainer() const { return m_a11yContainer; }
-    emscripten::val inputHandlerElement() const { return m_windowContents; }
+    emscripten::val inputHandlerElement() const { return m_window; }
+    emscripten::val inputElement() const { return m_inputElement; }
 
     // QNativeInterface::Private::QWasmWindow
     emscripten::val document() const override { return m_document; }
-    emscripten::val clientArea() const override { return m_qtWindow; }
+    emscripten::val clientArea() const override { return m_decoratedWindow; }
 
     // QWasmWindowTreeNode:
     emscripten::val containerElement() final;
     QWasmWindowTreeNode *parentNode() final;
 
+public slots:
+    void onTransientParentChanged(QWindow *newTransientParent);
+    void onModalityChanged();
+
 private:
     friend class QWasmScreen;
-    static constexpr auto minSizeForRegularWindows = 100;
+    static constexpr auto defaultWindowSize = 160;
+
+    QMetaObject::Connection m_transientWindowChangedConnection;
+    QMetaObject::Connection m_modalityChangedConnection;
 
     // QWasmWindowTreeNode:
     QWasmWindow *asWasmWindow() final;
     void onParentChanged(QWasmWindowTreeNode *previous, QWasmWindowTreeNode *current,
-                         QWasmWindowStack::PositionPreference positionPreference) final;
+                         QWasmWindowStack<>::PositionPreference positionPreference) final;
 
+    void shutdown();
     void invalidate();
     bool hasFrame() const;
     bool hasTitleBar() const;
@@ -123,41 +137,76 @@ private:
     void applyWindowState();
     void commitParent(QWasmWindowTreeNode *parent);
 
+    void handleKeyEvent(const KeyEvent &event);
     bool processKey(const KeyEvent &event);
+    void handleKeyForInputContextEvent(const KeyEvent &event);
     bool processKeyForInputContext(const KeyEvent &event);
-    bool processPointer(const PointerEvent &event);
+    void handleInputEvent(emscripten::val event);
+    void handleCompositionStartEvent(emscripten::val event);
+    void handleCompositionUpdateEvent(emscripten::val event);
+    void handleCompositionEndEvent(emscripten::val event);
+    void handleBeforeInputEvent(emscripten::val event);
+
+    void handlePointerEnterLeaveEvent(const PointerEvent &event);
+    bool processPointerEnterLeave(const PointerEvent &event);
+    void releasePointerGrab(const MouseEvent &event);
+    void processPointer(const PointerEvent &event);
+    bool deliverPointerEvent(const PointerEvent &event);
+    void handleWheelEvent(const emscripten::val &event);
     bool processWheel(const WheelEvent &event);
+    Qt::WindowFlags fixTopLevelWindowFlags(Qt::WindowFlags) const;
+    bool shouldBeAboveTransientParentFlags(Qt::WindowFlags flags) const;
+    QWasmWindowStack<>::PositionPreference positionPreferenceFromWindowFlags(Qt::WindowFlags) const;
 
-    QWindow *m_window = nullptr;
     QWasmCompositor *m_compositor = nullptr;
     QWasmBackingStore *m_backingStore = nullptr;
-    QWasmDeadKeySupport *m_deadKeySupport;
     QRect m_normalGeometry {0, 0, 0 ,0};
 
     emscripten::val m_document;
-    emscripten::val m_qtWindow;
-    emscripten::val m_windowContents;
-    emscripten::val m_canvasContainer;
+    emscripten::val m_decoratedWindow;
+    emscripten::val m_window;
     emscripten::val m_a11yContainer;
     emscripten::val m_canvas;
+    emscripten::val m_focusHelper;
+    emscripten::val m_inputElement;
+
     emscripten::val m_context2d = emscripten::val::undefined();
 
     std::unique_ptr<NonClientArea> m_nonClientArea;
-    std::unique_ptr<ClientArea> m_clientArea;
 
     QWasmWindowTreeNode *m_commitedParent = nullptr;
 
-    std::unique_ptr<qstdweb::EventCallback> m_keyDownCallback;
-    std::unique_ptr<qstdweb::EventCallback> m_keyUpCallback;
-    std::unique_ptr<qstdweb::EventCallback> m_keyDownCallbackForInputContext;
-    std::unique_ptr<qstdweb::EventCallback> m_keyUpCallbackForInputContext;
+    QWasmEventHandler m_keyDownCallback;
+    QWasmEventHandler m_keyUpCallback;
+    QWasmEventHandler m_keyDownCallbackForInputContext;
+    QWasmEventHandler m_keyUpCallbackForInputContext;
+    QWasmEventHandler m_inputCallback;
+    QWasmEventHandler m_compositionStartCallback;
+    QWasmEventHandler m_compositionUpdateCallback;
+    QWasmEventHandler m_compositionEndCallback;
+    QWasmEventHandler m_beforeInputCallback;
 
-    std::unique_ptr<qstdweb::EventCallback> m_pointerLeaveCallback;
-    std::unique_ptr<qstdweb::EventCallback> m_pointerEnterCallback;
+    QWasmEventHandler m_pointerDownCallback;
+    QWasmEventHandler m_pointerMoveCallback;
+    QWasmEventHandler m_pointerUpCallback;
+    QWasmEventHandler m_pointerCancelCallback;
+    QWasmEventHandler m_pointerLeaveCallback;
+    QWasmEventHandler m_pointerEnterCallback;
 
-    std::unique_ptr<qstdweb::EventCallback> m_dropCallback;
+    QWasmEventHandler m_dragOverCallback;
+    QWasmEventHandler m_dragStartCallback;
+    QWasmEventHandler m_dragEndCallback;
+    QWasmEventHandler m_dropCallback;
+    QWasmEventHandler m_dragEnterCallback;
+    QWasmEventHandler m_dragLeaveCallback;
 
-    std::unique_ptr<qstdweb::EventCallback> m_wheelEventCallback;
+    QWasmEventHandler m_wheelEventCallback;
+
+    QMap<int, QWindowSystemInterface::TouchPoint> m_pointerIdToTouchPoints;
+
+    QWasmEventHandler m_cutCallback;
+    QWasmEventHandler m_copyCallback;
+    QWasmEventHandler m_pasteCallback;
 
     Qt::WindowStates m_state = Qt::WindowNoState;
     Qt::WindowStates m_previousWindowState = Qt::WindowNoState;
@@ -166,6 +215,7 @@ private:
 
     QPoint m_lastPointerMovePoint;
 
+    std::optional<int> m_capturedPointerId = std::nullopt;
     WId m_winId = 0;
     bool m_wantCapture = false;
     bool m_hasTitle = false;
