@@ -2,12 +2,14 @@
 // Copyright (C) 2017 Eurogiciel, author: <philippe.coval@eurogiciel.fr>
 // Copyright (C) 2023 David Edmundson <davidedmundson@kde.org>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qwaylandxdgshell_p.h"
 
 #include "qwaylandxdgexporterv2_p.h"
 #include "qwaylandxdgdialogv1_p.h"
 #include "qwaylandxdgtopleveliconv1_p.h"
+#include "qwaylandsessionmanager_p.h"
 
 #include <QtWaylandClient/private/qwaylanddisplay_p.h>
 #include <QtWaylandClient/private/qwaylandwindow_p.h>
@@ -23,6 +25,16 @@ QT_BEGIN_NAMESPACE
 
 namespace QtWaylandClient {
 
+template <typename T, auto f>
+struct WithDestructor : public T
+{
+    using T::T;
+    ~WithDestructor()
+    {
+        f(this->object());
+    }
+};
+
 QWaylandXdgSurface::Toplevel::Toplevel(QWaylandXdgSurface *xdgSurface)
     : QtWayland::xdg_toplevel(xdgSurface->get_toplevel())
     , m_xdgSurface(xdgSurface)
@@ -45,6 +57,12 @@ QWaylandXdgSurface::Toplevel::Toplevel(QWaylandXdgSurface *xdgSurface)
         m_xdgDialog.reset(m_xdgSurface->m_shell->m_xdgDialogWm->getDialog(object()));
         m_xdgDialog->set_modal();
     }
+
+#ifndef QT_NO_SESSIONMANAGER
+    const QString sessionRestoreId = xdgSurface->window()->sessionRestoreId();
+    if (!sessionRestoreId.isEmpty() && QWaylandSessionManager::instance()->session())
+        m_session.reset(new WithDestructor<QtWayland::xx_toplevel_session_v1, xx_toplevel_session_v1_destroy>(QWaylandSessionManager::instance()->session()->restore_toplevel(object(), sessionRestoreId)));
+#endif
 }
 
 QWaylandXdgSurface::Toplevel::~Toplevel()
@@ -59,9 +77,6 @@ QWaylandXdgSurface::Toplevel::~Toplevel()
 
 void QWaylandXdgSurface::Toplevel::applyConfigure()
 {
-    if (!(m_applied.states & (Qt::WindowMaximized|Qt::WindowFullScreen)))
-        m_normalSize = m_xdgSurface->m_window->windowContentGeometry().size();
-
     if ((m_pending.states & Qt::WindowActive) && !(m_applied.states & Qt::WindowActive)
         && !m_xdgSurface->m_window->display()->isKeyboardAvailable())
         m_xdgSurface->m_window->display()->handleWindowActivated(m_xdgSurface->m_window);
@@ -73,6 +88,9 @@ void QWaylandXdgSurface::Toplevel::applyConfigure()
     m_xdgSurface->m_window->handleToplevelWindowTilingStatesChanged(m_toplevelStates);
     m_xdgSurface->m_window->handleWindowStatesChanged(m_pending.states);
 
+    if (!(m_applied.states & (Qt::WindowMaximized|Qt::WindowFullScreen)))
+        m_normalSize = m_xdgSurface->m_window->windowContentGeometry().size();
+
     // If the width or height is zero, the client should decide the size on its own.
     QSize surfaceSize;
 
@@ -179,14 +197,19 @@ void QWaylandXdgSurface::Toplevel::xdg_toplevel_close()
 
 void QWaylandXdgSurface::Toplevel::requestWindowFlags(Qt::WindowFlags flags)
 {
-    if (m_decoration) {
-        if (flags & Qt::FramelessWindowHint) {
-            delete m_decoration;
-            m_decoration = nullptr;
-        } else {
-            m_decoration->unsetMode();
-        }
+    if (flags & Qt::FramelessWindowHint) {
+        delete m_decoration;
+        m_decoration = nullptr;
+    } else {
+        auto decorationManager = m_xdgSurface->m_shell->decorationManager();
+        // with decorationManagerV1 we can't turn decorations back on again
+        if (!m_decoration && decorationManager && decorationManager->version() >= 2)
+            m_decoration = m_xdgSurface->m_shell->decorationManager()->createToplevelDecoration(object());
+
+        if (m_decoration)
+            m_decoration->requestMode(QWaylandXdgToplevelDecorationV1::mode_server_side);
     }
+
 }
 
 void QWaylandXdgSurface::Toplevel::requestWindowStates(Qt::WindowStates states)
@@ -569,6 +592,7 @@ bool QWaylandXdgSurface::requestActivate()
 
             const auto tokenProvider = activation->requestXdgActivationToken(
                     wlWindow->display(), wlWindow->wlSurface(), serial, appId);
+            tokenProvider->setParent(this);
             connect(tokenProvider, &QWaylandXdgActivationTokenV1::done, this,
                     [this](const QString &token) {
                         m_shell->activation()->activate(token, window()->wlSurface());
@@ -679,108 +703,138 @@ void QWaylandXdgSurface::setWindowPosition(const QPoint &position)
     window()->updateExposure();
 }
 
+static QtWayland::xdg_positioner::gravity gravityFromEdge(Qt::Edges edges)
+{
+    switch (edges) {
+    case Qt::Edges():
+        return QtWayland::xdg_positioner::gravity_none;
+    case Qt::TopEdge:
+        return QtWayland::xdg_positioner::gravity_top;
+    case Qt::TopEdge | Qt::RightEdge:
+        return QtWayland::xdg_positioner::gravity_top_right;
+    case Qt::RightEdge:
+        return QtWayland::xdg_positioner::gravity_right;
+    case Qt::BottomEdge | Qt::RightEdge:
+        return QtWayland::xdg_positioner::gravity_bottom_right;
+    case Qt::BottomEdge:
+        return QtWayland::xdg_positioner::gravity_bottom;
+    case Qt::BottomEdge | Qt::LeftEdge:
+        return QtWayland::xdg_positioner::gravity_bottom_left;
+    case Qt::LeftEdge:
+        return QtWayland::xdg_positioner::gravity_left;
+    case Qt::TopEdge | Qt::LeftEdge:
+        return QtWayland::xdg_positioner::gravity_top_left;
+    }
+    qCWarning(lcQpaWayland) << "Cannot map positioner gravity " << edges;
+    return QtWayland::xdg_positioner::gravity_none;
+}
+
+static QtWayland::xdg_positioner::anchor anchorFromEdge(Qt::Edges edges)
+{
+    switch (edges) {
+    case Qt::Edges():
+        return QtWayland::xdg_positioner::anchor_none;
+    case Qt::TopEdge:
+        return QtWayland::xdg_positioner::anchor_top;
+    case Qt::TopEdge | Qt::RightEdge:
+        return QtWayland::xdg_positioner::anchor_top_right;
+    case Qt::RightEdge:
+        return QtWayland::xdg_positioner::anchor_right;
+    case Qt::BottomEdge | Qt::RightEdge:
+        return QtWayland::xdg_positioner::anchor_bottom_right;
+    case Qt::BottomEdge:
+        return QtWayland::xdg_positioner::anchor_bottom;
+    case Qt::BottomEdge | Qt::LeftEdge:
+        return QtWayland::xdg_positioner::anchor_bottom_left;
+    case Qt::LeftEdge:
+        return QtWayland::xdg_positioner::anchor_left;
+    case Qt::TopEdge | Qt::LeftEdge:
+        return QtWayland::xdg_positioner::anchor_top_left;
+    }
+    qCWarning(lcQpaWayland) << "Cannot map positioner anchor" << edges;
+    return QtWayland::xdg_positioner::anchor_none;
+}
+
 std::unique_ptr<QWaylandXdgSurface::Positioner> QWaylandXdgSurface::createPositioner(QWaylandWindow *parent)
 {
     std::unique_ptr<Positioner> positioner(new Positioner(m_shell));
+
+    // Default case, map the guessed global position to a relative position
+    QRect placementAnchor = QRect(m_window->geometry().topLeft() - parent->geometry().topLeft(), QSize(1,1));
+    Qt::Edges anchor = Qt::TopEdge | Qt::RightEdge;
+    Qt::Edges gravity = Qt::BottomEdge | Qt::RightEdge;
+    uint32_t constraintAdjustment = QtWayland::xdg_positioner::constraint_adjustment_slide_x | QtWayland::xdg_positioner::constraint_adjustment_slide_y;
+
+    // Compensate the margins to appear exactly at the position provided by QPlatformWindow::geometry
+    // These marnings have nothing to do with parent controls, don't apply them when positioning via parent control geometry
+    QMargins windowMargins = m_window->windowContentMargins() - m_window->clientSideMargins();
+    placementAnchor.translate(windowMargins.left(), windowMargins.top());
+
+    // Override from window type
+    if (m_window->parentControlGeometry().isValid())
+        placementAnchor = m_window->parentControlGeometry();
+
+    switch (m_window->extendedWindowType()) {
+    case QNativeInterface::Private::QWaylandWindow::Menu:
+    case QNativeInterface::Private::QWaylandWindow::WindowType::ComboBox:
+        anchor = Qt::BottomEdge | Qt::LeftEdge;
+        gravity = Qt::BottomEdge | Qt::RightEdge;
+        constraintAdjustment = QtWayland::xdg_positioner::constraint_adjustment_slide_x |
+                QtWayland::xdg_positioner::constraint_adjustment_flip_y | QtWayland::xdg_positioner::constraint_adjustment_slide_y;
+        break;
+    case QNativeInterface::Private::QWaylandWindow::SubMenu:
+        anchor = Qt::TopEdge | Qt::RightEdge;
+        gravity = Qt::BottomEdge | Qt::RightEdge;
+        constraintAdjustment = QtWayland::xdg_positioner::constraint_adjustment_flip_x |
+                QtWayland::xdg_positioner::constraint_adjustment_slide_y;
+        break;
+    case QNativeInterface::Private::QWaylandWindow::ToolTip:
+        anchor = Qt::BottomEdge | Qt::RightEdge;
+        gravity = Qt::BottomEdge | Qt::RightEdge;
+        constraintAdjustment = QtWayland::xdg_positioner::constraint_adjustment_flip_x | QtWayland::xdg_positioner::constraint_adjustment_slide_x |
+                QtWayland::xdg_positioner::constraint_adjustment_flip_y | QtWayland::xdg_positioner::constraint_adjustment_slide_y;
+        break;
+    default:
+        break;
+    }
+
+    if (qApp->layoutDirection() == Qt::RightToLeft) {
+        if (anchor & (Qt::RightEdge | Qt::LeftEdge))
+            anchor ^= (Qt::RightEdge | Qt::LeftEdge);
+        if (gravity & (Qt::RightEdge | Qt::LeftEdge))
+            gravity ^= (Qt::RightEdge | Qt::LeftEdge);
+    }
+
+    // Override with properties fauxAPI
+    const QVariant placementAnchorVariant = m_window->window()->property("_q_waylandPopupAnchorRect");
+    if (placementAnchorVariant.isValid())
+        placementAnchor = placementAnchorVariant.toRect();
+    const QVariant anchorVariant = m_window->window()->property("_q_waylandPopupAnchor");
+    if (anchorVariant.isValid())
+        anchor = anchorVariant.value<Qt::Edges>();
+    const QVariant popupGravityVariant = m_window->window()->property("_q_waylandPopupGravity");
+    if (popupGravityVariant.isValid())
+        gravity = popupGravityVariant.value<Qt::Edges>();
+    const QVariant constraintAdjustmentVariant = m_window->window()->property("_q_waylandPopupConstraintAdjustment");
+    if (constraintAdjustmentVariant.isValid())
+        constraintAdjustment = constraintAdjustmentVariant.toUInt();
+
     // set_popup expects a position relative to the parent
     QRect windowGeometry = m_window->windowContentGeometry();
-    QMargins windowMargins = m_window->windowContentMargins() - m_window->clientSideMargins();
     QMargins parentMargins = parent->windowContentMargins() - parent->clientSideMargins();
-
-    // These property overrides may be removed when public API becomes available
-    QRect placementAnchor = m_window->window()->property("_q_waylandPopupAnchorRect").toRect();
-    if (!placementAnchor.isValid()) {
-        placementAnchor = QRect(m_window->geometry().topLeft() - parent->geometry().topLeft(), QSize(1,1));
-    }
-    placementAnchor.translate(windowMargins.left(), windowMargins.top());
     placementAnchor.translate(-parentMargins.left(), -parentMargins.top());
 
-    uint32_t anchor = QtWayland::xdg_positioner::anchor_top_left;
-    const QVariant anchorVariant = m_window->window()->property("_q_waylandPopupAnchor");
-    if (anchorVariant.isValid()) {
-        switch (anchorVariant.value<Qt::Edges>()) {
-        case Qt::Edges():
-            anchor = QtWayland::xdg_positioner::anchor_none;
-            break;
-        case Qt::TopEdge:
-            anchor = QtWayland::xdg_positioner::anchor_top;
-            break;
-        case Qt::TopEdge | Qt::RightEdge:
-            anchor = QtWayland::xdg_positioner::anchor_top_right;
-            break;
-        case Qt::RightEdge:
-            anchor = QtWayland::xdg_positioner::anchor_right;
-            break;
-        case Qt::BottomEdge | Qt::RightEdge:
-            anchor = QtWayland::xdg_positioner::anchor_bottom_right;
-            break;
-        case Qt::BottomEdge:
-            anchor = QtWayland::xdg_positioner::anchor_bottom;
-            break;
-        case Qt::BottomEdge | Qt::LeftEdge:
-            anchor = QtWayland::xdg_positioner::anchor_bottom_left;
-            break;
-        case Qt::LeftEdge:
-            anchor = QtWayland::xdg_positioner::anchor_left;
-            break;
-        case Qt::TopEdge | Qt::LeftEdge:
-            anchor = QtWayland::xdg_positioner::anchor_top_left;
-            break;
-        }
-    }
-
-    uint32_t gravity = QtWayland::xdg_positioner::gravity_bottom_right;
-    const QVariant popupGravityVariant = m_window->window()->property("_q_waylandPopupGravity");
-    if (popupGravityVariant.isValid()) {
-        switch (popupGravityVariant.value<Qt::Edges>()) {
-        case Qt::Edges():
-            gravity = QtWayland::xdg_positioner::gravity_none;
-            break;
-        case Qt::TopEdge:
-            gravity = QtWayland::xdg_positioner::gravity_top;
-            break;
-        case Qt::TopEdge | Qt::RightEdge:
-            gravity = QtWayland::xdg_positioner::gravity_top_right;
-            break;
-        case Qt::RightEdge:
-            gravity = QtWayland::xdg_positioner::gravity_right;
-            break;
-        case Qt::BottomEdge | Qt::RightEdge:
-            gravity = QtWayland::xdg_positioner::gravity_bottom_right;
-            break;
-        case Qt::BottomEdge:
-            gravity = QtWayland::xdg_positioner::gravity_bottom;
-            break;
-        case Qt::BottomEdge | Qt::LeftEdge:
-            gravity = QtWayland::xdg_positioner::gravity_bottom_left;
-            break;
-        case Qt::LeftEdge:
-            gravity = QtWayland::xdg_positioner::gravity_left;
-            break;
-        case Qt::TopEdge | Qt::LeftEdge:
-            gravity = QtWayland::xdg_positioner::gravity_top_left;
-            break;
-        }
-    }
-
-    uint32_t constraintAdjustment = QtWayland::xdg_positioner::constraint_adjustment_slide_x | QtWayland::xdg_positioner::constraint_adjustment_slide_y;
-    const QVariant constraintAdjustmentVariant = m_window->window()->property("_q_waylandPopupConstraintAdjustment");
-    if (constraintAdjustmentVariant.isValid()) {
-        constraintAdjustment = constraintAdjustmentVariant.toUInt();
-    }
-
     positioner->set_anchor_rect(placementAnchor.x(),
                                 placementAnchor.y(),
                                 placementAnchor.width(),
                                 placementAnchor.height());
-    positioner->set_anchor(anchor);
-    positioner->set_gravity(gravity);
+    positioner->set_anchor(anchorFromEdge(anchor));
+    positioner->set_gravity(gravityFromEdge(gravity));
     positioner->set_size(windowGeometry.width(), windowGeometry.height());
     positioner->set_constraint_adjustment(constraintAdjustment);
     return positioner;
 }
 
-
 void QWaylandXdgSurface::setIcon(const QIcon &icon)
 {
     if (!m_shell->m_topLevelIconManager || !m_toplevel)
