@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qwaylandintegration_p.h"
 
@@ -18,7 +19,6 @@
 #include "qwaylandplatformservices_p.h"
 #include "qwaylandscreen_p.h"
 #include "qwaylandcursor_p.h"
-#include "qwaylandeventdispatcher_p.h"
 
 #if defined(Q_OS_MACOS)
 #  include <QtGui/private/qcoretextfontdatabase_p.h>
@@ -26,6 +26,7 @@
 #else
 #  include <QtGui/private/qgenericunixfontdatabase_p.h>
 #endif
+#include <QtGui/private/qgenericunixeventdispatcher_p.h>
 #include <QtGui/private/qgenericunixtheme_p.h>
 
 #include <QtGui/private/qguiapplication_p.h>
@@ -56,6 +57,7 @@
 #include "qwaylandinputdeviceintegration_p.h"
 #include "qwaylandinputdeviceintegrationfactory_p.h"
 #include "qwaylandwindow_p.h"
+#include "qwaylandsessionmanager_p.h"
 
 #include <QtWaylandClient/private/qwayland-xdg-system-bell-v1.h>
 
@@ -87,6 +89,8 @@ QWaylandIntegration::QWaylandIntegration(const QString &platformName)
     : mPlatformName(platformName), mFontDb(new QGenericUnixFontDatabase())
 #endif
 {
+    QCoreApplication::setAttribute(Qt::AA_CompressHighFrequencyEvents);
+
     mDisplay.reset(new QWaylandDisplay(this));
     mPlatformServices.reset(new QWaylandPlatformServices(mDisplay.data()));
 
@@ -126,19 +130,20 @@ bool QWaylandIntegration::hasCapability(QPlatformIntegration::Capability cap) co
     case MultipleWindows:
     case NonFullScreenWindows:
         return true;
-    case RasterGLSurface:
-        return true;
     case WindowActivation:
         return true;
     case ScreenWindowGrabbing: // whether QScreen::grabWindow() is supported
         return false;
+    case OffscreenSurface:
+        return mDisplay->clientBufferIntegration()
+                && mDisplay->clientBufferIntegration()->canCreatePlatformOffscreenSurface();
     default: return QPlatformIntegration::hasCapability(cap);
     }
 }
 
 QPlatformWindow *QWaylandIntegration::createPlatformWindow(QWindow *window) const
 {
-    if ((window->surfaceType() == QWindow::OpenGLSurface || window->surfaceType() == QWindow::RasterGLSurface)
+    if (window->surfaceType() == QWindow::OpenGLSurface
         && mDisplay->clientBufferIntegration())
         return mDisplay->clientBufferIntegration()->createEglWindow(window);
 
@@ -158,6 +163,13 @@ QPlatformOpenGLContext *QWaylandIntegration::createPlatformOpenGLContext(QOpenGL
     return nullptr;
 }
 
+QPlatformOffscreenSurface *QWaylandIntegration::createPlatformOffscreenSurface(QOffscreenSurface *surface) const
+{
+    if (mDisplay->clientBufferIntegration())
+        return mDisplay->clientBufferIntegration()->createPlatformOffscreenSurface(surface);
+    return nullptr;
+}
+
 QOpenGLContext *QWaylandIntegration::createOpenGLContext(EGLContext context, EGLDisplay contextDisplay, QOpenGLContext *shareContext) const
 {
     return mClientBufferIntegration->createOpenGLContext(context, contextDisplay, shareContext);
@@ -171,7 +183,7 @@ QPlatformBackingStore *QWaylandIntegration::createPlatformBackingStore(QWindow *
 
 QAbstractEventDispatcher *QWaylandIntegration::createEventDispatcher() const
 {
-    return QWaylandEventDispatcher::createEventDispatcher();
+    return createUnixEventDispatcher();
 }
 
 QPlatformNativeInterface *QWaylandIntegration::createPlatformNativeInterface()
@@ -529,6 +541,14 @@ QWaylandShellIntegration *QWaylandIntegration::createShellIntegration(const QStr
     }
 }
 
+#ifndef QT_NO_SESSIONMANAGER
+QPlatformSessionManager *QWaylandIntegration::createPlatformSessionManager(const QString &id, const QString &key) const
+{
+    Q_UNUSED(key);
+    return new QWaylandSessionManager(mDisplay.data(), id);
+}
+#endif
+
 void QWaylandIntegration::reset()
 {
     mServerBufferIntegration.reset();
