@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef  QT_NO_CURSOR
 #include "qwindowscursor.h"
@@ -30,7 +31,7 @@ static bool initResources()
 
 QT_BEGIN_NAMESPACE
 
-using namespace Qt::Literals::StringLiterals;
+using namespace Qt::StringLiterals;
 
 /*!
     \class QWindowsCursorCacheKey
@@ -39,8 +40,8 @@ using namespace Qt::Literals::StringLiterals;
     \internal
 */
 
-QWindowsPixmapCursorCacheKey::QWindowsPixmapCursorCacheKey(const QCursor &c)
-    : bitmapCacheKey(c.pixmap().cacheKey()), maskCacheKey(0)
+QWindowsPixmapCursorCacheKey::QWindowsPixmapCursorCacheKey(const QCursor &c, qreal scaleFactor)
+    : bitmapCacheKey(c.pixmap().cacheKey()), maskCacheKey(0), scaleFactor(scaleFactor)
 {
     if (!bitmapCacheKey) {
         Q_ASSERT(!c.bitmap().isNull());
@@ -48,6 +49,8 @@ QWindowsPixmapCursorCacheKey::QWindowsPixmapCursorCacheKey(const QCursor &c)
         bitmapCacheKey = c.bitmap().cacheKey();
         maskCacheKey = c.mask().cacheKey();
     }
+    hotspotCacheKey.x = c.hotSpot().x();
+    hotspotCacheKey.y = c.hotSpot().y();
 }
 
 /*!
@@ -523,6 +526,7 @@ CursorHandlePtr QWindowsCursor::standardWindowCursor(Qt::CursorShape shape)
 
 HCURSOR QWindowsCursor::m_overriddenCursor = nullptr;
 HCURSOR QWindowsCursor::m_overrideCursor = nullptr;
+POINT QWindowsCursor::m_cursorPositionCache = {0,0};
 
 /*!
     \brief Return cached pixmap cursor or create new one.
@@ -530,7 +534,8 @@ HCURSOR QWindowsCursor::m_overrideCursor = nullptr;
 
 CursorHandlePtr QWindowsCursor::pixmapWindowCursor(const QCursor &c)
 {
-    const QWindowsPixmapCursorCacheKey cacheKey(c);
+    const qreal scaleFactor = QHighDpiScaling::factor(m_screen);
+    const QWindowsPixmapCursorCacheKey cacheKey(c, scaleFactor);
     PixmapCursorCache::iterator it = m_pixmapCursorCache.find(cacheKey);
     if (it == m_pixmapCursorCache.end()) {
         if (m_pixmapCursorCache.size() > 50) {
@@ -545,7 +550,6 @@ CursorHandlePtr QWindowsCursor::pixmapWindowCursor(const QCursor &c)
                     ++it;
             }
         }
-        const qreal scaleFactor = QHighDpiScaling::factor(m_screen);
         const QPixmap pixmap = c.pixmap();
         const HCURSOR hc = pixmap.isNull()
             ? createBitmapCursor(c, scaleFactor)
@@ -633,8 +637,9 @@ void QWindowsCursor::clearOverrideCursor()
 QPoint QWindowsCursor::mousePosition()
 {
     POINT p;
-    GetCursorPos(&p);
-    return QPoint(p.x, p.y);
+    if (GetCursorPos(&p))
+        m_cursorPositionCache = p;
+    return QPoint(m_cursorPositionCache.x, m_cursorPositionCache.y);
 }
 
 QWindowsCursor::State QWindowsCursor::cursorState()
@@ -658,6 +663,7 @@ QPoint QWindowsCursor::pos() const
 
 void QWindowsCursor::setPos(const QPoint &pos)
 {
+    m_cursorPositionCache = {pos.x(), pos.y()};
     SetCursorPos(pos.x() , pos.y());
 }
 
@@ -705,6 +711,7 @@ QPixmap QWindowsCursor::dragDefaultCursor(Qt::DropAction action) const
         break;
     }
 
+#if QT_CONFIG(imageformat_xpm)
     static const char * const ignoreDragCursorXpmC[] = {
     "24 30 3 1",
     ".        c None",
@@ -740,6 +747,7 @@ QPixmap QWindowsCursor::dragDefaultCursor(Qt::DropAction action) const
     "............XaaaaaaaaX..",
     ".............XXaaaaXX...",
     "...............XXXX....."};
+#endif
 
     if (m_ignoreDragCursor.isNull()) {
         HCURSOR cursor = LoadCursor(nullptr, IDC_NO);
@@ -758,8 +766,10 @@ QPixmap QWindowsCursor::dragDefaultCursor(Qt::DropAction action) const
 
             m_ignoreDragCursor = QPixmap::fromImage(colorImage);
             delete [] colorBits;
+#if QT_CONFIG(imageformat_xpm)
         } else {
             m_ignoreDragCursor = QPixmap(ignoreDragCursorXpmC);
+#endif
         }
 
         DeleteObject(iconInfo.hbmMask);
@@ -773,7 +783,8 @@ HCURSOR QWindowsCursor::hCursor(const QCursor &c) const
 {
     const Qt::CursorShape shape = c.shape();
     if (shape == Qt::BitmapCursor) {
-        const auto pit = m_pixmapCursorCache.constFind(QWindowsPixmapCursorCacheKey(c));
+        const qreal scaleFactor = QHighDpiScaling::factor(m_screen);
+        const auto pit = m_pixmapCursorCache.constFind(QWindowsPixmapCursorCacheKey(c, scaleFactor));
         if (pit != m_pixmapCursorCache.constEnd())
             return pit.value()->handle();
     } else {
