@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QWINDOWSCURSOR_H
 #define QWINDOWSCURSOR_H
@@ -50,20 +15,31 @@ QT_BEGIN_NAMESPACE
 
 struct QWindowsPixmapCursorCacheKey
 {
-    explicit QWindowsPixmapCursorCacheKey(const QCursor &c);
+    explicit QWindowsPixmapCursorCacheKey(const QCursor &c, qreal scaleFactor);
 
     qint64 bitmapCacheKey;
     qint64 maskCacheKey;
+    union {
+        qint64 hashKey;
+        struct {
+            qint32 x;
+            qint32 y;
+        };
+    } hotspotCacheKey;
+    qreal scaleFactor;
 };
 
 inline bool operator==(const QWindowsPixmapCursorCacheKey &k1, const QWindowsPixmapCursorCacheKey &k2)
 {
-    return k1.bitmapCacheKey == k2.bitmapCacheKey && k1.maskCacheKey == k2.maskCacheKey;
+    return k1.bitmapCacheKey == k2.bitmapCacheKey &&
+           k1.maskCacheKey == k2.maskCacheKey &&
+           k1.hotspotCacheKey.hashKey == k2.hotspotCacheKey.hashKey &&
+           k1.scaleFactor == k2.scaleFactor;
 }
 
-inline uint qHash(const QWindowsPixmapCursorCacheKey &k, uint seed) noexcept
+inline size_t qHash(const QWindowsPixmapCursorCacheKey &k, size_t seed) noexcept
 {
-    return (uint(k.bitmapCacheKey) + uint(k.maskCacheKey)) ^ seed;
+    return (size_t(k.bitmapCacheKey) + size_t(k.maskCacheKey)) ^ seed;
 }
 
 class CursorHandle
@@ -113,6 +89,8 @@ public:
     QPoint pos() const override;
     void setPos(const QPoint &pos) override;
 
+    QSize size() const override;
+
     static HCURSOR createPixmapCursor(QPixmap pixmap, const QPoint &hotSpot, qreal scaleFactor = 1);
     static HCURSOR createPixmapCursor(const PixmapCursor &pc, qreal scaleFactor = 1) { return createPixmapCursor(pc.pixmap, pc.hotSpot, scaleFactor); }
     static PixmapCursor customCursor(Qt::CursorShape cursorShape, const QPlatformScreen *screen = nullptr);
@@ -145,6 +123,7 @@ private:
 
     static HCURSOR m_overriddenCursor;
     static HCURSOR m_overrideCursor;
+    static POINT m_cursorPositionCache;
 };
 
 QT_END_NAMESPACE
