@@ -1,53 +1,16 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qwindowsinputcontext.h"
 #include "qwindowscontext.h"
 #include "qwindowswindow.h"
 #include "qwindowsintegration.h"
-#include "qwindowsmousehandler.h"
 
 #include <QtCore/qdebug.h>
 #include <QtCore/qobject.h>
 #include <QtCore/qrect.h>
 #include <QtCore/qtextboundaryfinder.h>
-#include <QtCore/qoperatingsystemversion.h>
 
 #include <QtGui/qevent.h>
 #include <QtGui/qtextformat.h>
@@ -160,7 +123,6 @@ Q_CORE_EXPORT QLocale qt_localeFromLCID(LCID id); // from qlocale_win.cpp
     needs to be checked (mouse grab might interfere with candidate window).
 
     \internal
-    \ingroup qt-lighthouse-win
 */
 
 
@@ -199,7 +161,6 @@ bool QWindowsInputContext::hasCapability(Capability capability) const
 
 void QWindowsInputContext::reset()
 {
-    QPlatformInputContext::reset();
     if (!m_compositionContext.hwnd)
         return;
     qCDebug(lcQpaInputMethods) << __FUNCTION__;
@@ -276,19 +237,9 @@ void QWindowsInputContext::showInputPanel()
     if (!m_caretCreated && m_transparentBitmap)
         m_caretCreated = CreateCaret(platformWindow->handle(), m_transparentBitmap, 0, 0);
 
-    // For some reason, the on-screen keyboard is only triggered on the Surface
-    // with Windows 10 if the Windows IME is (re)enabled _after_ the caret is shown.
     if (m_caretCreated) {
         cursorRectChanged();
-        // We only call ShowCaret() on Windows 10 as in earlier versions the caret
-        // would actually be visible (QTBUG-74492) and the workaround for the
-        // Surface seems unnecessary there anyway. But leave it hidden for IME.
-        if (QOperatingSystemVersion::current() >= QOperatingSystemVersion::Windows10)
-            ShowCaret(platformWindow->handle());
-        else
-            HideCaret(platformWindow->handle());
-        setWindowsImeEnabled(platformWindow, false);
-        setWindowsImeEnabled(platformWindow, true);
+        ShowCaret(platformWindow->handle());
     }
 }
 
@@ -333,7 +284,6 @@ void QWindowsInputContext::update(Qt::InputMethodQueries queries)
 {
     if (queries & Qt::ImEnabled)
         updateEnabled();
-    QPlatformInputContext::update(queries);
 }
 
 void QWindowsInputContext::cursorRectChanged()
@@ -664,9 +614,9 @@ void QWindowsInputContext::handleInputLanguageChanged(WPARAM wparam, LPARAM lpar
     m_locale = qt_localeFromLCID(m_languageId);
     emitLocaleChanged();
 
-    qCDebug(lcQpaInputMethods) << __FUNCTION__ << hex << showbase
+    qCDebug(lcQpaInputMethods) << __FUNCTION__ << Qt::hex << Qt::showbase
         << oldLanguageId  << "->" << newLanguageId << "Character set:"
-        << DWORD(wparam) << dec << noshowbase << m_locale;
+        << DWORD(wparam) << Qt::dec << Qt::noshowbase << m_locale;
 }
 
 /*!
@@ -724,9 +674,9 @@ int QWindowsInputContext::reconvertString(RECONVERTSTRING *reconv)
     reconv->dwCompStrOffset = DWORD(startPos) * sizeof(ushort); // byte count.
     reconv->dwTargetStrLen = reconv->dwCompStrLen;
     reconv->dwTargetStrOffset = reconv->dwCompStrOffset;
-    ushort *pastReconv = reinterpret_cast<ushort *>(reconv + 1);
+    auto *pastReconv = reinterpret_cast<ushort *>(reconv + 1);
     std::copy(surroundingText.utf16(), surroundingText.utf16() + surroundingText.size(),
-              QT_MAKE_UNCHECKED_ARRAY_ITERATOR(pastReconv));
+              pastReconv);
     return memSize;
 }
 
