@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qwindowsmimeregistry.h"
 #include "qwindowscontext.h"
@@ -90,6 +91,7 @@ static inline bool readDib(QBuffer &buffer, QImage &img)
     return true;
 }
 
+#if QT_CONFIG(imageformat_bmp)
 static QByteArray writeDib(const QImage &img)
 {
     QByteArray ba;
@@ -188,6 +190,7 @@ static bool qt_write_dibv5(QDataStream &s, QImage image)
     delete[] buf;
     return true;
 }
+#endif //QT_CONFIG(imageformat_bmp)
 
 // helpers for using global memory
 
@@ -793,7 +796,7 @@ bool QWindowsMimeHtml::convertFromMime(const FORMATETC &formatetc, const QMimeDa
 }
 
 
-#ifndef QT_NO_IMAGEFORMAT_BMP
+#if QT_CONFIG(imageformat_bmp)
 class QWindowsMimeImage : public QWindowsMimeConverter
 {
 public:
@@ -944,7 +947,7 @@ QVariant QWindowsMimeImage::convertToMime(const QString &mimeType, IDataObject *
     // Failed
     return result;
 }
-#endif
+#endif //QT_CONFIG(imageformat_bmp)
 
 class QBuiltInMimes : public QWindowsMimeConverter
 {
@@ -1341,9 +1344,9 @@ void QWindowsMimeRegistry::ensureInitialized() const
 {
     if (m_internalMimeCount == 0) {
         m_internalMimeCount = -1; // prevent reentrancy when types register themselves
-#ifndef QT_NO_IMAGEFORMAT_BMP
+#if QT_CONFIG(imageformat_bmp)
         (void)new QWindowsMimeImage;
-#endif //QT_NO_IMAGEFORMAT_BMP
+#endif //QT_CONFIG(imageformat_bmp)
         (void)new QLastResortMimes;
         (void)new QWindowsMimeText;
         (void)new QWindowsMimeURI;
