@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qwindowsscreen.h"
 #include "qwindowscontext.h"
@@ -7,6 +8,7 @@
 #include "qwindowsintegration.h"
 #include "qwindowscursor.h"
 #include "qwindowstheme.h"
+#include "qwindowswindowclassregistry.h"
 
 #include <QtCore/qt_windows.h>
 
@@ -702,8 +704,8 @@ void QWindowsScreenManager::initialize()
 {
     qCDebug(lcQpaScreen) << "Initializing screen manager";
 
-    auto className = QWindowsContext::instance()->registerWindowClass(
-        QWindowsContext::classNamePrefix() + QLatin1String("ScreenChangeObserverWindow"),
+    auto className = QWindowsWindowClassRegistry::instance()->registerWindowClass(
+        "ScreenChangeObserverWindow"_L1,
         qDisplayChangeObserverWndProc);
 
     // HWND_MESSAGE windows do not get WM_DISPLAYCHANGE, so we need to create
@@ -782,7 +784,7 @@ void QWindowsScreenManager::addScreen(const QWindowsScreenData &screenData)
     // change here, now that we are processing the WM_DISPLAYCHANGE.
     const auto allWindows = QGuiApplication::allWindows();
     for (QWindow *w : allWindows) {
-        if (w->isVisible() && w->handle() && w->type() != Qt::Desktop) {
+        if (w->isVisible() && w->handle()) {
             if (QWindowsWindow *window = QWindowsWindow::windowsWindowOf(w))
                 window->checkForScreenChanged(QWindowsWindow::ScreenChangeMode::FromScreenAdded);
         }
@@ -806,7 +808,7 @@ void QWindowsScreenManager::removeScreen(int index)
         unsigned movedWindowCount = 0;
         const QWindowList tlws = QGuiApplication::topLevelWindows();
         for (QWindow *w : tlws) {
-            if (w->screen() == screen && w->handle() && w->type() != Qt::Desktop) {
+            if (w->screen() == screen && w->handle()) {
                 if (w->isVisible() && w->windowState() != Qt::WindowMinimized
                     && (QWindowsWindow::baseWindowOf(w)->exStyle() & WS_EX_TOOLWINDOW)) {
                     moveToVirtualScreen(w, primaryScreen);
