@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <QtCore/qt_windows.h>
 
@@ -14,6 +15,7 @@
 #endif
 #include "qwindowsscreen.h"
 #include "qwindowswindow.h"
+#include "qwindowswindowclassregistry.h"
 #include <commctrl.h>
 #include <objbase.h>
 #include <commoncontrols.h>
@@ -39,9 +41,9 @@
 #include <QtGui/private/qabstractfileiconengine_p.h>
 #include <QtGui/private/qwindowsfontdatabase_p.h>
 #include <private/qhighdpiscaling_p.h>
-#include <private/qsystemlibrary_p.h>
 #include <private/qwinregistry_p.h>
 #include <QtCore/private/qfunctions_win_p.h>
+#include <QtGui/private/qwindowsthemecache_p.h>
 
 #include <algorithm>
 
@@ -95,38 +97,67 @@ static constexpr QColor getSysColor(winrt::Windows::UI::Color &&color)
 }
 #endif
 
+static inline QColor getSysColor(int index)
+{
+    COLORREF cr = GetSysColor(index);
+    return QColor(GetRValue(cr), GetGValue(cr), GetBValue(cr));
+}
+
 [[maybe_unused]] [[nodiscard]] static inline QColor qt_accentColor(AccentColorLevel level)
 {
+    QColor accent;
+    QColor accentLight;
+    QColor accentLighter;
+    QColor accentLightest;
+    QColor accentDark;
+    QColor accentDarker;
+    QColor accentDarkest;
+
 #if QT_CONFIG(cpp_winrt)
-    using namespace winrt::Windows::UI::ViewManagement;
-    const auto settings = UISettings();
-    const QColor accent = getSysColor(settings.GetColorValue(UIColorType::Accent));
-    const QColor accentLight = getSysColor(settings.GetColorValue(UIColorType::AccentLight1));
-    const QColor accentLighter = getSysColor(settings.GetColorValue(UIColorType::AccentLight2));
-    const QColor accentLightest = getSysColor(settings.GetColorValue(UIColorType::AccentLight3));
-    const QColor accentDark = getSysColor(settings.GetColorValue(UIColorType::AccentDark1));
-    const QColor accentDarker = getSysColor(settings.GetColorValue(UIColorType::AccentDark2));
-    const QColor accentDarkest = getSysColor(settings.GetColorValue(UIColorType::AccentDark3));
-#else
-    const QWinRegistryKey registry(HKEY_CURRENT_USER, LR"(Software\Microsoft\Windows\DWM)");
-    if (!registry.isValid())
-        return {};
-    const QVariant value = registry.value(L"AccentColor");
-    if (!value.isValid())
-        return {};
-    // The retrieved value is in the #AABBGGRR format, we need to
-    // convert it to the #AARRGGBB format which Qt expects.
-    const QColor abgr = QColor::fromRgba(qvariant_cast<DWORD>(value));
-    if (!abgr.isValid())
-        return {};
-    const QColor accent = QColor::fromRgb(abgr.blue(), abgr.green(), abgr.red(), abgr.alpha());
-    const QColor accentLight = accent.lighter(120);
-    const QColor accentLighter = accentLight.lighter(120);
-    const QColor accentLightest = accentLighter.lighter(120);
-    const QColor accentDark = accent.darker(120);
-    const QColor accentDarker = accentDark.darker(120);
-    const QColor accentDarkest = accentDarker.darker(120);
+    QT_TRY {
+        using namespace winrt::Windows::UI::ViewManagement;
+        const auto settings = UISettings();
+        accent = getSysColor(settings.GetColorValue(UIColorType::Accent));
+        accentLight = getSysColor(settings.GetColorValue(UIColorType::AccentLight1));
+        accentLighter = getSysColor(settings.GetColorValue(UIColorType::AccentLight2));
+        accentLightest = getSysColor(settings.GetColorValue(UIColorType::AccentLight3));
+        accentDark = getSysColor(settings.GetColorValue(UIColorType::AccentDark1));
+        accentDarker = getSysColor(settings.GetColorValue(UIColorType::AccentDark2));
+        accentDarkest = getSysColor(settings.GetColorValue(UIColorType::AccentDark3));
+    } QT_CATCH(...) {
+        // pass, just fall back to WIN32 API implementation
+    }
 #endif
+
+    if (!accent.isValid()) {
+        accent = []()->QColor {
+            // MS uses the aquatic contrast theme as an example in the URL below:
+            // https://learn.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-getsyscolor#windows-1011-system-colors
+            if (QWindowsTheme::queryHighContrast())
+                return getSysColor(COLOR_HIGHLIGHT);
+            const QWinRegistryKey registry(HKEY_CURRENT_USER, LR"(Software\Microsoft\Windows\DWM)");
+            if (!registry.isValid())
+                return {};
+            const QVariant value = registry.value(L"AccentColor");
+            if (!value.isValid())
+                return {};
+            // The retrieved value is in the #AABBGGRR format, we need to
+            // convert it to the #AARRGGBB format which Qt expects.
+            const QColor abgr = QColor::fromRgba(qvariant_cast<DWORD>(value));
+            if (!abgr.isValid())
+                return {};
+            return QColor::fromRgb(abgr.blue(), abgr.green(), abgr.red(), abgr.alpha());
+        }();
+        if (!accent.isValid())
+            return {};
+        accentLight = accent.lighter(120);
+        accentLighter = accentLight.lighter(120);
+        accentLightest = accentLighter.lighter(120);
+        accentDark = accent.darker(120);
+        accentDarker = accentDark.darker(120);
+        accentDarkest = accentDarker.darker(120);
+    }
+
     switch (level) {
     case AccentColorDarkest:
         return accentDarkest;
@@ -145,12 +176,6 @@ static constexpr QColor getSysColor(winrt::Windows::UI::Color &&color)
     }
 }
 
-static inline QColor getSysColor(int index)
-{
-    COLORREF cr = GetSysColor(index);
-    return QColor(GetRValue(cr), GetGValue(cr), GetBValue(cr));
-}
-
 // QTBUG-48823/Windows 10: SHGetFileInfo() (as called by item views on file system
 // models has been observed to trigger a WM_PAINT on the mainwindow. Suppress the
 // behavior by running it in a thread.
@@ -332,46 +357,39 @@ void QWindowsTheme::populateLightSystemBasePalette(QPalette &result)
 
 void QWindowsTheme::populateDarkSystemBasePalette(QPalette &result)
 {
-    QColor foreground, background,
-           accent, accentDark, accentDarker, accentDarkest,
-           accentLight, accentLighter, accentLightest;
+    QColor foreground;
+    QColor background;
 #if QT_CONFIG(cpp_winrt)
-    using namespace winrt::Windows::UI::ViewManagement;
-    const auto settings = UISettings();
-
     // We have to craft a palette from these colors. The settings.UIElementColor(UIElementType) API
     // returns the old system colors, not the dark mode colors. If the background is black (which it
     // usually), then override it with a dark gray instead so that we can go up and down the lightness.
     if (QWindowsTheme::queryColorScheme() == Qt::ColorScheme::Dark) {
-        // the system is actually running in dark mode, so UISettings will give us dark colors
-        foreground = getSysColor(settings.GetColorValue(UIColorType::Foreground));
-        background = [&settings]() -> QColor {
-            auto systemBackground = getSysColor(settings.GetColorValue(UIColorType::Background));
-            if (systemBackground == Qt::black)
-                systemBackground = QColor(0x1E, 0x1E, 0x1E);
-            return systemBackground;
-        }();
-        accent = qt_accentColor(AccentColorNormal);
-        accentDark = qt_accentColor(AccentColorDark);
-        accentDarker = qt_accentColor(AccentColorDarker);
-        accentDarkest = qt_accentColor(AccentColorDarkest);
-        accentLight = qt_accentColor(AccentColorLight);
-        accentLighter = qt_accentColor(AccentColorLighter);
-        accentLightest = qt_accentColor(AccentColorLightest);
-    } else
+        QT_TRY {
+            using namespace winrt::Windows::UI::ViewManagement;
+            const auto settings = UISettings();
+            // the system is actually running in dark mode, so UISettings will give us dark colors
+            foreground = getSysColor(settings.GetColorValue(UIColorType::Foreground));
+            background = [&settings]() -> QColor {
+                auto systemBackground = getSysColor(settings.GetColorValue(UIColorType::Background));
+                if (systemBackground == Qt::black)
+                    systemBackground = QColor(0x1E, 0x1E, 0x1E);
+                return systemBackground;
+            }();
+        } QT_CATCH(...) {
+            // pass, just fall back to WIN32 API implementation
+        }
+    }
 #endif
-    {
+    if (!background.isValid()) {
         // If the system is running in light mode, then we need to make up our own dark palette
         foreground = Qt::white;
         background = QColor(0x1E, 0x1E, 0x1E);
-        accent = qt_accentColor(AccentColorNormal);
-        accentDark = accent.darker(120);
-        accentDarker = accentDark.darker(120);
-        accentDarkest = accentDarker.darker(120);
-        accentLight = accent.lighter(120);
-        accentLighter = accentLight.lighter(120);
-        accentLightest = accentLighter.lighter(120);
     }
+
+    const QColor accent = qt_accentColor(AccentColorNormal);
+    const QColor accentDarkest = qt_accentColor(AccentColorDarkest);
+    const QColor accentLighter = qt_accentColor(AccentColorLighter);
+    const QColor accentLightest = qt_accentColor(AccentColorLightest);
     const QColor linkColor = accentLightest;
     const QColor buttonColor = background.lighter(200);
 
@@ -488,6 +506,39 @@ static inline QPalette *menuBarPalette(const QPalette &menuPalette, bool light)
 const char *QWindowsTheme::name = "windows";
 QWindowsTheme *QWindowsTheme::m_instance = nullptr;
 
+LRESULT QT_WIN_CALLBACK qThemeChangeObserverWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
+{
+    switch (message) {
+    case WM_SETTINGCHANGE:
+        // Only refresh the theme if the user changes the personalize settings
+        if (!((wParam == 0) && (lParam != 0) // lParam sometimes may be NULL.
+            && (wcscmp(reinterpret_cast<LPCWSTR>(lParam), L"ImmersiveColorSet") == 0)))
+            break;
+        Q_FALLTHROUGH();
+    case WM_THEMECHANGED:
+    case WM_SYSCOLORCHANGE:
+    case WM_DWMCOLORIZATIONCOLORCHANGED:
+        qCDebug(lcQpaTheme) << "Handling theme change due to"
+            << qUtf8Printable(decodeMSG(MSG{hwnd, message, wParam, lParam, 0, {0, 0}}).trimmed());
+        QWindowsTheme::handleThemeChange();
+
+        MSG msg; // Clear the message queue, we've already reacted to the change
+        while (PeekMessage(&msg, hwnd, 0, 0, PM_REMOVE));
+
+        // FIXME: Despite clearing the message queue above, Windows will send
+        // us redundant theme change events for our single window. We want the
+        // theme change delivery to be synchronous, so we can't easily debounce
+        // them by peeking into the QWSI event queue, but perhaps there are other
+        // ways.
+
+        break;
+    default:
+        break;
+    }
+
+    return DefWindowProc(hwnd, message, wParam, lParam);
+}
+
 QWindowsTheme::QWindowsTheme()
 {
     m_instance = this;
@@ -497,6 +548,16 @@ QWindowsTheme::QWindowsTheme()
     std::fill(m_palettes, m_palettes + NPalettes, nullptr);
     refresh();
     refreshIconPixmapSizes();
+
+    auto className = QWindowsWindowClassRegistry::instance()->registerWindowClass(
+        "ThemeChangeObserverWindow"_L1,
+        qThemeChangeObserverWndProc);
+    // HWND_MESSAGE windows do not get the required theme events,
+    // so we use a real top-level window that we never show.
+    m_themeChangeObserver = CreateWindowEx(0, reinterpret_cast<LPCWSTR>(className.utf16()),
+        nullptr, WS_TILED, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
+        nullptr, nullptr, GetModuleHandle(nullptr), nullptr);
+    Q_ASSERT(m_themeChangeObserver);
 }
 
 QWindowsTheme::~QWindowsTheme()
@@ -506,6 +567,13 @@ QWindowsTheme::~QWindowsTheme()
     m_instance = nullptr;
 }
 
+void QWindowsTheme::destroyThemeChangeWindow()
+{
+    qCDebug(lcQpaTheme) << "Destroying theme change window";
+    DestroyWindow(m_themeChangeObserver);
+    m_themeChangeObserver = nullptr;
+}
+
 static inline QStringList iconThemeSearchPaths()
 {
     const QFileInfo appDir(QCoreApplication::applicationDirPath() + "/icons"_L1);
@@ -602,26 +670,39 @@ Qt::ColorScheme QWindowsTheme::effectiveColorScheme()
 void QWindowsTheme::requestColorScheme(Qt::ColorScheme scheme)
 {
     s_colorSchemeOverride = scheme;
-    handleSettingsChanged();
+    handleThemeChange();
 }
 
-void QWindowsTheme::handleSettingsChanged()
+Qt::ContrastPreference QWindowsTheme::contrastPreference() const
 {
+    return queryHighContrast() ? Qt::ContrastPreference::HighContrast
+                               : Qt::ContrastPreference::NoPreference;
+}
+
+void QWindowsTheme::handleThemeChange()
+{
+    auto *integration = QWindowsIntegration::instance();
+    if (!integration)
+        return;
+
+    QWindowsThemeCache::clearAllThemeCaches();
+
     const auto oldColorScheme = s_colorScheme;
     s_colorScheme = Qt::ColorScheme::Unknown; // make effectiveColorScheme() query registry
-    const auto newColorScheme = effectiveColorScheme();
-    const bool colorSchemeChanged = newColorScheme != oldColorScheme;
-    s_colorScheme = newColorScheme;
-    if (!colorSchemeChanged)
-        return;
-    auto integration = QWindowsIntegration::instance();
-    integration->updateApplicationBadge();
-    if (integration->darkModeHandling().testFlag(QWindowsApplication::DarkModeStyle)) {
-        QWindowsTheme::instance()->refresh();
-        QWindowSystemInterface::handleThemeChange<QWindowSystemInterface::SynchronousDelivery>();
+    s_colorScheme = effectiveColorScheme();
+    if (s_colorScheme != oldColorScheme) {
+        // Only propagate color scheme changes if the scheme actually changed
+        integration->updateApplicationBadge();
+
+        for (QWindowsWindow *w : std::as_const(QWindowsContext::instance()->windows()))
+            w->setDarkBorder(s_colorScheme == Qt::ColorScheme::Dark);
     }
-    for (QWindowsWindow *w : std::as_const(QWindowsContext::instance()->windows()))
-        w->setDarkBorder(s_colorScheme == Qt::ColorScheme::Dark);
+
+    // But always reset palette and fonts, and signal the theme
+    // change, as other parts of the theme could have changed,
+    // such as the accent color.
+    QWindowsTheme::instance()->refresh();
+    QWindowSystemInterface::handleThemeChange<QWindowSystemInterface::SynchronousDelivery>();
 }
 
 void QWindowsTheme::clearPalettes()
@@ -792,12 +873,6 @@ QPlatformSystemTrayIcon *QWindowsTheme::createPlatformSystemTrayIcon() const
 }
 #endif
 
-void QWindowsTheme::windowsThemeChanged(QWindow * window)
-{
-    refresh();
-    QWindowSystemInterface::handleThemeChange(window);
-}
-
 static int fileIconSizes[FileIconSizeCount];
 
 void QWindowsTheme::refreshIconPixmapSizes()
@@ -819,15 +894,15 @@ Q_GUI_EXPORT QPixmap qt_pixmapFromWinHICON(HICON icon);
 
 static QPixmap loadIconFromShell32(int resourceId, QSizeF size)
 {
-    if (const HMODULE hmod = QSystemLibrary::load(L"shell32")) {
-        auto iconHandle =
-            static_cast<HICON>(LoadImage(hmod, MAKEINTRESOURCE(resourceId),
-                                         IMAGE_ICON, int(size.width()), int(size.height()), 0));
-        if (iconHandle) {
-            QPixmap iconpixmap = qt_pixmapFromWinHICON(iconHandle);
-            DestroyIcon(iconHandle);
-            return iconpixmap;
-        }
+    HMODULE shell32 = ::GetModuleHandleW(L"shell32.dll");
+    Q_ASSERT(shell32);
+    auto iconHandle =
+        static_cast<HICON>(LoadImage(shell32, MAKEINTRESOURCE(resourceId),
+                                     IMAGE_ICON, int(size.width()), int(size.height()), 0));
+    if (iconHandle) {
+        QPixmap iconpixmap = qt_pixmapFromWinHICON(iconHandle);
+        DestroyIcon(iconHandle);
+        return iconpixmap;
     }
     return QPixmap();
 }
