@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QWINDOWSTHEME_H
 #define QWINDOWSTHEME_H
@@ -31,7 +32,11 @@ public:
 #endif
     QVariant themeHint(ThemeHint) const override;
 
-    Appearance appearance() const override;
+    Qt::ColorScheme colorScheme() const override;
+    void requestColorScheme(Qt::ColorScheme scheme) override;
+    Qt::ContrastPreference contrastPreference() const override;
+
+    static void handleThemeChange();
 
     const QPalette *palette(Palette type = SystemPalette) const override
         { return m_palettes[type]; }
@@ -41,8 +46,8 @@ public:
     QPixmap standardPixmap(StandardPixmap sp, const QSizeF &size) const override;
 
     QIcon fileIcon(const QFileInfo &fileInfo, QPlatformTheme::IconOptions iconOptions = {}) const override;
+    QIconEngine *createIconEngine(const QString &iconName) const override;
 
-    void windowsThemeChanged(QWindow *window);
     void displayChanged() { refreshIconPixmapSizes(); }
 
     QList<QSize> availableFileIconSizes() const { return m_fileIconSizes; }
@@ -53,24 +58,38 @@ public:
     void showPlatformMenuBar() override;
 
     static bool useNativeMenus();
-    static bool queryDarkMode();
-    static bool queryHighContrast();
 
     void refreshFonts();
     void refresh();
 
     static const char *name;
 
+    static QPalette systemPalette(Qt::ColorScheme);
+    static bool queryHighContrast();
 private:
     void clearPalettes();
     void refreshPalettes();
     void clearFonts();
     void refreshIconPixmapSizes();
 
+    static void populateLightSystemBasePalette(QPalette &result);
+    static void populateDarkSystemBasePalette(QPalette &result);
+
+    static Qt::ColorScheme queryColorScheme();
+    static Qt::ColorScheme effectiveColorScheme();
+
     static QWindowsTheme *m_instance;
+    static inline Qt::ColorScheme s_colorScheme = Qt::ColorScheme::Unknown;
+    static inline Qt::ColorScheme s_colorSchemeOverride = Qt::ColorScheme::Unknown;
+
+    friend class QWindowsContext;
+
     QPalette *m_palettes[NPalettes];
     QFont *m_fonts[NFonts];
     QList<QSize> m_fileIconSizes;
+
+    HWND m_themeChangeObserver = nullptr;
+    void destroyThemeChangeWindow();
 };
 
 QT_END_NAMESPACE
