@@ -1,54 +1,20 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
-#include <QtCore/QtConfig>
-#ifndef QT_NO_ACCESSIBILITY
+#include <QtGui/qtguiglobal.h>
+#if QT_CONFIG(accessibility)
 
 #include "qwindowsuiatextrangeprovider.h"
 #include "qwindowsuiamainprovider.h"
 #include "qwindowsuiautils.h"
 #include "qwindowscontext.h"
 
-#include <QtGui/QAccessible>
-#include <QtGui/QAccessibleInterface>
-#include <QtCore/QDebug>
-#include <QtCore/QString>
+#include <QtGui/qaccessible.h>
+#include <QtCore/qloggingcategory.h>
+#include <QtCore/qstring.h>
+#include <QtCore/qvarlengtharray.h>
+#include <QtCore/private/qcomvariant_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -81,7 +47,7 @@ HRESULT QWindowsUiaTextRangeProvider::Clone(ITextRangeProvider **pRetVal)
     if (!pRetVal)
         return E_INVALIDARG;
 
-    *pRetVal = new QWindowsUiaTextRangeProvider(id(), m_startOffset, m_endOffset);
+    *pRetVal = makeComObject<QWindowsUiaTextRangeProvider>(id(), m_startOffset, m_endOffset).Detach();
     return S_OK;
 }
 
@@ -93,7 +59,7 @@ HRESULT QWindowsUiaTextRangeProvider::Compare(ITextRangeProvider *range, BOOL *p
     if (!range || !pRetVal)
         return E_INVALIDARG;
 
-    QWindowsUiaTextRangeProvider *targetProvider = static_cast<QWindowsUiaTextRangeProvider *>(range);
+    auto *targetProvider = static_cast<QWindowsUiaTextRangeProvider *>(range);
     *pRetVal = ((targetProvider->m_startOffset == m_startOffset) && (targetProvider->m_endOffset == m_endOffset));
     return S_OK;
 }
@@ -111,7 +77,7 @@ HRESULT QWindowsUiaTextRangeProvider::CompareEndpoints(TextPatternRangeEndpoint
     if (!targetRange || !pRetVal)
         return E_INVALIDARG;
 
-    QWindowsUiaTextRangeProvider *targetProvider = static_cast<QWindowsUiaTextRangeProvider *>(targetRange);
+    auto *targetProvider = static_cast<QWindowsUiaTextRangeProvider *>(targetRange);
 
     int point = (endpoint == TextPatternRangeEndpoint_Start) ? m_startOffset : m_endOffset;
     int targetPoint = (targetEndpoint == TextPatternRangeEndpoint_Start) ?
@@ -143,8 +109,10 @@ HRESULT QWindowsUiaTextRangeProvider::ExpandToEnclosingUnit(TextUnit unit)
             m_endOffset = m_startOffset + 1;
         } else {
             QString text = textInterface->text(0, len);
-            for (int t = m_startOffset; t >= 0; --t) {
-                if (!isTextUnitSeparator(unit, text[t]) && ((t == 0) || isTextUnitSeparator(unit, text[t - 1]))) {
+            const int start = m_startOffset >= 0 && m_startOffset < len
+                              ? m_startOffset : len - 1;
+            for (int t = start; t >= 0; --t) {
+                if (!text.isEmpty() && !isTextUnitSeparator(unit, text[t]) && ((t == 0) || isTextUnitSeparator(unit, text[t - 1]))) {
                     m_startOffset = t;
                     break;
                 }
@@ -191,16 +159,25 @@ HRESULT STDMETHODCALLTYPE QWindowsUiaTextRangeProvider::GetAttributeValue(TEXTAT
 
     switch (attributeId) {
     case UIA_IsReadOnlyAttributeId:
-        setVariantBool(accessible->state().readOnly, pRetVal);
+        *pRetVal = QComVariant{ accessible->state().readOnly ? true : false }.release();
         break;
     case UIA_CaretPositionAttributeId:
         if (textInterface->cursorPosition() == 0)
-            setVariantI4(CaretPosition_BeginningOfLine, pRetVal);
+            *pRetVal = QComVariant{ static_cast<long>(CaretPosition_BeginningOfLine) }.release();
         else if (textInterface->cursorPosition() == textInterface->characterCount())
-            setVariantI4(CaretPosition_EndOfLine, pRetVal);
+            *pRetVal = QComVariant{ static_cast<long>(CaretPosition_EndOfLine) }.release();
         else
-            setVariantI4(CaretPosition_Unknown, pRetVal);
+            *pRetVal = QComVariant{ static_cast<long>(CaretPosition_Unknown) }.release();
         break;
+    case UIA_StrikethroughStyleAttributeId:
+    {
+        const QString value = valueForIA2Attribute(textInterface, QStringLiteral("text-line-through-type"));
+        if (value.isEmpty())
+            break;
+        const TextDecorationLineStyle uiaLineStyle = uiaLineStyleForIA2LineStyle(value);
+        *pRetVal = QComVariant{ static_cast<long>(uiaLineStyle) }.release();
+        break;
+    }
     default:
         break;
     }
@@ -228,7 +205,7 @@ HRESULT QWindowsUiaTextRangeProvider::GetBoundingRectangles(SAFEARRAY **pRetVal)
         return UIA_E_ELEMENTNOTAVAILABLE;
 
     int len = textInterface->characterCount();
-    QList<QRect> rectList;
+    QVarLengthArray<QRect> rectList;
 
     if ((m_startOffset >= 0) && (m_endOffset <= len) && (m_startOffset < m_endOffset)) {
         int start, end;
@@ -238,12 +215,12 @@ HRESULT QWindowsUiaTextRangeProvider::GetBoundingRectangles(SAFEARRAY **pRetVal)
             int endRange = qMin(end, m_endOffset);
             if (startRange < endRange) {
                 // Calculates a bounding rectangle for the line and adds it to the list.
-                QRect startRect = textInterface->characterRect(startRange);
-                QRect endRect = textInterface->characterRect(endRange - 1);
-                QRect lineRect(qMin(startRect.x(), endRect.x()),
-                               qMin(startRect.y(), endRect.y()),
-                               qMax(startRect.x() + startRect.width(), endRect.x() + endRect.width()) - qMin(startRect.x(), endRect.x()),
-                               qMax(startRect.y() + startRect.height(), endRect.y() + endRect.height()) - qMin(startRect.y(), endRect.y()));
+                const QRect startRect = textInterface->characterRect(startRange);
+                const QRect endRect = textInterface->characterRect(endRange - 1);
+                const QRect lineRect(qMin(startRect.x(), endRect.x()),
+                                     qMin(startRect.y(), endRect.y()),
+                                     qMax(startRect.x() + startRect.width(), endRect.x() + endRect.width()) - qMin(startRect.x(), endRect.x()),
+                                     qMax(startRect.y() + startRect.height(), endRect.y() + endRect.height()) - qMin(startRect.y(), endRect.y()));
                 rectList.append(lineRect);
             }
             if (end >= len) break;
@@ -290,7 +267,7 @@ HRESULT QWindowsUiaTextRangeProvider::GetEnclosingElement(IRawElementProviderSim
     if (!accessible)
         return UIA_E_ELEMENTNOTAVAILABLE;
 
-    *pRetVal = QWindowsUiaMainProvider::providerForAccessible(accessible);
+    *pRetVal = QWindowsUiaMainProvider::providerForAccessible(accessible).Detach();
     return S_OK;
 }
 
@@ -318,7 +295,7 @@ HRESULT QWindowsUiaTextRangeProvider::GetText(int maxLength, BSTR *pRetVal)
 
     if ((maxLength > -1) && (rangeText.size() > maxLength))
         rangeText.truncate(maxLength);
-    *pRetVal = bStrFromQString(rangeText);
+    *pRetVal = QBStr(rangeText).release();
     return S_OK;
 }
 
@@ -341,14 +318,14 @@ HRESULT QWindowsUiaTextRangeProvider::Move(TextUnit unit, int count, int *pRetVa
 
     int len = textInterface->characterCount();
 
-    if (len < 1)
+    if (len < 1 || count == 0)  // MSDN: "Zero has no effect."
         return S_OK;
 
     if (unit == TextUnit_Character) {
         // Moves the start point, ensuring it lies within the bounds.
-        int start = qBound(0, m_startOffset + count, len - 1);
+        int start = qBound(0, m_startOffset + count, len);
         // If range was initially empty, leaves it as is; otherwise, normalizes it to one char.
-        m_endOffset = (m_endOffset > m_startOffset) ? start + 1 : start;
+        m_endOffset = (m_endOffset > m_startOffset) ? qMin(start + 1, len) : start;
         *pRetVal = start - m_startOffset; // Returns the actually moved distance.
         m_startOffset = start;
     } else {
@@ -374,7 +351,7 @@ HRESULT QWindowsUiaTextRangeProvider::MoveEndpointByRange(TextPatternRangeEndpoi
     qCDebug(lcQpaUiAutomation) << __FUNCTION__
         << "endpoint=" << endpoint << "targetRange=" << targetRange << "targetEndpoint=" << targetEndpoint << "this: " << this;
 
-    QWindowsUiaTextRangeProvider *targetProvider = static_cast<QWindowsUiaTextRangeProvider *>(targetRange);
+    auto *targetProvider = static_cast<QWindowsUiaTextRangeProvider *>(targetRange);
 
     int targetPoint = (targetEndpoint == TextPatternRangeEndpoint_Start) ?
                 targetProvider->m_startOffset : targetProvider->m_endOffset;
@@ -419,7 +396,7 @@ HRESULT QWindowsUiaTextRangeProvider::MoveEndpointByUnit(TextPatternRangeEndpoin
 
     if (unit == TextUnit_Character) {
         if (endpoint == TextPatternRangeEndpoint_Start) {
-            int boundedValue = qBound(0, m_startOffset + count, len - 1);
+            int boundedValue = qBound(0, m_startOffset + count, len);
             *pRetVal = boundedValue - m_startOffset;
             m_startOffset = boundedValue;
             m_endOffset = qBound(m_startOffset, m_endOffset, len);
@@ -436,15 +413,17 @@ HRESULT QWindowsUiaTextRangeProvider::MoveEndpointByUnit(TextPatternRangeEndpoin
         if (endpoint == TextPatternRangeEndpoint_Start) {
             if (count > 0) {
                 for (int t = m_startOffset; (t < len - 1) && (moved < count); ++t) {
-                    if (isTextUnitSeparator(unit, text[t]) && !isTextUnitSeparator(unit, text[t + 1])) {
+                    if (!text.isEmpty() && isTextUnitSeparator(unit, text[t]) && !isTextUnitSeparator(unit, text[t + 1])) {
                         m_startOffset = t + 1;
                         ++moved;
                     }
                 }
                 m_endOffset = qBound(m_startOffset, m_endOffset, len);
             } else {
-                for (int t = m_startOffset - 1; (t >= 0) && (moved > count); --t) {
-                    if (!isTextUnitSeparator(unit, text[t]) && ((t == 0) || isTextUnitSeparator(unit, text[t - 1]))) {
+                const int start = m_startOffset >= 0 && m_startOffset <= len
+                                  ? m_startOffset : len;
+                for (int t = start - 1; (t >= 0) && (moved > count); --t) {
+                    if (!text.isEmpty() &&!isTextUnitSeparator(unit, text[t]) && ((t == 0) || isTextUnitSeparator(unit, text[t - 1]))) {
                         m_startOffset = t;
                         --moved;
                     }
@@ -519,9 +498,9 @@ HRESULT QWindowsUiaTextRangeProvider::Select()
 }
 
 // Not supported.
-HRESULT QWindowsUiaTextRangeProvider::FindTextW(BSTR /* text */, BOOL /* backward */,
-                                                BOOL /* ignoreCase */,
-                                                ITextRangeProvider **pRetVal)
+HRESULT QWindowsUiaTextRangeProvider::FindText(BSTR /* text */, BOOL /* backward */,
+                                               BOOL /* ignoreCase */,
+                                               ITextRangeProvider **pRetVal)
 {
     if (!pRetVal)
         return E_INVALIDARG;
@@ -549,6 +528,42 @@ HRESULT QWindowsUiaTextRangeProvider::unselect()
     return S_OK;
 }
 
+// helper method to retrieve the value of the given IAccessible2 text attribute,
+// or an empty string if not set
+QString QWindowsUiaTextRangeProvider::valueForIA2Attribute(QAccessibleTextInterface *textInterface,
+                                                           const QString &key)
+{
+    Q_ASSERT(textInterface);
+
+    int startOffset;
+    int endOffset;
+    const QString attributes = textInterface->attributes(m_startOffset, &startOffset, &endOffset);
+    // don't report if attributes don't apply for the whole range
+    if (startOffset > m_startOffset || endOffset < m_endOffset)
+        return {};
+
+    for (auto attr : QStringTokenizer{attributes, u';'})
+    {
+        const QList<QStringView> items = attr.split(u':', Qt::SkipEmptyParts, Qt::CaseSensitive);
+        if (items.count() == 2 && items[0] == key)
+            return items[1].toString();
+    }
+
+    return {};
+}
+
+TextDecorationLineStyle QWindowsUiaTextRangeProvider::uiaLineStyleForIA2LineStyle(const QString &ia2LineStyle)
+{
+    if (ia2LineStyle == QStringLiteral("none"))
+        return TextDecorationLineStyle_None;
+    if (ia2LineStyle == QStringLiteral("single"))
+        return TextDecorationLineStyle_Single;
+    if (ia2LineStyle == QStringLiteral("double"))
+        return TextDecorationLineStyle_Double;
+
+    return TextDecorationLineStyle_Other;
+}
+
 QT_END_NAMESPACE
 
-#endif // QT_NO_ACCESSIBILITY
+#endif // QT_CONFIG(accessibility)
