@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qstandardgestures_p.h"
 #include "qgesture.h"
@@ -286,9 +287,9 @@ QGestureRecognizer::Result QSwipeGestureRecognizer::recognize(QGesture *state,
             const QEventPoint &p3 = ev->points().at(2);
 
             if (d->lastPositions[0].isNull()) {
-                d->lastPositions[0] = p1.globalPressPosition().toPoint();
-                d->lastPositions[1] = p2.globalPressPosition().toPoint();
-                d->lastPositions[2] = p3.globalPressPosition().toPoint();
+                d->lastPositions[0] = p1.globalPressPosition();
+                d->lastPositions[1] = p2.globalPressPosition();
+                d->lastPositions[2] = p3.globalPressPosition();
             }
             d->hotSpot = p1.globalPosition();
             d->isHotSpotSet = true;
@@ -311,9 +312,9 @@ QGestureRecognizer::Result QSwipeGestureRecognizer::recognize(QGesture *state,
             static const int directionChangeThreshold = MoveThreshold / 8;
             if (qAbs(xDistance) > MoveThreshold || qAbs(yDistance) > MoveThreshold) {
                 // measure the distance to check if the direction changed
-                d->lastPositions[0] = p1.globalPosition().toPoint();
-                d->lastPositions[1] = p2.globalPosition().toPoint();
-                d->lastPositions[2] = p3.globalPosition().toPoint();
+                d->lastPositions[0] = p1.globalPosition();
+                d->lastPositions[1] = p2.globalPosition();
+                d->lastPositions[2] = p3.globalPosition();
                 result = QGestureRecognizer::TriggerGesture;
                 // QTBUG-46195, small changes in direction should not cause the gesture to be canceled.
                 if (d->verticalDirection == QSwipeGesture::NoDirection || qAbs(yDistance) > directionChangeThreshold) {
@@ -347,8 +348,14 @@ QGestureRecognizer::Result QSwipeGestureRecognizer::recognize(QGesture *state,
                 result = QGestureRecognizer::Ignore;
                 break;
             case QSwipeGesturePrivate::ThreePointsReached:
-                result = (ev->touchPointStates() & QEventPoint::State::Pressed)
-                    ? QGestureRecognizer::CancelGesture : QGestureRecognizer::Ignore;
+                if (ev->touchPointStates() & QEventPoint::State::Pressed) {
+                    result = QGestureRecognizer::CancelGesture;
+                } else if (d->verticalDirection != QSwipeGesture::NoDirection ||
+                           d->horizontalDirection != QSwipeGesture::NoDirection) {
+                    result = QGestureRecognizer::TriggerGesture;
+                } else {
+                    result = QGestureRecognizer::Ignore;
+                }
                 break;
             }
         }
@@ -368,7 +375,7 @@ void QSwipeGestureRecognizer::reset(QGesture *state)
     d->verticalDirection = d->horizontalDirection = QSwipeGesture::NoDirection;
     d->swipeAngle = 0;
 
-    d->lastPositions[0] = d->lastPositions[1] = d->lastPositions[2] = QPoint();
+    d->lastPositions[0] = d->lastPositions[1] = d->lastPositions[2] = QPointF();
     d->state = QSwipeGesturePrivate::NoGesture;
     d->velocityValue = 0;
     d->time.invalidate();
@@ -414,8 +421,8 @@ QGestureRecognizer::Result QTapGestureRecognizer::recognize(QGesture *state,
         const auto ev = static_cast<const QTouchEvent *>(event);
         if (q->state() != Qt::NoGesture && ev->points().size() == 1) {
             const QEventPoint &p = ev->points().at(0);
-            QPoint delta = p.position().toPoint() - p.pressPosition().toPoint();
-            enum { TapRadius = 40 };
+            const QPointF delta = p.position() - p.pressPosition();
+            constexpr auto TapRadius = 40.;
             if (delta.manhattanLength() <= TapRadius) {
                 if (event->type() == QEvent::TouchEnd)
                     result = QGestureRecognizer::FinishGesture;
@@ -475,7 +482,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
         return QGestureRecognizer::FinishGesture | QGestureRecognizer::ConsumeEventHint;
     }
 
-    enum { TapRadius = 40 };
+    constexpr auto TapRadius = 40.;
 
     switch (event->type()) {
 #if QT_CONFIG(graphicsview)
@@ -489,7 +496,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
 #endif
     case QEvent::MouseButtonPress: {
         const QMouseEvent *me = static_cast<const QMouseEvent *>(event);
-        d->position = me->globalPosition().toPoint();
+        d->position = me->globalPosition();
         q->setHotSpot(d->position);
         d->tapAndHoldTimer.start(QTapAndHoldGesturePrivate::Timeout * 1ms, q);
         return QGestureRecognizer::MayBeGesture; // we don't show a sign of life until the timeout
@@ -511,7 +518,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
         const QTouchEvent *ev = static_cast<const QTouchEvent *>(event);
         if (d->tapAndHoldTimer.isActive() && ev->points().size() == 1) {
             const QEventPoint &p = ev->points().at(0);
-            QPoint delta = p.position().toPoint() - p.pressPosition().toPoint();
+            const QPointF delta = p.position() - p.pressPosition();
             if (delta.manhattanLength() <= TapRadius)
                 return QGestureRecognizer::MayBeGesture;
         }
@@ -519,7 +526,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
     }
     case QEvent::MouseMove: {
         const QMouseEvent *me = static_cast<const QMouseEvent *>(event);
-        QPoint delta = me->globalPosition().toPoint() - d->position.toPoint();
+        const QPointF delta = me->globalPosition() - d->position;
         if (d->tapAndHoldTimer.isActive() && delta.manhattanLength() <= TapRadius)
             return QGestureRecognizer::MayBeGesture;
         return QGestureRecognizer::CancelGesture;
@@ -527,7 +534,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
 #if QT_CONFIG(graphicsview)
     case QEvent::GraphicsSceneMouseMove: {
         const QGraphicsSceneMouseEvent *gsme = static_cast<const QGraphicsSceneMouseEvent *>(event);
-        QPoint delta = gsme->screenPos() - d->position.toPoint();
+        const QPointF delta = gsme->screenPos() - d->position;
         if (d->tapAndHoldTimer.isActive() && delta.manhattanLength() <= TapRadius)
             return QGestureRecognizer::MayBeGesture;
         return QGestureRecognizer::CancelGesture;
