@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QObject>
+#include "myobject1.h"
 
 class MyObject2 : public QObject {
     Q_OBJECT
@@ -9,9 +10,11 @@ public:
     MyObject2() = default;
 };
 
-#include "main.moc"
-
 int main()
 {
+    MyObject1 obj1;
+    MyObject2 obj2;
     return 0;
 }
+
+#include "main.moc"
