@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 
@@ -55,26 +30,36 @@ class tst_LargeFile
 {
     Q_OBJECT
 
+    // Member constants:
+    static constexpr int blockSize = 1 << 12;
+#if defined(Q_OS_DARWIN)
+    // HFS+ does not support sparse files, so we limit file size for the test on Mac OS.
+    static constexpr int maxAllowedSizeBits = 24; // 16 MiB
+#elif defined(Q_OS_QNX)
+    // Many of the filesystems that QNX supports use a 32-bit format.
+    // This means that files are limited to 2 GiB − 1 bytes.
+    // Limit max size to 256MiB
+    static constexpr int maxAllowedSizeBits = 28; // 256 MiB
+#elif defined(Q_OS_VXWORKS)
+    // VxWorks doesn't support sparse files, also, default /tmp directory is a RAM-disk which
+    // limits its capacity.
+    static constexpr int maxAllowedSizeBits = 28; // 256 MiB
+#elif defined (Q_OS_WASM)
+    static constexpr int maxAllowedSizeBits = 28; // 256 MiB
+#elif defined(QT_LARGEFILE_SUPPORT)
+    static constexpr int maxAllowedSizeBits = 36; // 64 GiB
+#  define MUST_SET_MAX_SIZE_BITS
+#else
+    static constexpr int maxAllowedSizeBits = 24; // 16 MiB
+#endif
+
 public:
     tst_LargeFile()
-        : blockSize(1 << 12)
-        , maxSizeBits()
-        , fd_(-1)
-        , stream_(0)
+#ifdef MUST_SET_MAX_SIZE_BITS
+        // QEMU only supports < 4GiB files (and maxSizeBits must be a multiple of 4)
+        : maxSizeBits(QTestPrivate::isRunningArmOnX86() ? 28 : maxAllowedSizeBits)
+#endif
     {
-    #if defined(QT_LARGEFILE_SUPPORT) && !defined(Q_OS_MAC)
-        maxSizeBits = 36; // 64 GiB
-    #elif defined(Q_OS_MAC)
-        // HFS+ does not support sparse files, so we limit file size for the test
-        // on Mac OS.
-        maxSizeBits = 24; // 16 MiB
-    #else
-        maxSizeBits = 24; // 16 MiB
-    #endif
-
-        // QEMU only supports < 4GB files
-        if (QTestPrivate::isRunningArmOnX86())
-            maxSizeBits = qMin(maxSizeBits, 28);
     }
 
 private:
@@ -122,15 +107,16 @@ private slots:
     void mapFile_data() { sparseFileData(); }
 
 private:
-    const int blockSize;
-    int maxSizeBits;
+    // Member variables:
 
+    // May be revised down by tests (implying order-dependencies among tests):
+    int maxSizeBits = maxAllowedSizeBits;
     QFile largeFile;
 
     QByteArrayList generatedBlocks;
 
-    int fd_;
-    FILE *stream_;
+    int fd_ = -1;
+    FILE *stream_ = nullptr;
 
     QSharedPointer<QTemporaryDir> m_tempDir;
     QString m_previousCurrent;
@@ -299,7 +285,7 @@ void tst_LargeFile::createSparseFile()
     DWORD bytes;
     if (!::DeviceIoControl(handle, FSCTL_SET_SPARSE, NULL, 0, NULL, 0,
             &bytes, NULL)) {
-        QWARN("Unable to set test file as sparse. "
+        qWarning("Unable to set test file as sparse. "
             "Limiting test file to 16MiB.");
         maxSizeBits = 24;
     }
@@ -347,11 +333,10 @@ void tst_LargeFile::fillFileSparsely()
         {
             if (failed) {
                 this_->maxSizeBits = lastKnownGoodIndex;
-                QWARN( qPrintable(
-                    QString("QFile::error %1: '%2'. Maximum size bits reset to %3.")
-                        .arg(this_->largeFile.error())
-                        .arg(this_->largeFile.errorString())
-                        .arg(this_->maxSizeBits)) );
+                qWarning("QFile::error %d: '%s'. Maximum size bits reset to %d.",
+                        this_->largeFile.error(),
+                        qPrintable(this_->largeFile.errorString()),
+                        this_->maxSizeBits);
             } else
                 lastKnownGoodIndex = qMax<int>(index_, lastKnownGoodIndex);
         }
@@ -495,10 +480,15 @@ void tst_LargeFile::mapFile()
         uchar *address = largeFile.map(position + offset, blockSize - offset);
 
         QVERIFY( address );
-        if ( !std::equal(block.begin() + offset, block.end(), reinterpret_cast<char*>(address)) ) {
-            qDebug() << "Expected:" << block.toHex();
-            qDebug() << "Actual  :" << QByteArray(reinterpret_cast<char*>(address), blockSize).toHex();
-            QVERIFY(false);
+        {
+            auto report = qScopeGuard([block, address, blkSz = blockSize]() {
+                qDebug() << "Expected:" << block.toHex();
+                qDebug() << "Actual  :"
+                         << QByteArray(reinterpret_cast<char *>(address), blkSz).toHex();
+            });
+            QVERIFY(std::equal(block.begin() + offset, block.end(),
+                               reinterpret_cast<char *>(address)));
+            report.dismiss();
         }
 
         QVERIFY( largeFile.unmap( address ) );
@@ -512,21 +502,26 @@ void tst_LargeFile::mapFile()
 //Linux: memory-mapping beyond EOF usually succeeds, but depends on the filesystem
 //  32-bit: limited to 44-bit offsets (when sizeof(off_t) == 8)
 //Windows: memory-mapping beyond EOF is not allowed
+//wasm: as for linux
+//VxWorks: memory-mapping beyond EOF is not allowed
 void tst_LargeFile::mapOffsetOverflow()
 {
-    enum {
-#ifdef Q_OS_WIN
-        Succeeds = false,
-        MaxOffset = 63
+#if defined(Q_OS_WIN)
+    constexpr bool Succeeds = false;
+    constexpr int MaxOffset = 63;
+#elif defined(Q_OS_WASM)
+    constexpr bool Succeeds = true;
+    constexpr int MaxOffset = sizeof(QT_OFF_T) > 4 ? 43 : 30;
+#elif (defined(Q_OS_LINUX) || defined(Q_OS_ANDROID)) && (Q_PROCESSOR_WORDSIZE == 4)
+    constexpr bool Succeeds = true;
+    constexpr int MaxOffset = sizeof(QT_OFF_T) > 4 ? 43 : 30;
+#elif defined(Q_OS_VXWORKS)
+    constexpr bool Succeeds = false;
+    constexpr int MaxOffset = 8 * sizeof(QT_OFF_T) - 1;
 #else
-        Succeeds = true,
-#  if (defined(Q_OS_LINUX) || defined(Q_OS_ANDROID)) && Q_PROCESSOR_WORDSIZE == 4
-        MaxOffset = sizeof(QT_OFF_T) > 4 ? 43 : 30
-#  else
-        MaxOffset = 8 * sizeof(QT_OFF_T) - 1
-#  endif
+    constexpr bool Succeeds = true;
+    constexpr int MaxOffset = 8 * sizeof(QT_OFF_T) - 1;
 #endif
-    };
 
     QByteArray zeroPage(blockSize, '\0');
     for (int i = maxSizeBits + 1; i < 63; ++i) {
@@ -534,13 +529,17 @@ void tst_LargeFile::mapOffsetOverflow()
         uchar *address = 0;
         qint64 offset = Q_INT64_C(1) << i;
 
-        if (succeeds)
-            QTest::ignoreMessage(QtWarningMsg, "QFSFileEngine::map: Mapping a file beyond its size is not portable");
+        if (succeeds) {
+            QTest::ignoreMessage(QtWarningMsg, "QFSFileEngine::map: "
+                                 "Mapping a file beyond its size is not portable");
+        }
         address = largeFile.map(offset, blockSize);
         QCOMPARE(!!address, succeeds);
 
-        if (succeeds)
-            QTest::ignoreMessage(QtWarningMsg, "QFSFileEngine::map: Mapping a file beyond its size is not portable");
+        if (succeeds) {
+            QTest::ignoreMessage(QtWarningMsg, "QFSFileEngine::map: "
+                                 "Mapping a file beyond its size is not portable");
+        }
         address = largeFile.map(offset + blockSize, blockSize);
         QCOMPARE(!!address, succeeds);
     }
@@ -548,4 +547,3 @@ void tst_LargeFile::mapOffsetOverflow()
 
 QTEST_APPLESS_MAIN(tst_LargeFile)
 #include "tst_largefile.moc"
-
