@@ -19,6 +19,7 @@
 #include <qdebug.h>
 #include <qmetaobject.h>
 #include <qscopeguard.h>
+#include <private/qlatch_p.h>
 #include <private/qobject_p.h>
 #include <private/qthread_p.h>
 
@@ -36,6 +37,8 @@
 #include <exception>
 #endif
 
+#include <thread>
+
 #include <QtTest/private/qemulationdetector_p.h>
 
 using namespace std::chrono_literals;
@@ -81,6 +84,8 @@ private slots:
     void connectThreadFinishedSignalToObjectDeleteLaterSlot();
     void wait2();
     void wait3_slowDestructor();
+    void multiThreadWait_data();
+    void multiThreadWait();
     void destroyFinishRace();
     void startFinishRace();
     void startAndQuitCustomEventLoop();
@@ -103,6 +108,8 @@ private slots:
     void terminateSelfStressTest();
 
     void bindingListCleanupAfterDelete();
+
+    void qualityOfService();
 };
 
 enum { one_minute = 60 * 1000, five_minutes = 5 * one_minute };
@@ -138,11 +145,13 @@ class Current_Thread : public QThread
 public:
     Qt::HANDLE id;
     QThread *thread;
+    bool runCalledInCurrentThread = false;
 
     void run() override
     {
         id = QThread::currentThreadId();
         thread = QThread::currentThread();
+        runCalledInCurrentThread = thread->isCurrentThread();
     }
 };
 
@@ -277,6 +286,11 @@ void tst_QThread::currentThreadId()
     QVERIFY(thread.wait(five_minutes));
     QVERIFY(thread.id != nullptr);
     QVERIFY(thread.id != QThread::currentThreadId());
+    QVERIFY(!thread.isCurrentThread());
+    QVERIFY(!thread.thread->isCurrentThread());
+    QVERIFY(thread.QThread::thread()->isCurrentThread());
+    QVERIFY(thread.runCalledInCurrentThread);
+    QVERIFY(qApp->thread()->isCurrentThread());
 }
 
 void tst_QThread::currentThread()
@@ -703,19 +717,18 @@ void noop(void*) { }
 class NativeThreadWrapper
 {
 public:
-    NativeThreadWrapper() : qthread(nullptr), waitForStop(false) {}
+    NativeThreadWrapper() : qthread(nullptr), stopSemaphore(1) {}
     void start(FunctionPointer functionPointer = noop, void *data = nullptr);
     void startAndWait(FunctionPointer functionPointer = noop, void *data = nullptr);
     void join();
-    void setWaitForStop() { waitForStop = true; }
+    void setWaitForStop() { stopSemaphore.acquire(); }
     void stop();
 
     ThreadHandle nativeThreadHandle;
     QThread *qthread;
-    QWaitCondition startCondition;
-    QMutex mutex;
-    bool waitForStop;
-    QWaitCondition stopCondition;
+    QSemaphore startSemaphore;
+    QSemaphore stopSemaphore;
+
 protected:
     static void *runUnix(void *data);
     static unsigned WIN_FIX_STDCALL runWin(void *data);
@@ -739,9 +752,8 @@ void NativeThreadWrapper::start(FunctionPointer functionPointer, void *data)
 
 void NativeThreadWrapper::startAndWait(FunctionPointer functionPointer, void *data)
 {
-    QMutexLocker locker(&mutex);
     start(functionPointer, data);
-    startCondition.wait(locker.mutex());
+    startSemaphore.acquire();
 }
 
 void NativeThreadWrapper::join()
@@ -762,20 +774,13 @@ void *NativeThreadWrapper::runUnix(void *that)
     nativeThreadWrapper->qthread = QThread::currentThread();
 
     // Release main thread.
-    {
-        QMutexLocker lock(&nativeThreadWrapper->mutex);
-        nativeThreadWrapper->startCondition.wakeOne();
-    }
+    nativeThreadWrapper->startSemaphore.release();
 
     // Run function.
     nativeThreadWrapper->functionPointer(nativeThreadWrapper->data);
 
     // Wait for stop.
-    {
-        QMutexLocker lock(&nativeThreadWrapper->mutex);
-        if (nativeThreadWrapper->waitForStop)
-            nativeThreadWrapper->stopCondition.wait(lock.mutex());
-    }
+    nativeThreadWrapper->stopSemaphore.acquire();
 
     return nullptr;
 }
@@ -788,9 +793,7 @@ unsigned WIN_FIX_STDCALL NativeThreadWrapper::runWin(void *data)
 
 void NativeThreadWrapper::stop()
 {
-    QMutexLocker lock(&mutex);
-    waitForStop = false;
-    stopCondition.wakeOne();
+    stopSemaphore.release();
 }
 
 static bool threadAdoptedOk = false;
@@ -980,13 +983,11 @@ void tst_QThread::adoptMultipleThreadsOverlap()
     for (int i = 0; i < numThreads; ++i) {
         nativeThreads.append(new NativeThreadWrapper());
         nativeThreads.at(i)->setWaitForStop();
-        nativeThreads.at(i)->mutex.lock();
         nativeThreads.at(i)->start();
     }
     for (int i = 0; i < numThreads; ++i) {
-        nativeThreads.at(i)->startCondition.wait(&nativeThreads.at(i)->mutex);
+        nativeThreads.at(i)->startSemaphore.acquire();
         QObject::connect(nativeThreads.at(i)->qthread, SIGNAL(finished()), &recorder, SLOT(slot()));
-        nativeThreads.at(i)->mutex.unlock();
     }
 
     QObject::connect(nativeThreads.at(numThreads - 1)->qthread, SIGNAL(finished()), &QTestEventLoop::instance(), SLOT(exitLoop()));
@@ -1010,7 +1011,7 @@ void tst_QThread::adoptedThreadBindingStatus()
     nativeThread.startAndWait();
     QVERIFY(nativeThread.qthread);
     auto privThread = static_cast<QThreadPrivate *>(QObjectPrivate::get(nativeThread.qthread));
-    QVERIFY(privThread->m_statusOrPendingObjects.bindingStatus());
+    QVERIFY(privThread->data->m_statusOrPendingObjects.bindingStatus());
 
     nativeThread.stop();
     nativeThread.join();
@@ -1190,6 +1191,193 @@ void tst_QThread::wait3_slowDestructor()
     QVERIFY(thread.wait(one_minute));
 }
 
+void tst_QThread::multiThreadWait_data()
+{
+    QTest::addColumn<QList<int>>("deadlines");
+    auto addRow = [](auto &&... list) {
+        static_assert(sizeof...(list) <= 5,
+                "Limited by std::array in tst_QThread::multiThreadWait()");
+        QList<int> deadlines = { std::move(list)... };
+        QByteArrayList name;
+        for (int value : deadlines) {
+            if (value < 0)
+                name.append("Forever");
+            else
+                name.append(QByteArray::number(value));
+        }
+        QTest::newRow(name.join('-').constData()) << deadlines;
+    };
+
+    // control
+    addRow(-1);
+    addRow(0);
+    addRow(25);
+    addRow(250);
+
+    addRow(0, 0);
+    addRow(0, 0, 0, 0, 0);
+    addRow(-1, -1);
+    addRow(-1, -1, -1, -1, -1);
+
+    // this is probably too fast and the Forever gets in too quickly
+    addRow(0, -1);
+
+    // any positive time below 100ms (see below) is expected to timeout
+    addRow(25, -1);
+    addRow(25, 50, -1);
+    addRow(50, 25, -1);
+    addRow(-1, 25, 25, 25);
+    addRow(25, 2000);
+    addRow(25, 2000, 25, -1);
+}
+
+void tst_QThread::multiThreadWait()
+{
+    static constexpr auto TimeoutThreshold = 100ms;
+    auto isExpectedToTimeout = [](unsigned value) {
+        return value < TimeoutThreshold.count();
+    };
+
+    class TargetThread : public QThread {
+    public:
+        QSemaphore sync;
+        void run() override
+        {
+            sync.acquire();
+        }
+    };
+
+    // Design of this test:
+    //
+    // The WaiterThread class is used to test both threads that time out during
+    // QThread::wait() and those that succeed. Both the WaiterThread and the
+    // main thread operate two QElapsedTimers, each before and after the
+    // arriveAndWait() calls. The WaiterThread QElapsedTimer started after
+    // arriveAndWait() measures the time around QThread::wait().
+    //
+    // To avoid using QThread::wait() to wait on a thread that itself doing
+    // QThread::wait() in a test that is testing exactly that function, the
+    // main thread waits for the WaiterThreads first on a QSemaphore. If those
+    // tryAcquire() fail, there's a bug in QThread::wait().
+    //
+    // For wait() calls that are expected to timeout, the WaiterThread's
+    // QElapsedTimer around the wait() call will be compared against the
+    // QElapsedTimer from the main thread which was started before any thread
+    // was started (read: the main thread's starting time point is always
+    // earlier than that of the WaiterThreads). The main thread then waits for
+    // the WaiterThreads that time out, before ending measuring the
+    // QElapsedTimers. This implies the time measured around the
+    // QThread::wait() must strictly less than the time the main thread's
+    // QElapsedTimer.
+    //
+    // After this, the main thread causes the TargetThread to exit, which
+    // should cause QThread::wait() to return with success in the remaining
+    // WaiterThread.
+    //
+    // For wait() calls that are expected to succeed, the time measured around
+    // the QThread::wait() must be strictly less than or equal to the original
+    // timeout (though see note below). Additionally, we verify that
+    // QThread::wait() didn't return too soon (potentially before TargetThread
+    // had actually exited) by comparing the thread's total time to the
+    // QElapsedTimer on the main thread that was started after the
+    // arriveAndWait() (read: starting time point is strictly later than that
+    // of the WaiterThreads'). The threads must have waited a time greater than
+    // or equal to this time.
+    //
+    // Note on race condition: there's one race condition on a successful
+    // non-forever wait. It is possible the deadline calculated by the main
+    // thread has come and gone before QThread::wait() was called, meaning it
+    // is called with a deadline in the past and will thus return with failure.
+    // We mitigate this by only using a timeouts for success much greater than
+    // the expected runtime of the test (2s vs ~10 ms).
+    class WaiterThread : public QThread {
+    public:
+        QLatch *barrier;
+        QSemaphore *endSema;
+        QThread *target;
+        QDeadlineTimer deadline;
+        QElapsedTimer::Duration totalDuration = {};
+        QElapsedTimer::Duration waitedDuration = {};
+        int result = -1;
+        void run() override
+        {
+            QElapsedTimer total, waitOnly;
+            total.start();
+            barrier->arriveAndWait();
+            waitOnly.start();
+            result = target->wait(deadline);
+            waitedDuration = waitOnly.durationElapsed();
+            totalDuration = total.durationElapsed();
+            endSema->release();
+        }
+    };
+
+    QFETCH(QList<int>, deadlines);
+    TargetThread target;
+    target.start();
+
+    QLatch barrier(deadlines.size() + 1);      // plus the main thread
+    QSemaphore timeoutSema, successSema;
+    std::array<std::unique_ptr<WaiterThread>, 5> threads;   // 5 threads is enough
+    int expectedTimeoutCount = 0;
+    for (int i = 0; i < deadlines.size(); ++i) {
+        threads[i] = std::make_unique<WaiterThread>();
+        threads[i]->barrier = &barrier;
+        if (isExpectedToTimeout(deadlines.at(i))) {
+            ++expectedTimeoutCount;
+            threads[i]->endSema = &timeoutSema;
+        } else {
+            threads[i]->endSema = &successSema;
+        }
+        threads[i]->target = &target;
+    }
+    for (int i = 0; i < deadlines.size(); ++i)
+        threads[i]->deadline = QDeadlineTimer(deadlines.at(i));
+
+    // start the threads and synchronize everyone
+    QElapsedTimer timeoutTimer, waitTimer;
+    timeoutTimer.start();
+    for (int i = 0; i < deadlines.size(); ++i)
+        threads[i]->start();
+    barrier.arriveAndWait();
+    waitTimer.start();
+
+    // then wait for the threads that are expected to timeout to do so
+    QVERIFY(timeoutSema.tryAcquire(expectedTimeoutCount, QDeadlineTimer::Forever));
+
+    // compute the elapsed time for timing comparisons
+    std::this_thread::sleep_for(5ms);   // short, but long enough to avoid rounding errors
+    auto waitElapsed = waitTimer.durationElapsed();
+    auto timeoutElapsed = timeoutTimer.durationElapsed();
+    std::this_thread::sleep_for(5ms);
+
+    // cause the target thread to exit, so the successful threads do succeed
+    target.sync.release();
+    int expectedSuccessCount = deadlines.size() - expectedTimeoutCount;
+    QVERIFY(successSema.tryAcquire(expectedSuccessCount, QDeadlineTimer::Forever));
+
+    // wait for all the threads to end, before QVERIFY/QCOMPAREs
+    for (int i = 0; i < deadlines.size(); ++i)
+        threads[i]->wait();
+    target.wait();
+
+    for (int i = 0; i < deadlines.size(); ++i) {
+        auto printI = qScopeGuard([i] { qWarning("i = %i", i); });
+        if (isExpectedToTimeout(deadlines.at(i))) {
+            QCOMPARE_LT(threads[i]->waitedDuration, timeoutElapsed);
+            QCOMPARE(threads[i]->result, false);
+        } else {
+            // if it was a success, it must have waited less than the deadline
+            if (deadlines.at(i) >= 0)
+                QCOMPARE_LE(threads[i]->waitedDuration, deadlines.at(i) * 1ms);
+            QCOMPARE_GE(threads[i]->totalDuration, waitElapsed);
+            QCOMPARE(threads[i]->result, true);
+        }
+        printI.dismiss();
+        threads[i].reset();
+    }
+}
+
 void tst_QThread::destroyFinishRace()
 {
     class Thread : public QThread { void run() override {} };
@@ -1281,9 +1469,10 @@ void tst_QThread::isRunningInFinished()
     }
 }
 
-class DummyEventDispatcher : public QAbstractEventDispatcher {
+class DummyEventDispatcher : public QAbstractEventDispatcherV2
+{
+    Q_OBJECT
 public:
-    DummyEventDispatcher() : QAbstractEventDispatcher() {}
     bool processEvents(QEventLoop::ProcessEventsFlags) override {
         visited.storeRelaxed(true);
         emit awake();
@@ -1292,11 +1481,19 @@ public:
     }
     void registerSocketNotifier(QSocketNotifier *) override {}
     void unregisterSocketNotifier(QSocketNotifier *) override {}
-    void registerTimer(int, qint64, Qt::TimerType, QObject *) override {}
-    bool unregisterTimer(int) override { return false; }
+    void registerTimer(Qt::TimerId id, Duration, Qt::TimerType, QObject *) override
+    {
+        if (registeredTimerId <= Qt::TimerId::Invalid)
+            registeredTimerId = id;
+    }
+    bool unregisterTimer(Qt::TimerId id) override
+    {
+        Qt::TimerId oldId = std::exchange(registeredTimerId, Qt::TimerId::Invalid);
+        return id == oldId;
+    }
     bool unregisterTimers(QObject *) override { return false; }
-    QList<TimerInfo> registeredTimers(QObject *) const override { return QList<TimerInfo>(); }
-    int remainingTime(int) override { return 0; }
+    QList<TimerInfoV2> timersForObject(QObject *) const override { return {}; }
+    Duration remainingTime(Qt::TimerId) const override { return 0s; }
     void wakeUp() override {}
     void interrupt() override {}
 
@@ -1306,25 +1503,47 @@ public:
 #endif
 
     QBasicAtomicInt visited; // bool
+    Qt::TimerId registeredTimerId = Qt::TimerId::Invalid;
 };
 
-class ThreadObj : public QObject
+struct ThreadLocalContent
 {
-    Q_OBJECT
-public slots:
-    void visit() {
-        emit visited();
+    static inline const QMetaObject *atStart;
+    static inline const QMetaObject *atEnd;
+    QSemaphore *sem;
+    QBasicTimer timer;
+
+    ThreadLocalContent(QObject *obj, QSemaphore *sem)
+        : sem(sem)
+    {
+        ensureEventDispatcher();
+        atStart = QAbstractEventDispatcher::instance()->metaObject();
+        timer.start(10s, obj);
+    }
+    ~ThreadLocalContent()
+    {
+        ensureEventDispatcher();
+        atEnd = QAbstractEventDispatcher::instance()->metaObject();
+        timer.stop();
+        sem->release();
+    }
+
+    void ensureEventDispatcher()
+    {
+        // QEventLoop's constructor has a call to QThreadData::ensureEventDispatcher()
+        QEventLoop dummy;
     }
-signals:
-    void visited();
 };
 
 void tst_QThread::customEventDispatcher()
 {
+    ThreadLocalContent::atStart = ThreadLocalContent::atEnd = nullptr;
+
     QThread thr;
     // there should be no ED yet
     QVERIFY(!thr.eventDispatcher());
     DummyEventDispatcher *ed = new DummyEventDispatcher;
+    QPointer<DummyEventDispatcher> weak_ed(ed);
     thr.setEventDispatcher(ed);
     // the new ED should be set
     QCOMPARE(thr.eventDispatcher(), ed);
@@ -1333,25 +1552,40 @@ void tst_QThread::customEventDispatcher()
     thr.start();
     // start() should not overwrite the ED
     QCOMPARE(thr.eventDispatcher(), ed);
+    QVERIFY(!weak_ed.isNull());
 
-    ThreadObj obj;
+    QObject obj;
     obj.moveToThread(&thr);
     // move was successful?
     QCOMPARE(obj.thread(), &thr);
-    QEventLoop loop;
-    connect(&obj, SIGNAL(visited()), &loop, SLOT(quit()), Qt::QueuedConnection);
-    QMetaObject::invokeMethod(&obj, "visit", Qt::QueuedConnection);
-    loop.exec();
+
+    QSemaphore threadLocalSemaphore;
+    QMetaObject::invokeMethod(&obj, [&]() {
+#if !QT_CONFIG(broken_threadlocal_dtors)
+        // On Windows, the thread_locals are unsequenced between DLLs, so this
+        // could run after QThreadPrivate::finish().
+        // On Unix, QThread doesn't use thread_local if support is broken.
+        static thread_local
+#endif
+                ThreadLocalContent d(&obj, &threadLocalSemaphore);
+    }, Qt::BlockingQueuedConnection);
+
     // test that the ED has really been used
     QVERIFY(ed->visited.loadRelaxed());
+    // and it's ours
+    QCOMPARE(ThreadLocalContent::atStart->className(), "DummyEventDispatcher");
 
-    QPointer<DummyEventDispatcher> weak_ed(ed);
     QVERIFY(!weak_ed.isNull());
     thr.quit();
+
     // wait for thread to be stopped
     QVERIFY(thr.wait(30000));
+    QVERIFY(threadLocalSemaphore.tryAcquire(1, 30s));
+
     // test that ED has been deleted
     QVERIFY(weak_ed.isNull());
+    // test that ED was ours
+    QCOMPARE(ThreadLocalContent::atEnd->className(), "DummyEventDispatcher");
 }
 
 class Job : public QObject
@@ -1899,7 +2133,6 @@ void tst_QThread::terminateSelfStressTest()
     struct Thread : QThread {
         void run() override {
             terminate();
-            while (true) sleep(1ns); // QTBUG-127008
         }
     };
 
@@ -1937,11 +2170,39 @@ void tst_QThread::bindingListCleanupAfterDelete()
     auto optr = std::make_unique<QObject>();
     optr->moveToThread(&t);
     auto threadPriv =  static_cast<QThreadPrivate *>(QObjectPrivate::get(&t));
-    auto list = threadPriv->m_statusOrPendingObjects.list();
+    auto list = threadPriv->data->m_statusOrPendingObjects.list();
     QVERIFY(list);
     optr.reset();
     QVERIFY(list->empty());
 }
 
+void tst_QThread::qualityOfService()
+{
+    QThread th;
+    QThread::currentThread()->setObjectName("Main thread");
+    th.setObjectName("test thread");
+    auto guard = qScopeGuard([&th](){ th.quit(); th.wait(); });
+    QCOMPARE(th.serviceLevel(), QThread::QualityOfService::Auto);
+    th.setServiceLevel(QThread::QualityOfService::High);
+    QCOMPARE(th.serviceLevel(), QThread::QualityOfService::High);
+    th.setServiceLevel(QThread::QualityOfService::Eco);
+    QCOMPARE(th.serviceLevel(), QThread::QualityOfService::Eco);
+
+    th.start();
+    auto obj = std::make_unique<QObject>();
+    obj->moveToThread(&th);
+
+    QThread::QualityOfService launchedThreadServiceLevel = {};
+    QMetaObject::invokeMethod(obj.get(), [](){
+        return QThread::currentThread()->serviceLevel();
+    }, Qt::BlockingQueuedConnection, qReturnArg(launchedThreadServiceLevel));
+
+    QCOMPARE(launchedThreadServiceLevel, QThread::QualityOfService::Eco);
+
+    QMetaObject::invokeMethod(obj.get(), [](){
+        QThread::currentThread()->setServiceLevel(QThread::QualityOfService::High);
+    }, Qt::BlockingQueuedConnection);
+}
+
 QTEST_MAIN(tst_QThread)
 #include "tst_qthread.moc"
