@@ -1,37 +1,15 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
-#include <QtTest/QtTest>
+#include <QTest>
+#include <QBuffer>
 
 #include <qcoreapplication.h>
 #include <qdebug.h>
+#if QT_CONFIG(settings)
 #include <qsettings.h>
+#endif
 #include <qtextformat.h>
 #include <private/qtextformat_p.h>
 #include <qtextdocument.h>
@@ -51,7 +29,9 @@ Q_OBJECT
 private slots:
     void getSetCheck();
     void defaultAlignment();
+#if QT_CONFIG(settings)
     void testQTextCharFormat() const;
+#endif
     void testUnderlinePropertyPrecedence();
     void toFormat();
     void resolveFont();
@@ -65,12 +45,15 @@ private slots:
     void setFont_collection_data();
     void setFont_collection();
     void clearCollection();
+    void setFontFeatures();
+    void setFontVariableAxes();
 
 #ifndef QT_NO_DATASTREAM
     void dataStreamCompatibility();
 #endif
 };
 
+#if QT_CONFIG(settings)
 /*! \internal
   This (used to) trigger a crash in:
 
@@ -85,6 +68,7 @@ void tst_QTextFormat::testQTextCharFormat() const
 
     settings.value("test", test);
 }
+#endif
 
 // Testing get/set functions
 void tst_QTextFormat::getSetCheck()
@@ -211,7 +195,7 @@ void tst_QTextFormat::resolveFont()
     QTextCursor(&doc).insertText("Test", fmt);
 
     QList<QTextFormat> formats = doc.allFormats();
-    QCOMPARE(formats.count(), 3);
+    QCOMPARE(formats.size(), 3);
 
     QCOMPARE(formats.at(2).type(), int(QTextFormat::CharFormat));
     fmt = formats.at(2).toCharFormat();
@@ -336,8 +320,8 @@ void tst_QTextFormat::getSetTabs()
       public:
         Comparator(const QList<QTextOption::Tab> &tabs, const QList<QTextOption::Tab> &tabs2)
         {
-            QCOMPARE(tabs.count(), tabs2.count());
-            for(int i=0; i < tabs.count(); i++) {
+            QCOMPARE(tabs.size(), tabs2.size());
+            for(int i=0; i < tabs.size(); i++) {
                 QTextOption::Tab t1 = tabs[i];
                 QTextOption::Tab t2 = tabs2[i];
                 QCOMPARE(t1.position, t2.position);
@@ -388,7 +372,7 @@ void tst_QTextFormat::testTabsUsed()
     QCOMPARE(line.cursorToX(4), 100.);
 
     QTextOption option = layout->textOption();
-    QCOMPARE(option.tabs().count(), tabs.count());
+    QCOMPARE(option.tabs().size(), tabs.size());
 
 }
 
@@ -672,19 +656,102 @@ void tst_QTextFormat::clearCollection()
     charFormat2.setUnderlineStyle(QTextCharFormat::SingleUnderline);
     int formatIndex2 = collection.indexForFormat(charFormat2);
     QCOMPARE(formatIndex2, 1);
-    QCOMPARE(collection.formats.count(), 2);
-    QCOMPARE(collection.hashes.count(), 2);
+    QCOMPARE(collection.formats.size(), 2);
+    QCOMPARE(collection.hashes.size(), 2);
     QCOMPARE(collection.defaultFont(), f);
 
     collection.clear();
-    QCOMPARE(collection.formats.count(), 0);
-    QCOMPARE(collection.hashes.count(), 0);
+    QCOMPARE(collection.formats.size(), 0);
+    QCOMPARE(collection.hashes.size(), 0);
     QCOMPARE(collection.indexForFormat(charFormat2), 0);
-    QCOMPARE(collection.formats.count(), 1);
-    QCOMPARE(collection.hashes.count(), 1);
+    QCOMPARE(collection.formats.size(), 1);
+    QCOMPARE(collection.hashes.size(), 1);
     QCOMPARE(collection.defaultFont(), f); // kept, QTextDocument::clear or setPlainText should not reset the font set by setDefaultFont
 }
 
+void tst_QTextFormat::setFontFeatures()
+{
+    {
+        QFont font;
+        font.setFeature("abcd", 1234);
+        font.setFeature("efgh", 5678);
+
+        QTextCharFormat format;
+        format.setFont(font);
+
+        QFont resolvedFont = format.font();
+        QCOMPARE(resolvedFont.featureTags().size(), 2);
+        QCOMPARE(resolvedFont.featureValue("abcd"), 1234);
+        QCOMPARE(resolvedFont.featureValue("efgh"), 5678);
+
+        const auto features = format.fontFeatures();
+        QCOMPARE(features.size(), 2);
+        QCOMPARE(features.value("abcd"), 1234);
+        QCOMPARE(features.value("efgh"), 5678);
+    }
+
+    {
+        QTextCharFormat format;
+
+        QMap<QFont::Tag, quint32> features;
+        features.insert("abcd", 4321);
+        features.insert("efgh", 8765);
+        format.setFontFeatures(features);
+
+        QFont resolvedFont = format.font();
+        QCOMPARE(resolvedFont.featureTags().size(), 2);
+        QCOMPARE(resolvedFont.featureValue("abcd"), 4321);
+        QCOMPARE(resolvedFont.featureValue("efgh"), 8765);
+
+        features = format.fontFeatures();
+        QCOMPARE(features.size(), 2);
+        QCOMPARE(features.value("abcd"), 4321);
+        QCOMPARE(features.value("efgh"), 8765);
+    }
+}
+
+void tst_QTextFormat::setFontVariableAxes()
+{
+    {
+        QFont font;
+        font.setVariableAxis("abcd", 12.25);
+        font.setVariableAxis("efgh", 13.25);
+
+        QTextCharFormat format;
+        format.setFont(font);
+
+        QFont resolvedFont = format.font();
+        QCOMPARE(resolvedFont.variableAxisTags().size(), 2);
+        QCOMPARE(resolvedFont.variableAxisValue("abcd"), 12.25);
+        QCOMPARE(resolvedFont.variableAxisValue("efgh"), 13.25);
+
+        const auto axes = format.fontVariableAxes();
+        QCOMPARE(axes.size(), 2);
+        QCOMPARE(axes.value("abcd"), 12.25);
+        QCOMPARE(axes.value("efgh"), 13.25);
+    }
+
+    {
+        QTextCharFormat format;
+
+        QMap<QFont::Tag, float> axes;
+        axes.insert("abcd", 12.25);
+        axes.insert("efgh", 13.25);
+        format.setFontVariableAxes(axes);
+
+        QFont resolvedFont = format.font();
+        QCOMPARE(resolvedFont.variableAxisTags().size(), 2);
+        QCOMPARE(resolvedFont.variableAxisValue("abcd"), 12.25);
+        QCOMPARE(resolvedFont.variableAxisValue("efgh"), 13.25);
+
+        axes = format.fontVariableAxes();
+        QCOMPARE(axes.size(), 2);
+        QCOMPARE(axes.value("abcd"), 12.25);
+        QCOMPARE(axes.value("efgh"), 13.25);
+    }
+
+}
+
 #ifndef QT_NO_DATASTREAM
 void tst_QTextFormat::dataStreamCompatibility()
 {
@@ -693,14 +760,17 @@ void tst_QTextFormat::dataStreamCompatibility()
     QTextCharFormat format;
     format.setFontStretch(42);
     format.setFontLetterSpacingType(QFont::AbsoluteSpacing);
+    format.setFontFamilies({QLatin1String("Arial")});
 
     // Sanity check
     {
         QMap<int, QVariant> properties = format.properties();
         QVERIFY(properties.contains(QTextFormat::FontLetterSpacingType));
         QVERIFY(properties.contains(QTextFormat::FontStretch));
+        QVERIFY(properties.contains(QTextFormat::FontFamilies));
         QVERIFY(!properties.contains(QTextFormat::OldFontLetterSpacingType));
         QVERIFY(!properties.contains(QTextFormat::OldFontStretch));
+        QVERIFY(!properties.contains(QTextFormat::OldFontFamily));
     }
 
     QByteArray memory;
@@ -728,8 +798,10 @@ void tst_QTextFormat::dataStreamCompatibility()
                 QMap<int, QVariant> properties = other.properties();
                 QVERIFY(properties.contains(QTextFormat::FontLetterSpacingType));
                 QVERIFY(properties.contains(QTextFormat::FontStretch));
+                QVERIFY(properties.contains(QTextFormat::FontFamilies));
                 QVERIFY(!properties.contains(QTextFormat::OldFontLetterSpacingType));
                 QVERIFY(!properties.contains(QTextFormat::OldFontStretch));
+                QVERIFY(!properties.contains(QTextFormat::OldFontFamily));
             }
         }
 
@@ -746,8 +818,10 @@ void tst_QTextFormat::dataStreamCompatibility()
             stream >> properties;
             QVERIFY(properties.contains(QTextFormat::FontLetterSpacingType));
             QVERIFY(properties.contains(QTextFormat::FontStretch));
+            QVERIFY(properties.contains(QTextFormat::FontFamilies));
             QVERIFY(!properties.contains(QTextFormat::OldFontLetterSpacingType));
             QVERIFY(!properties.contains(QTextFormat::OldFontStretch));
+            QVERIFY(!properties.contains(QTextFormat::OldFontFamily));
         }
     }
 
@@ -777,8 +851,10 @@ void tst_QTextFormat::dataStreamCompatibility()
                 QMap<int, QVariant> properties = other.properties();
                 QVERIFY(properties.contains(QTextFormat::FontLetterSpacingType));
                 QVERIFY(properties.contains(QTextFormat::FontStretch));
+                QVERIFY(properties.contains(QTextFormat::FontFamilies));
                 QVERIFY(!properties.contains(QTextFormat::OldFontLetterSpacingType));
                 QVERIFY(!properties.contains(QTextFormat::OldFontStretch));
+                QVERIFY(!properties.contains(QTextFormat::OldFontFamily));
             }
         }
 
@@ -797,11 +873,80 @@ void tst_QTextFormat::dataStreamCompatibility()
             stream >> properties;
             QVERIFY(!properties.contains(QTextFormat::FontLetterSpacingType));
             QVERIFY(!properties.contains(QTextFormat::FontStretch));
+            QVERIFY(!properties.contains(QTextFormat::FontFamilies));
             QVERIFY(properties.contains(QTextFormat::OldFontLetterSpacingType));
             QVERIFY(properties.contains(QTextFormat::OldFontStretch));
+            QVERIFY(properties.contains(QTextFormat::OldFontFamily));
         }
     }
 
+    // Don't mix up FontFeatures and OldTextUnderlineColor
+    memory.clear();
+    {
+        {
+            QBuffer buffer(&memory);
+            buffer.open(QIODevice::WriteOnly);
+
+            QFont font;
+            font.setFeature("abcd", 1234);
+
+            QTextCharFormat format;
+            format.setFont(font);
+
+            QDataStream stream(&buffer);
+
+            stream << format;
+        }
+
+        {
+            QBuffer buffer(&memory);
+            buffer.open(QIODevice::ReadOnly);
+
+            QDataStream stream(&buffer);
+
+            QTextFormat other;
+            stream >> other;
+
+            QMap<int, QVariant> properties = other.properties();
+            QVERIFY(properties.contains(QTextFormat::FontFeatures));
+
+            auto features = other.property(QTextFormat::FontFeatures).value<QMap<QFont::Tag, quint32>>();
+            QCOMPARE(features.value("abcd"), 1234);
+        }
+    }
+
+    memory.clear();
+    {
+        {
+            QBuffer buffer(&memory);
+            buffer.open(QIODevice::WriteOnly);
+
+            QFont font;
+            font.setFeature("abcd", 1234);
+
+            QTextCharFormat format;
+            format.setFont(font);
+
+            QDataStream stream(&buffer);
+            stream.setVersion(QDataStream::Qt_5_15);
+
+            stream << format;
+        }
+
+        {
+            QBuffer buffer(&memory);
+            buffer.open(QIODevice::ReadOnly);
+
+            QDataStream stream(&buffer);
+            stream.setVersion(QDataStream::Qt_5_15);
+
+            QTextFormat other;
+            stream >> other;
+            QMap<int, QVariant> properties = other.properties();
+            QVERIFY(!properties.contains(QTextFormat::FontFeatures));
+            QVERIFY(!properties.contains(QTextFormat::OldTextUnderlineColor));
+        }
+    }
 }
 #endif // QT_NO_DATASTREAM
 
