@@ -37,6 +37,7 @@ TESTS = ['assert', 'badxml', 'benchlibcallgrind', 'benchlibcounting',
          'fetchbogus', 'findtestdata', 'float', 'globaldata', 'longstring',
          'maxwarnings', 'mouse', 'multiexec', 'pairdiagnostics', 'pass',
          'printdatatags', 'printdatatagswithglobaltags', 'qexecstringlist',
+         'selected',
          'signaldumper', 'silent', 'silent_fatal', 'singleskip', 'skip',
          'skipblacklisted', 'skipcleanup', 'skipcleanuptestcase', 'skipinit',
          'skipinitdata',
@@ -292,6 +293,7 @@ def shouldIgnoreTest(testname, format):
                         "benchliboptions",
                         "printdatatags",
                         "printdatatagswithglobaltags",
+                        "selected",
                         "silent",
                         "silent_fatal",
                         "crashes",
