@@ -19,6 +19,7 @@
 #if QT_CONFIG(process)
 # include <QtCore/QProcess>
 #endif
+#include <QtCore/QSettings>
 #include <QtCore/private/qeventloop_p.h>
 
 #include <QtGui/QFontDatabase>
@@ -262,6 +263,7 @@ QT_WARNING_POP
         }
     };
 
+    QApplication::sendPostedEvents();
     EventWatcher watcher;
 
     QCOMPARE(watcher.palette_changed, 0);
@@ -272,11 +274,13 @@ QT_WARNING_POP
     qApp->setFont(font);
     QApplication::processEvents();
 #if QT_DEPRECATED_SINCE(6, 0)
+    // signal, plus events delivered to qApp and QStyleHints
+    QCOMPARE(watcher.palette_changed, 3);
+    QCOMPARE(watcher.font_changed, 3);
+#else
+    // events delivered to qApp and QStyleHints
     QCOMPARE(watcher.palette_changed, 2);
     QCOMPARE(watcher.font_changed, 2);
-#else
-    QCOMPARE(watcher.palette_changed, 1);
-    QCOMPARE(watcher.font_changed, 1);
 #endif
 }
 
@@ -2393,7 +2397,7 @@ void tst_QApplication::wheelEventPropagation()
 
     auto innerVBar = innerArea.verticalScrollBar();
     innerVBar->setObjectName("innerArea_vbar");
-    QCOMPARE(innerVBar->isVisible(), innerScrolls);
+    QTRY_COMPARE(innerVBar->isVisible(), innerScrolls);
     auto innerHBar = innerArea.horizontalScrollBar();
     innerHBar->setObjectName("innerArea_hbar");
     QCOMPARE(innerHBar->isVisible(), innerScrolls);
