@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -31,6 +31,7 @@
 #include <QtWidgets/private/qapplication_p.h>
 
 using namespace QTestPrivate;
+using namespace Qt::StringLiterals;
 
 // Compare a window position that may go through scaling in the platform plugin with fuzz.
 static inline bool qFuzzyCompareWindowPosition(const QPoint &p1, const QPoint p2, int fuzz)
@@ -111,8 +112,13 @@ private slots:
     void mouseMoveWithPopup_data();
     void mouseMoveWithPopup();
 
+    void showHideWindowHandle_data();
+    void showHideWindowHandle();
+
     void resetFocusObjectOnDestruction();
 
+    void cleanupOnDestruction();
+
 private:
     QSize m_testWidgetSize;
     const int m_fuzz;
@@ -504,17 +510,24 @@ void tst_QWidget_window::tst_paintEventOnResize_QTBUG50796()
 
     PaintTestWidget *native = new PaintTestWidget(&root);
     native->winId(); // We're testing native widgets
-    native->setGeometry(10, 10, 50, 50);
+    native->setGeometry(50, 50, 50, 50); // position it outside of potentially unsafe area
 
     root.show();
     QVERIFY(QTest::qWaitForWindowExposed(&root));
     QVERIFY(QTest::qWaitForWindowActive(&root));
     QVERIFY(native->isVisible());
 
+    // The top level widget can receive new margins after show
+    // if the size is big enough to end up overlapping safe areas.
+    int safeMarginsResizeCount = 0;
+    connect(root.windowHandle(), &QWindow::safeAreaMarginsChanged, this, [&]() {
+        ++safeMarginsResizeCount;
+    });
+
     native->paintEventCount = 0;
     native->resize(native->width() + 10, native->height() + 10);
     QTest::qWait(50); // Wait for paint events
-    QTRY_COMPARE(native->paintEventCount, 1); // Only one paint event must occur
+    QTRY_COMPARE(native->paintEventCount - safeMarginsResizeCount, 1); // Only one paint event must occur
 }
 
 #if QT_CONFIG(draganddrop)
@@ -686,7 +699,6 @@ void tst_QWidget_window::tst_dnd()
 
     dndTestWidget.show();
     QVERIFY(QTest::qWaitForWindowExposed(&dndTestWidget));
-    QApplicationPrivate::setActiveWindow(&dndTestWidget);
     QVERIFY(QTest::qWaitForWindowActive(&dndTestWidget));
 
     QMimeData mimeData;
@@ -801,6 +813,8 @@ void tst_QWidget_window::tst_dnd_events()
     // catch regressions at cross platform code: QGuiApplication::processDrag/Leave().
     if (platformName != "xcb")
         return;
+    if (qgetenv("XDG_CURRENT_DESKTOP").toLower().contains("ubuntu") && QSysInfo::productVersion() == "24.04"_L1)
+        QSKIP("This hangs on Ubuntu 24.04 X11, see also QTBUG-129567.");
 
     const QString expectedDndEvents = "DragEnter DragMove DropEvent DragEnter DragMove "
                                       "DropEvent DragEnter DragMove DropEvent ";
@@ -821,7 +835,8 @@ void tst_QWidget_window::tst_dnd_events()
 
     // Some dnd implementation rely on running internal event loops, so we have to use
     // the following queued signal hack to simulate mouse clicks in the widget.
-    QObject::connect(&dndWidget, &DnDEventRecorder::dragMoveReceived, this, [=]() {
+    auto connection = QObject::connect(&dndWidget, &DnDEventRecorder::dragMoveReceived, this,
+                                        [=]() {
         QTest::mouseRelease(window, Qt::LeftButton);
     }, Qt::QueuedConnection);
 
@@ -832,7 +847,7 @@ void tst_QWidget_window::tst_dnd_events()
     QCOMPARE(dndWidget._dndEvents, expectedDndEvents);
 
     dndWidget._dndEvents.clear();
-    dndWidget.disconnect();
+    dndWidget.disconnect(connection);
     int step = 0;
     QObject::connect(&dndWidget, &DnDEventRecorder::dragMoveReceived, this, [window, &step]() {
         switch (step++) {
@@ -1191,7 +1206,7 @@ void tst_QWidget_window::tst_showhide_count()
             return QObject::eventFilter(receiver, event);
         }
     private:
-        using Entry = QPair<QWidget*, QEvent::Type>;
+        using Entry = std::pair<QWidget*, QEvent::Type>;
         QHash<Entry, int> counter;
     };
 
@@ -1358,10 +1373,10 @@ void tst_QWidget_window::setWindowState_data()
     QString platformName = QGuiApplication::platformName().toLower();
 
     QTest::addColumn<Qt::WindowStates>("state");
-    QTest::newRow("0") << Qt::WindowStates();
-    QTest::newRow("Qt::WindowMaximized") << Qt::WindowStates(Qt::WindowMaximized);
-    QTest::newRow("Qt::WindowMinimized") << Qt::WindowStates(Qt::WindowMinimized);
-    QTest::newRow("Qt::WindowFullScreen") << Qt::WindowStates(Qt::WindowFullScreen);
+    QTest::newRow("nostate") << Qt::WindowStates();
+    QTest::newRow("maximized") << Qt::WindowStates(Qt::WindowMaximized);
+    QTest::newRow("minimized") << Qt::WindowStates(Qt::WindowMinimized);
+    QTest::newRow("fullscreen") << Qt::WindowStates(Qt::WindowFullScreen);
 
     if (platformName != "xcb" && platformName != "windows" && !platformName.startsWith("wayland")
         && platformName != "offscreen")
@@ -1630,12 +1645,159 @@ void tst_QWidget_window::mouseMoveWithPopup()
 
     // but the release event will still be delivered to the first popup - dialogs might not get it
     QCOMPARE(mouseAction(Qt::LeftButton), QEvent::MouseButtonRelease);
-    if (topLevel.popup->mouseReleaseCount != 1
-        && !QGuiApplication::platformName().startsWith(QLatin1String("windows"), Qt::CaseInsensitive))
+    if (topLevel.popup->mouseReleaseCount != 1)
         QEXPECT_FAIL("Dialog", "Platform specific behavior", Continue);
     QCOMPARE(topLevel.popup->mouseReleaseCount, 1);
 }
 
+struct ShowHideEntry {
+    QEvent::Type action;
+    Qt::WindowType target;
+    using List = QList<ShowHideEntry>;
+};
+
+void tst_QWidget_window::showHideWindowHandle_data()
+{
+    QTest::addColumn<ShowHideEntry::List>("entries");
+
+    QTest::addRow("show/hide widget") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Widget }, { QEvent::Hide, Qt::Widget }
+    };
+    QTest::addRow("show/hide window") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Window }, { QEvent::Hide, Qt::Window }
+    };
+    QTest::addRow("show widget, hide window") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Widget }, { QEvent::Hide, Qt::Window }
+    };
+    QTest::addRow("show window, hide widget") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Window }, { QEvent::Hide, Qt::Widget }
+    };
+    QTest::addRow("show/hide widget, then show window, hide widget") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Widget }, { QEvent::Hide, Qt::Widget },
+        { QEvent::Show, Qt::Window }, { QEvent::Hide, Qt::Widget }
+    };
+    QTest::addRow("show widget, close widget, show widget") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Widget }, { QEvent::Close, Qt::Widget }, { QEvent::Show, Qt::Widget }
+    };
+    QTest::addRow("show widget, close widget, show window") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Widget }, { QEvent::Close, Qt::Widget }, { QEvent::Show, Qt::Window }
+    };
+    QTest::addRow("show widget, close window, show widget") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Widget }, { QEvent::Close, Qt::Window }, { QEvent::Show, Qt::Widget }
+    };
+    QTest::addRow("show widget, close window, show window") << ShowHideEntry::List{
+        { QEvent::Show, Qt::Widget }, { QEvent::Close, Qt::Window }, { QEvent::Show, Qt::Window }
+    };
+}
+
+void tst_QWidget_window::showHideWindowHandle()
+{
+    QWidget parent;
+    parent.setObjectName("Parent");
+    QCOMPARE(parent.isVisible(), false);
+    QCOMPARE(parent.testAttribute(Qt::WA_WState_ExplicitShowHide), false);
+    QCOMPARE(parent.testAttribute(Qt::WA_WState_Visible), false);
+    QCOMPARE(parent.testAttribute(Qt::WA_WState_Hidden), true);
+
+    QWidget child;
+    child.setObjectName("Child");
+    QCOMPARE(child.isVisible(), false);
+    QCOMPARE(child.testAttribute(Qt::WA_WState_ExplicitShowHide), false);
+    QCOMPARE(child.testAttribute(Qt::WA_WState_Visible), false);
+    QCOMPARE(child.testAttribute(Qt::WA_WState_Hidden), true);
+
+    child.setParent(&parent);
+    QCOMPARE(child.isVisible(), false);
+    QCOMPARE(child.testAttribute(Qt::WA_WState_ExplicitShowHide), false);
+    QCOMPARE(child.testAttribute(Qt::WA_WState_Visible), false);
+    QCOMPARE(child.testAttribute(Qt::WA_WState_Hidden), false);
+
+    QFETCH(QList<ShowHideEntry>, entries);
+    for (const auto entry : entries) {
+
+        if (entry.action == QEvent::Show) {
+            if (entry.target == Qt::Window && !parent.windowHandle()) {
+                parent.setAttribute(Qt::WA_NativeWindow);
+                QVERIFY(parent.windowHandle());
+
+                QCOMPARE(parent.isVisible(), false);
+                QCOMPARE(parent.testAttribute(Qt::WA_WState_ExplicitShowHide), false);
+                QCOMPARE(parent.testAttribute(Qt::WA_WState_Visible), false);
+                QCOMPARE(parent.testAttribute(Qt::WA_WState_Hidden), true);
+            }
+
+            bool wasExplicitShowHide = parent.testAttribute(Qt::WA_WState_ExplicitShowHide);
+
+            if (entry.target == Qt::Widget)
+                parent.show();
+            else
+                parent.windowHandle()->show();
+
+            QVERIFY(QTest::qWaitForWindowActive(&parent));
+
+            QCOMPARE(parent.isVisible(), true);
+            QVERIFY(parent.windowHandle());
+            QCOMPARE(parent.windowHandle()->isVisible(), true);
+
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_Visible), true);
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_Hidden), false);
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_ExplicitShowHide),
+                entry.target == Qt::Widget || wasExplicitShowHide);
+
+            QCOMPARE(child.isVisible(), true);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_ExplicitShowHide), false);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_Visible), true);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_Hidden), false);
+
+        } else if (entry.action == QEvent::Hide) {
+
+            bool wasExplicitShowHide = parent.testAttribute(Qt::WA_WState_ExplicitShowHide);
+
+            if (entry.target == Qt::Widget)
+                parent.hide();
+            else
+                parent.windowHandle()->hide();
+
+            QCOMPARE(parent.isVisible(), false);
+            QVERIFY(parent.windowHandle());
+            QCOMPARE(parent.windowHandle()->isVisible(), false);
+
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_Visible), false);
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_Hidden), true);
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_ExplicitShowHide),
+                entry.target == Qt::Widget || wasExplicitShowHide);
+
+            QCOMPARE(child.isVisible(), false);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_ExplicitShowHide), false);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_Visible), false);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_Hidden), false);
+
+        } else if (entry.action == QEvent::Close) {
+
+            bool wasExplicitShowHide = parent.testAttribute(Qt::WA_WState_ExplicitShowHide);
+
+            if (entry.target == Qt::Widget)
+                parent.close();
+            else
+                parent.windowHandle()->close();
+
+            QCOMPARE(parent.isVisible(), false);
+            QVERIFY(parent.windowHandle());
+            QCOMPARE(parent.windowHandle()->isVisible(), false);
+
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_Visible), false);
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_Hidden), true);
+            QCOMPARE(parent.testAttribute(Qt::WA_WState_ExplicitShowHide),
+                entry.target == Qt::Widget || wasExplicitShowHide);
+
+            QCOMPARE(child.isVisible(), false);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_ExplicitShowHide), false);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_Visible), false);
+            QCOMPARE(child.testAttribute(Qt::WA_WState_Hidden), false);
+        }
+    }
+}
+
 void tst_QWidget_window::resetFocusObjectOnDestruction()
 {
     if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation))
@@ -1672,5 +1834,30 @@ void tst_QWidget_window::resetFocusObjectOnDestruction()
     QCOMPARE(focusObjectChangedSpy.last().last().value<QObject*>(), nullptr);
 }
 
+class CreateDestroyWidget : public QWidget
+{
+public:
+    using QWidget::create;
+    using QWidget::destroy;
+};
+
+void tst_QWidget_window::cleanupOnDestruction()
+{
+    CreateDestroyWidget widget;
+    QWidget child(&widget);
+
+    QWidget grandChild(&child);
+    // Ensure there's not a 1:1 native window hierarhcy that we could
+    // recurse during QWidget::destroy(), triggering the issue that
+    // we were failing to clean up when not destroyed via QWidget.
+    grandChild.setAttribute(Qt::WA_DontCreateNativeAncestors);
+    grandChild.winId();
+
+    widget.destroy();
+    widget.create();
+
+    widget.show();
+}
+
 QTEST_MAIN(tst_QWidget_window)
 #include "tst_qwidget_window.moc"
