@@ -73,6 +73,7 @@ private slots:
     void overrideMenuAction();
     void statusTip();
     void widgetActionFocus();
+    void widgetActionFocusReason();
     void mouseActivation();
     void tearOff();
     void submenuTearOffDontClose();
@@ -662,6 +663,63 @@ void tst_QMenu::widgetActionFocus()
     QCOMPARE(m.activeAction(), (QAction *)wa);
 }
 
+void tst_QMenu::widgetActionFocusReason()
+{
+    class FocusReasonWidget : public QWidget
+    {
+    public:
+        explicit FocusReasonWidget(QWidget *parent = nullptr)
+            : QWidget(parent)
+        {
+            setFocusPolicy(Qt::StrongFocus);
+            // Give it some size so it doesn't skip over it.
+            setFixedSize(QSize(10, 10));
+        }
+        Qt::FocusReason focusInEventReason = Qt::NoFocusReason;
+
+    protected:
+        void focusInEvent(QFocusEvent *event) override
+        {
+            QWidget::focusInEvent(event);
+            focusInEventReason = event->reason();
+        }
+    };
+
+    // test if the backtab/tab focus reason is correctly handled when going down and up.
+    QWidget widget;
+    widget.show();
+    QVERIFY(QTest::qWaitForWindowActive(&widget));
+    QMenu *menu = new QMenu(&widget);
+
+    auto *w = new FocusReasonWidget(menu);
+
+    QWidgetAction *wa = new QWidgetAction(menu);
+    wa->setDefaultWidget(w);
+
+    menu->setActiveAction(menu->addAction("Top"));
+    menu->addAction(wa);
+    menu->addAction("Bottom");
+
+    menu->popup(QPoint());
+
+    QVERIFY(menu->isVisible());
+    QVERIFY(!w->hasFocus());
+
+    QTest::keyClick(QApplication::focusWidget(), Qt::Key_Down);
+    QVERIFY(w->hasFocus());
+    QCOMPARE(w->focusInEventReason, Qt::TabFocusReason);
+
+    QTest::keyClick(QApplication::focusWidget(), Qt::Key_Down);
+    QVERIFY(!w->hasFocus());
+
+    QTest::keyClick(QApplication::focusWidget(), Qt::Key_Up);
+    QVERIFY(w->hasFocus());
+    QCOMPARE(w->focusInEventReason, Qt::BacktabFocusReason);
+
+    QTest::keyClick(QApplication::focusWidget(), Qt::Key_Up);
+    QVERIFY(!w->hasFocus());
+}
+
 static QMenu *getTornOffMenu()
 {
     for (QWidget *w : QApplication::allWidgets()) {
@@ -2265,7 +2323,6 @@ void tst_QMenu::dontSelectDisabledActionByShortcut()
     QMainWindow w;
     auto mb = w.menuBar();
     auto m = mb->addMenu("me&nu");
-    auto zero = m->addAction("placeholder");
     auto first = m->addAction("disabled &o");
     auto second = m->addAction(QStringLiteral("enabled &o"));
 
